@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: __init__.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: __init__.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -27,11 +27,12 @@
 This module initializes the pyFormex global variables and
 defines a few essential functions.
 """
-__version__ = "0.8.1"
+__version__ = "0.8.2"
 __revision__ = ""
 Version = 'pyFormex %s' % __version__
 Copyright = 'Copyright (C) 2004-2009 Benedict Verhegghe'
 Url = 'http://pyformex.org'
+Description = "pyFormex is a tool for generating, manipulating and transforming large geometrical models of 3D structures by sequences of mathematical transformations."
 
 # The GUI parts
 app_started = False
@@ -49,11 +50,13 @@ StartTime = datetime.datetime.now()
 options = None     # the options found on the command line
 print_help = None  # the function to print(the pyformex help text (pyformex -h))
 
-cfg = None         # the current configuration
+cfg = None         # the current session configuration
+prefcfg = None     # the preferenced configuration 
 refcfg = None      # the reference configuration 
-preffile = None    # the file where current configuration will be saved
+preffile = None    # the file where the preferenced configuration will be saved
 
-PF = {}            # globals that will be offered to scripts
+PF = {}            # explicitely exported globals
+_PF_ = {}          # globals that will be offered to scripts
     
 scriptName = None
 
@@ -64,10 +67,10 @@ def message(s):
 
 warning = message
 
-def debug(s,lead="DEBUG"):
+def debug(s,lead="DEBUG",level=-1):
     """Print a debug message"""
     try: # to make sure that debug() can be used before options are set
-        if options.debug:
+        if options.debug < 0 or (options.debug % level > 0):
             raise
         pass
     except:
@@ -79,4 +82,6 @@ def debugt(s):
     debug(s,time.time())
 
 
+
+
 ### End
