@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -49,7 +49,7 @@ What
 """
 
 import copy
-from mydict import Dict
+from mydict import Dict,returnNone
 
 
 def formatDict(d):
@@ -292,9 +292,28 @@ class Config(Dict):
                 return self._default_(key)
 
 
+    def __delitem__(self,key):
+        """Allows items to be delete with del self[section/key].
+
+        """
+        i = key.rfind('/')
+        if i == -1:
+            Dict.__delitem__(self,key)
+        else:
+            del self[key[:i]][key[i+1:]]
+
+
     def __str__(self):
-        """Format the Config in a way that can be read back."""
-        s = "# Config written by pyFormex    -*- PYTHON -*-\n\n"
+        """Format the Config in a way that can be read back.
+
+        This function is mostly used to format the data for writing it to
+        a configuration file. See the write() method.
+
+        The return value is a multiline string with Python statements that can
+        be read back through Python to recreate the Config data. Usually
+        this is done with the Config.read() method.
+        """
+        s = ''
         for k,v in self.iteritems():
             if not isinstance(v,Dict):
                 s += formatDict({k:v})
@@ -302,10 +321,42 @@ class Config(Dict):
             if isinstance(v,Dict):
                 s += "\n[%s]\n" % k
                 s += formatDict(v)
-        s += "\n# End of config\n"
         return s
 
 
+    def write(self,filename,header="# Config written by pyFormex    -*- PYTHON -*-\n\n",trailer="\n# End of config\n"):
+        """Write the config to the given file
+
+        The configuration data will be written to the file with the given name
+        in a text format that is both readable by humans and by the
+        Config.read() method.
+        
+        The header and trailer arguments are strings that will be added at
+        the start and end of the outputfile. Make sure they are valid
+        Python statements (or comments) and that they contain the needed
+        line separators, if you want to be able to read it back.
+        """
+        fil = file(filename,'w')
+        fil.write(header)
+        fil.write("%s" % self)
+        fil.write(trailer)
+        fil.close()
+        
+
+    def keys(self,descend=True):
+        """Return the keys in the config.
+
+        By default this descends one level of Dicts.
+        """
+        keys = Dict.keys(self)
+        if descend:
+            for k,v in self.iteritems():
+                if isinstance(v,Dict):
+                    keys += ['%s/%s' % (k,ki) for ki in v.keys()]
+                
+        return keys
+
+
 if __name__ == '__main__':
 
 
@@ -352,10 +403,22 @@ rng = range(_n)
     print("BUT!!!!")
     show("D['cc']")
 
-    # THis should give an error
+    # This should give an error
     show("D['ee']")
     show("D.get('ee','NO Error')")
     show("D.get('cc/ee','NO Error')")
+
+    D['cc/bb'] = 'ok'
+    show("D.keys()")
+    del D['aa']
+    del D['cc/aa']
+    show("D.keys()")
+    del D['cc']
+    show("D.keys()")
+
+
+# End
+   
     
     
     
