@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: coords.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: coords.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -36,9 +36,9 @@ the :class:`Coords` class.
 While the user will mostly use the higher level classes, he might occasionally
 find good reason to use the :class:`Coords` class directly as well.
 """
-
 from arraytools import *
 from lib import misc
+from pyformex import options
 
 
 def bbox(objects):
@@ -48,7 +48,7 @@ def bbox(objects):
     The result is the eclosing bbox of all the objects in the list.
     Objects returning a None bbox are ignored.
     """
-    bboxes = [f.bbox() for f in objects]
+    bboxes = [f.bbox() for f in objects if hasattr(f,'bbox') and not isnan(f.bbox()).any()]
     bboxes = [bb for bb in bboxes if bb is not None]
     if len(bboxes) == 0:
         o = origin()
@@ -126,8 +126,13 @@ class Coords(ndarray):
         """Create a new instance of :class:`Coords`.
 
         """
+        # An empty coords object
         if data is None:
-            data = zeros((3,),dtype=Float)
+            data = ndarray((0,3),dtype=Float)
+
+        #
+        # !! Perhaps we should force at least dimension 2 ???
+        #
 
         # Turn the data into an array, and copy if requested
         ar = array(data, dtype=dtyp, copy=copy)
@@ -135,6 +140,8 @@ class Coords(ndarray):
             pass
         elif ar.shape[-1] in [1,2]:
             ar = concatenate([ar,zeros(ar.shape[:-1]+(3-ar.shape[-1],))],axis=-1)
+        elif ar.shape[-1] == 0: # allow empty coords objects 
+            ar = ar.reshape(0,3)
         else:
             raise ValueError,"Expected a length 1,2 or 3 for last array axis"
 
@@ -178,6 +185,8 @@ class Coords(ndarray):
         """Return the total number of points."""
         return asarray(self.shape[:-1]).prod()
 
+    ncoords = npoints
+
     def x(self):
         """Return the x-plane"""
         return self[...,0]
@@ -202,9 +211,11 @@ class Coords(ndarray):
         """
         if self.size > 0:
             s = self.points()
-            return Coords(row_stack([ s.min(axis=0), s.max(axis=0) ]))
+            bbox = row_stack([ s.min(axis=0), s.max(axis=0) ])
         else:
-            return None
+            o = origin()
+            bbox = [o,o]
+        return Coords(bbox)
 
 
     def center(self):
@@ -257,7 +268,7 @@ class Coords(ndarray):
     #  Distance
 
     def distanceFromPlane(self,p,n):
-        """Return the distance of points f from the plane (p,n).
+        """Return the distance of all points from the plane (p,n).
 
         p is a point specified by 3 coordinates.
         n is the normal vector to a plane, specified by 3 components.
@@ -269,14 +280,15 @@ class Coords(ndarray):
         """
         p = asarray(p).reshape((3))
         n = asarray(n).reshape((3))
-        n /= length(n)
+        n = normalize(n)
         d = inner(self,n) - inner(p,n)
         return d
 
 
     def distanceFromLine(self,p,n):
-        """Return the distance of points f from the line (p,n).
+        """Return the distance of all points from the line (p,n).
 
+        p,n are (1,3) or (npts,3) arrays defining 1 or npts lines
         p is a point on the line specified by 3 coordinates.
         n is a vector specifying the direction of the line through p.
 
@@ -284,15 +296,15 @@ class Coords(ndarray):
         each point to the line through p with direction n.
         All distance values are positive or zero.
         """
-        p = asarray(p).reshape((3))
-        n = asarray(n).reshape((3))
+        p = asarray(p)#.reshape((3))
+        n = asarray(n)#.reshape((3))
         t = cross(n,p-self)
         d = sqrt(sum(t*t,-1)) / length(n)
         return d
 
 
     def distanceFromPoint(self,p):
-        """Return the distance of points f from the point p.
+        """Return the distance of all points from the point p.
 
         p is a single point specified by 3 coordinates.
 
@@ -305,6 +317,67 @@ class Coords(ndarray):
         return sqrt(sum(d*d,-1))
 
 
+    def directionalSize(self,n,p=None):
+        """Return the extreme distances from the plane p,n.
+
+        The direction n can be specified by a 3 component vector or by
+        a single integer 0..2 designing one of the coordinate axes.
+
+        p is any point in space. If not specified, it is taken as the
+        center() of the Coords.
+
+        The return value is a tuple of two float values specifying the
+        extreme distances from the plane p,n.
+        """
+        if type(n) is int:
+            n = unitVector(n)
+        n = normalize(Coords(n))
+
+        if p is None:
+            p = self.center()
+        p = Coords(p)
+        
+        d = self.distanceFromPlane(p,n)
+        return d.min(),d.max()
+
+
+    def directionalWidth(self,n):
+        """Return the width of a Coords in the given direction.
+
+        The direction can be specified by a 3 component vector or by
+        a single integer 0..2 designing one of the coordinate axes.
+
+        The return value is the thickness of the object in the direction n.
+        """
+        xmin,xmax = self.directionalSize(n)
+        return xmax-xmin
+
+
+    def directionalExtremes(self,n,p=None):
+        """Return extremal planes in the direction n.
+
+        The direction n can be specified by a 3 component vector or by
+        a single integer 0..2 designing one of the coordinate axes.
+
+        p is any point in space. If not specified, it is taken as the
+        center() of the Coords.
+
+        The return value is a list of two points on the line (p,n) thus
+        that the planes with normal through these points define the
+        extremal planes of the Coords.
+        """
+        if type(n) is int:
+            n = unitVector(n)
+        n = normalize(Coords(n))
+
+        if p is None:
+            p = self.center()
+        p = Coords(p)
+        
+        d = self.distanceFromPlane(p,n)
+        return [ p+dist*n for dist in [d.min(),d.max() ] ]
+
+
     # Test position
 
     def test(self,dir=0,min=None,max=None,atol=0.):
@@ -920,6 +993,22 @@ class Coords(ndarray):
         return roll(self, int(n) % 3,axis=-1)
 
 
+    def projectOnPlane(self,P,n):
+        """Project :class:`Coords` on the plane(s) (P,n)
+
+        P and n define a plane or a set of planes by a point P and the normal
+        n. If a set of planes, there should be exactly ncoords planes.
+        Each of P and n thus can have shape (ncoords,3) or (1,3) or (3,).
+
+        Return a Coords with same shape as original, with the base points
+        of the lines through all the points of self, perpendicular to the
+        plane(s) (P,n), i.e. the projection of the points on the plane(s).
+        """
+        n = normalize(Coords(n).reshape(-1,3))
+        s =  - dotpr(n,(self.reshape(-1,3)-P))
+        return self + outer(s,n).reshape(self.shape)
+
+
     def projectOnSphere(self,radius=1.,center=[0.,0.,0.]):
         """Project :class:`Coords` on a sphere.
 
@@ -952,8 +1041,6 @@ class Coords(ndarray):
             f[...,i] *= s
         f += c
         return f
-
-
     def split(self):
         """Split the coordinate array in blocks along first axis.
 
@@ -967,15 +1054,16 @@ class Coords(ndarray):
 
 ##############################################################################
 
-    def fuse(self,nodesperbox=1,shift=0.5,rtol=1.e-5,atol=1.e-5):
+
+    def fuse(self,nodesperbox=1,shift=0.5,rtol=1.e-5,atol=1.e-5,repeat=True):
         """Find (almost) identical nodes and return a compressed set.
 
         This method finds the points that are very close and replaces them
         with a single point. The return value is a tuple of two arrays:
-        
+
         - the unique points as a :class:`Coords` object,
         - an integer (nnod) array holding an index in the unique
-        
+
         coordinates array for each of the original nodes. This index will
         have the same shape as the pshape() of the coords array.
 
@@ -985,18 +1073,28 @@ class Coords(ndarray):
         is computed, that is then used to sort the nodes.
         Then only nodes inside the same box are compared on almost equal
         coordinates, using the numpy allclose() function. Two coordinates are
-        considered close if they are within a relative tolerance rtol or absolute
-        tolerance atol. See numpy for detail. The default atol is set larger than
-        in numpy, because pyformex typically runs with single precision.
+        considered close if they are within a relative tolerance rtol or
+        absolute tolerance atol. See numpy for detail. The default atol is
+        set larger than in numpy, because pyformex typically runs with single
+        precision.
         Close nodes are replaced by a single one.
 
-        Currently the procedure does not guarantee to find all close nodes:
+        Running the procedure once does not guarantee to find all close nodes:
         two close nodes might be in adjacent boxes. The performance hit for
         testing adjacent boxes is rather high, and the probability of separating
-        two close nodes with the computed box limits is very small. Nevertheless
-        we intend to access this problem by repeating the procedure with the
-        boxes shifted in space.
+        two close nodes with the computed box limits is very small.
+        Therefore, the most sensible way is to run the procedure twice, with
+        a different shift value (they should differ more than the tolerance).
+        Specifying repeat=True will automatically do this.
         """
+        if repeat:
+            # Aplly twice with different shift value
+            coords,index = self.fuse(nodesperbox,shift,rtol,atol,repeat=False)
+            coords,index2 = coords.fuse(nodesperbox,shift+0.25,rtol,atol,repeat=False)
+            index = index2[index]
+            return coords,index
+
+        # This is the single pass
         x = self.points()
         nnod = x.shape[0]
         # Calculate box size
@@ -1009,7 +1107,7 @@ class Coords(ndarray):
             x = x[:1]
             e = zeros(nnod,dtype=int32)
             return x,e
-            
+
         vol = esz.prod()
         nboxes = nnod / nodesperbox # ideal total number of boxes
         boxsz = (vol/nboxes) ** (1./esz.shape[0])
@@ -1037,40 +1135,43 @@ class Coords(ndarray):
         # than enough for all practical purposes
         x = x.astype(float32)
         val = val.astype(int32)
-        flag = ones((nnod,),dtype=int32)   # 1 = new, 0 = existing node
-        sel = arange(nnod).astype(int32)   # replacement unique node nr
         tol = float32(max(abs(rtol*self.sizes()).max(),atol))
-        if hasattr(misc,'fuse'):
-            # use the lib
-            misc.fuse(x,val,flag,sel,tol)
-        else:
-            # !!!! this code should be moved to the emulated lib !
-            for i in range(nnod):
-                j = i-1
-                while j>=0 and val[i]==val[j]:
-                    if allclose(x[i],x[j],rtol=rtol,atol=atol):
-                        # node i is same as node j
-                        flag[i] = 0
-                        sel[i] = sel[j]
-                        sel[i+1:nnod] -= 1
-                        break
-                    j = j-1
-                    
+        nnod = val.shape[0]
+        flag = ones((nnod,),dtype=int32)   # 1 = new, 0 = existing node
+        # new fusing algorithm
+        sel = arange(nnod).astype(int32)      # replacement unique node nr
+        misc._fuse2(x,val,flag,sel,tol)     # fuse the close points
         x = x[flag>0]          # extract unique nodes
         s = sel[argsort(srt)]  # and indices for old nodes
         return (x,s.reshape(self.shape[:-1]))
 
 
+    def append(self,coords):
+        """Append coords to a Coords object.
+
+        The appended coords should have matching dimensions in all
+        but the first axis.
+
+        Returns the concatenated Coords object, without changing the current.
+
+        This is comparable to the :numpy:`append` function, but the result
+        is a :class:`Coords` object, the default axis is the first one
+        instead of the last, and it is a method rather than a function.
+        """
+        return Coords(append(self,coords,axis=0))
+            
+
     @classmethod
-    def concatenate(clas,L,**kargs):
+    def concatenate(clas,L,axis=0):
         """Concatenate a list of :class:`Coords` object.
 
         All :class:`Coords` object in the list L should have the same shape
-        except for the length of the first axis.
+        except for the length of the specified axis.
         This function is equivalent to the numpy concatenate, but makes
-        sure the result is a :class:`Coords` object.
+        sure the result is a :class:`Coords` object,and the default axis
+        is the first one instead of the last.
         """
-        return Coords(concatenate(L,axis=0,**kargs))
+        return Coords(concatenate(atleast_2d(*L),axis=axis))
 
 
     @classmethod
@@ -1163,6 +1264,14 @@ class Coords(ndarray):
         return reflect(*args,**kargs)
 
 
+
+# Creating special coordinate sets
+
+def origin():
+    """Return a single point with coordinates [0.,0.,0.]."""
+    return Coords(zeros((3),dtype=Float))
+
+
 ##############################################################################
 #
 #  Testing
@@ -1225,61 +1334,35 @@ if __name__ == "__main__":
         prt("X_rot",X.rotate(90.))
         prt("X_rot2",X.rotate(90.,0))
         
-        Y=prt("X_ref",X.reflect(1,2))
-
-        print(X.bbox())
-        print(Y.bbox())
-        print(bbox([X,Y]))
-        return
-        X3 = X.copy().reflect(1,1.5).translate(1,2)
-        print("X =",X)
-        print("X3 =",X3)
-        G = Coords.concatenate([X1,X3,X2,X3])
-        print("X1+X3+X2+X3 =",G)
-        print("unique:",G.unique())
-
-        Y = Coords([[[1,0,0],[0,1,0],[0,0,1]],[[2,0,0],[0,2,0],[0,0,2]]])
-        print(Y)
-        Y.translate([0.,100.,0.])
-        print(Y)
-
-        Y = Coords([1.0,0.0,0.0])
-        print(Y)
-        Y.translate([0.,100.,0.])
-        print(Y)
+        Y=prt("Y = X_reflect",X.reflect(1,2))
+        prt("X_bbox",X.bbox())
+        prt("Y_bbox",Y.bbox())
+        prt("(X+Y)bbox",bbox([X,Y]))
+        Z = X.copy().reflect(1,1.5).translate(1,2)
+        prt("X",X)
+        prt("Y",Y)
+        prt("Z",Z)
+        G = Coords.concatenate([X,Z,Y,Z],axis=0)
+        prt("X+Z+Y+Z",G)
         return
     
 
     def test():
-        """Run some additional examples.
+        """Run the tests.
 
-        This is intended for tests during development. This can be changed
-        at will.
+        This is intended for tests during development and can be
+        changed at will.
         """
         testX(Coords([[1,0,0],[0,1,0]]))
         testX(Coords([[[0,0,0],[1,0,0]],[[0,1,0],[1,1,0]]]))
         testX(Coords([1,0,0]))
-        testX(Coords())
+        try:
+            testX(Coords())
+        except:
+            print "Some test(s) failed for an empty Coords"
+            print "But that surely is no surprise"
         return
 
-
-    def testweave():
-        
-        code = r"""
-float xm,ym,zm;
-xm = ym = zm = 0.;
-int i;
-for(i=0;i<Na[0];i++) {
-xm += A2(i,0)
-ym += A2(i,1)
-zm += A2(i,2)
-}
-xm /= Na[0];
-ym /= Na[0];
-zm /= Na[0];
-
-return val = xm;
-"""
         
         
     f = 0
