@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: elements.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: elements.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,11 +29,11 @@ pyFormex. When interfacing with other programs, one should be aware
 that conversions may be necessary. Conversions to/from external programs
 should be done by the interface modules.
 """
+from numpy import array
+from olist import collectOnLength
+from math import sqrt
 
-import math
-golden_ratio = 0.5 * (1.0 + math.sqrt(5))
-
-import olist
+golden_ratio = 0.5 * (1.0 + sqrt(5.))
 
 
 class Element(object):
@@ -43,10 +43,17 @@ class Element(object):
     an uppercase character and further only lower case and digits.
 
     Each element is defined by the following attributes:
-    vertices: the natural coordinates of its vertices,
-    edges: a list of edges, each defined by a couple of node numbers,
-    faces: a list of faces, each defined by a list of minimum 3 node numbers,
-    element: a list of all node numbers
+
+    - `vertices`: the natural coordinates of its vertices,
+    - `edges`: a list of edges, each defined by a couple of node numbers,
+    - `faces`: a list of faces, each defined by a list of minimum 3 node
+      numbers,
+    - `element`: a list of all node numbers
+    - `drawfaces`: a list of faces to be drawn, if different from faces. This
+      is an optional attribute. If defined, it will be used instead of the
+      `faces` attribute to draw the element. This can e.g. be used to draw
+      approximate representations for higher order elements for which there
+      is no correct drawing function.
 
     The vertices of the elements are defined in a unit space [0,1] in each
     axis direction. 
@@ -58,7 +65,8 @@ class Element(object):
     numbered in a consecutive order spinning positively around the outward
     normal on the face.
     """
-    
+
+    ndim = 0
     vertices = []
     edges = []
     faces = []
@@ -72,14 +80,43 @@ class Element(object):
     def nfaces(self):
         return len(self.faces)
 
-    def getFaces():
-        return olist.collectOnLength(self.faces)
+    def getFaces(self):
+        return collectOnLength(self.faces)
 
 
 
+class Point(Element):
+    """A single node element"""
+    ndim = 0
+    
+    vertices = [ ( 0.0, 0.0, 0.0 ) ]
+
+    edges = [ ]
+
+    faces = [ ]
+
+    element = vertices[0]
+    
+
+class Line2(Element):
+    """A 2-node line segment"""
+    ndim = 1
+    
+    vertices = [ ( 0.0, 0.0, 0.0 ),
+                 ( 1.0, 0.0, 0.0 ),
+                 ]
+
+    edges = [ (0,1) ]
+
+    faces = [ ]
+
+    element = edges[0]
+
 
 class Tri3(Element):
     """A 3-node triangle"""
+    ndim = 2
+    
     vertices = [ ( 0.0, 0.0, 0.0 ),
                  ( 1.0, 0.0, 0.0 ),
                  ( 0.0, 1.0, 0.0 ),
@@ -92,8 +129,31 @@ class Tri3(Element):
     element = faces[0]
 
 
+class Tri6(Element):
+    """A 6-node triangle"""
+    ndim = 2
+    
+    vertices = [ ( 0.0, 0.0, 0.0 ),
+                 ( 1.0, 0.0, 0.0 ),
+                 ( 0.0, 1.0, 0.0 ),
+                 ( 0.5, 0.5, 0.0 ),
+                 ( 0.0, 0.5, 0.0 ),
+                 ( 0.5, 0.0, 0.0 ),
+                 ]
+
+    edges = [ (0,1), (1,2), (2,0) ]
+
+    faces = [ (0,1,2), ]
+
+    element = faces[0]
+
+    drawfaces = Tri3.faces
+
+
 class Quad4(Element):
     """A 4-node quadrilateral"""
+    ndim = 2
+    
     vertices = [ (  0.0,  0.0, 0.0 ),
                  (  1.0,  0.0, 0.0 ),
                  (  1.0,  1.0, 0.0 ),
@@ -111,28 +171,44 @@ class Quad4(Element):
         }
 
 
-class Quad9(Element):
-    """A 4-node quadrilateral"""
-    vertices = [ (  0.0,  0.0, 0.0 ),
-                 (  1.0,  0.0, 0.0 ),
-                 (  1.0,  1.0, 0.0 ),
-                 (  0.0,  1.0, 0.0 ),
-                 (  0.5,  0.0, 0.0 ),
-                 (  1.0,  0.5, 0.0 ),
-                 (  0.5,  1.0, 0.0 ),
-                 (  0.0,  0.5, 0.0 ),
-                 (  0.5,  0.5, 0.0 ),
-                 ]
+class Quad8(Element):
+    """A 8-node quadrilateral"""
+    ndim = 2
+    
+    vertices = Quad4.vertices + [ (  0.5,  0.0, 0.0 ),
+                                  (  1.0,  0.5, 0.0 ),
+                                  (  0.5,  1.0, 0.0 ),
+                                  (  0.0,  0.5, 0.0 ),
+                                  ]
 
     edges = [ (0,1), (1,2), (2,3), (3,0) ]
 
+    faces = [ (0,1,2,3,4,5,6,7), ]
+
+    element = faces[0]
+
+    drawfaces = Quad4.faces
+
+
+class Quad9(Element):
+    """A 9-node quadrilateral"""
+    ndim = 2
+    
+    vertices = Quad8.vertices + [ (  0.5,  0.5, 0.0 ), ]
+
+    edges = Quad8.edges
+
     faces = [ (0,1,2,3,4,5,6,7,8), ]
 
     element = faces[0]
 
+    drawfaces = Quad4.faces
+
 
 class Tet4(Element):
     """A 4-node tetrahedron"""
+    ndim = 3
+    
     vertices = [ ( 0.0, 0.0, 0.0 ),
                  ( 1.0, 0.0, 0.0 ),
                  ( 0.0, 1.0, 0.0 ),
@@ -148,6 +224,8 @@ class Tet4(Element):
 
 class Wedge6(Element):
     """A 6-node wedge element"""
+    ndim = 3
+    
     vertices = [ ( 0.0, 0.0, 1.0 ),
                  ( 1.0, 0.0, 1.0 ),
                  ( 0.0, 1.0, 1.0 ),
@@ -165,7 +243,8 @@ class Wedge6(Element):
 
 class Hex8(Element):
     """An 8-node hexahedron"""
-
+    ndim = 3
+    
     vertices = [ ( 0.0, 0.0, 0.0 ),  
                  ( 1.0, 0.0, 0.0 ),
                  ( 1.0, 1.0, 0.0 ),
@@ -192,6 +271,8 @@ class Icosa(Element):
 
     nfaces = 20, nedges = 30, nvertices = 12
     """
+    ndim = 3
+    
     phi = golden_ratio
     
     vertices = [ ( 0.0, 1.0, phi ),
