@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
 # $Id: Circle.py 154 2006-11-03 19:08:25Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -32,7 +32,7 @@ techniques = ['dialog', 'animation']
 """
 
 from simple import circle
-from plugins.geometry import rotationAngle
+from plugins.geomtools import rotationAngle
 
 # Test
 linewidth(1)
@@ -60,15 +60,15 @@ draw(curve)
 
 # Create circles in the middle of each segment, with normal along the segment
 # begin and end points of segment
-A = curve.f[:,0,:]
-B = curve.f[:,1,:]
+A = curve.coords[:,0,:]
+B = curve.coords[:,1,:]
 # midpoint and segment director
 C = 0.5*(B+A)
 D = B-A
 # vector initially normal to circle defined above
 nuc = array([0.,0.,1.])
 # rotation angles and vectors
-rot,ang = rotationAngle(nuc,D)
+ang,rot = rotationAngle(nuc,D)
 # diameters varying linearly with the |x| coordinate
 diam = 0.1*h*(2.-abs(C[:,0]))
 # finally, here are the circles:
