@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
 # $Id: Clip.py 154 2006-11-03 19:08:25Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -89,7 +89,7 @@ txt = [ 'First node has x between 1.5 and 3.5',
         '3 nodes close to line through [0.0,0.0,0.0] and [1.0,1.0,1.0]',
         ]
 
-color = getcfg('canvas/propcolors')[1:] # omit the black
+color = getcfg('canvas/colormap')[1:] # omit the black
 while len(color) < len(sel):
     color.extend(color)
 color[0:0] = ['black'] # restore the black
