@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: ColorScale.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: ColorScale.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -105,9 +105,8 @@ def getData():
     linewidth = res['Line width']
     lefttext = res['Text left of colorscale']
     font = res['Font']
-    print res['Position']
-    x,y = 200,50
-    w,h = 50,400
+    x,y = res['Position']
+    w,h = res['Size']
     # ok, now draw it
     drawColorScale(palet,minval,maxval,medval,maxexp,minexp,ncolors,dec,scale,grid,linewidth,lefttext,font,x,y,w,h)     
 
