@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: Curves.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Curves.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -35,6 +35,7 @@ from plugins.curve import *
 from odict import ODict
 from gui import widgets
 
+
 def BezierCurve(X,curl=None,closed=False):
     """Create a Bezier curve give 4 points
 
@@ -42,7 +43,6 @@ def BezierCurve(X,curl=None,closed=False):
     """
     ns = (X.shape[0]-1) / 3
     ip = 3*arange(ns+1)
-    print ip
     P = X[ip]
     ip = 3*arange(ns)
     ic = column_stack([ip+1,ip+2]).ravel()
@@ -55,12 +55,13 @@ method = ODict([
     ('Natural Spline', NaturalSpline),
     ('Bezier Spline', BezierSpline),
     ('Cardinal Bezier Spline', CardinalSpline),
+    ('Quadratic Bezier Spline', QuadBezierSpline),
     ('Cardinal Spline', CardinalSpline2),
     ('Polyline', PolyLine),
     ('Bezier Curve', BezierCurve),
 ])
 
-method_color = [ 'red','green','blue','cyan','magenta','yellow' ] 
+method_color = [ 'red','green','blue','cyan','magenta','yellow','white' ] 
 point_color = [ 'black','white' ] 
         
 open_or_closed = { True:'A closed', False:'An open' }
@@ -69,7 +70,7 @@ TA = None
 
 
 
-def drawCurve(ctype,dset,closed,endcond,tension,curl,interpoints,ndiv,extend,spread):
+def drawCurve(ctype,dset,closed,endcond,tension,curl,ndiv,ntot,extend,spread,drawtype,cutWP=False):
     global TA
     P = dataset[dset]
     text = "%s %s with %s points" % (open_or_closed[closed],ctype.lower(),len(P))
@@ -87,33 +88,27 @@ def drawCurve(ctype,dset,closed,endcond,tension,curl,interpoints,ndiv,extend,spr
         curl = float(curl)
         kargs['curl'] = curl
     S = method[ctype](P,**kargs)
-    if interpoints == 'subPoints':
-        X = S.subPoints(ndiv,extend)
-        point_color = 'black'
-        msize = 3
-    elif interpoints == 'pointsAt':
-        npts = ndiv*S.nparts + (extend[1]-extend[0]) * ndiv + 1
-        X = S.pointsAt(extend[0] + arange(npts)/ndiv)
-        point_color = 'white'
-        msize = 4
-    else: #if interpoints == 'random':
-        at = sort(random.rand(ndiv*S.nparts))*S.nparts
-        print at
-        X = S.pointsAt(at)
-        point_color = 'blue'
-        msize = 5
-    
-    PL = PolyLine(X,closed=closed)
+
+    print S.pointsOn()
+    print S.pointsOff()
 
     if spread:
-        at = PL.atLength(PL.nparts)
-        X = PL.pointsAt(at)
-        PL = PolyLine(X,closed=closed)
+        print ndiv,ntot
+        PL = S.approx(ndiv=ndiv,ntot=ntot)
+    else:
+        print ndiv,ntot
+        PL = S.approx(ndiv=ndiv)
         
-    draw(X, color=point_color,marksize=msize)
     im = method.keys().index(ctype)
-    draw(PL, color=method_color[im])
+    if cutWP:
+        PC = PL.cutWithPlane([0.,0.42,0.],[0.,1.,0.])
+        draw(PC[0],color=red)
+        draw(PC[1],color=green)
+    else:
+        draw(PL, color=method_color[im])
+    draw(PL.pointsOn(),color=black,marksize=4)
 
+    
 
 dataset = [
     Coords([[1., 0., 0.],[0., 1., 0.],[-1., 0., 0.],  [0., -1., 0.]]),
@@ -127,7 +122,7 @@ dataset = [
             [2., -1.5, -2.], [1.5, -1.5, 2.], [0., -8., 0.], [-1., -8., -1.],
             [3., -3., 1.]]),
     Coords([[0., 1., 0.],[0., 0.1, 0.],[0.1, 0., 0.],  [1., 0., 0.]]),
-    #Coords([[0., 1., 0.],[0.,0.,0.],[1., 0., 0.]]),
+    Coords([[0., 1., 0.],[0.,0.,0.],[0.,0.,0.],[1., 0., 0.]]),
     #Coords([[0., 1., 0.],[1., 0., 0.]]),
     ]
 
@@ -138,13 +133,15 @@ data_items = [
     ['EndCondition',None,'select',['notaknot','secder']],
     ['Tension',0.0],
     ['Curl',0.5],
-    ['InterPoints',None,'select',['subPoints','pointsAt','random']],
-    ['Nintervals',10],
-    ['SpreadEqually',False],
+    ['Ndiv',10],
+    ['SpreadEvenly',False],
+    ['Ntot',40],
     ['ExtendAtStart',0.0],
     ['ExtendAtEnd',0.0],
 #    ['FreeSpaced',[-0.1,0.0,0.1,0.25,1.5,2.75]],
+    ['DrawAs',None,'hradio',{'choices':['Curve','Polyline']}],
     ['Clear',True],
+    ['CutWithPlane',False],
     ]
 globals().update([i[:2] for i in data_items])
 if GD.PF.has_key('_Curves_data_'):
@@ -185,7 +182,7 @@ def show(all=False):
         Types = [CurveType]
     setDrawOptions({'bbox':'auto'})
     for Type in Types:
-        drawCurve(Type,int(DataSet),Closed,EndCondition,Tension,Curl,InterPoints,Nintervals,[ExtendAtStart,ExtendAtEnd],SpreadEqually)
+        drawCurve(Type,int(DataSet),Closed,EndCondition,Tension,Curl,Ndiv,Ntot,[ExtendAtStart,ExtendAtEnd],SpreadEvenly,DrawAs,CutWithPlane)
         setDrawOptions({'bbox':None})
 
 def showAll():
@@ -195,7 +192,11 @@ def timeOut():
     showAll()
     close()
     
-dialog = widgets.InputDialog(data_items,caption='Curve parameters',actions = [('Close',close),('Show All',showAll),('Show',show)],default='Show')
+dialog = widgets.InputDialog(
+    data_items,
+    caption='Curve parameters',
+    actions = [('Close',close),('Show All',showAll),('Show',show)],
+    default='Show')
 dialog.timeout = timeOut
 dialog.show()
        
