@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: Extrude.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Extrude.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -26,12 +26,14 @@
 """Extrude
 
 level = 'beginner'
-topics = ['geometry']
-techniques = ['connect']
+topics = ['formex']
+techniques = ['extrude']
 
 """
-
 clear()
+smoothwire()
+view('iso')
+
 a = Formex([0.,0.,0.])
 draw(a,color='black')
 sleep(1)
@@ -44,5 +46,5 @@ c = b.extrude(4,1.,0)
 draw(c,color='blue')
 sleep(1)
 
-d = c.extrude(7,1.,2)
+d = c.extrude(7,-1.,2)
 draw(d,color='green')
