@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: FeAbq.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: FeAbq.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -34,6 +34,7 @@ from plugins.fe import *
 from plugins.properties import *
 from plugins.fe_abq import *
 
+
 def quad():
     """Return a unit quadrilateral Formex."""
     return Formex(mpattern('123'))
@@ -53,13 +54,13 @@ pbc,pld = 1,6 # Properties corresponding to boundary/loaded nodes
 A = triquad().replic2(na,ma,1,1).setProp(pa)
 B = quad().replic2(nb,mb,1,1).translate([na,0,0]).setProp(pb)
 # Change every second element of B to property pb1
-B.p[arange(B.p.size) % 2 == 1] = pb1
+B.prop[arange(B.prop.size) % 2 == 1] = pb1
 C = A.rotate(90).setProp(pc)
 parts = [A,B,C]
 
 # Create the finite element model
 # A model contains a single set of nodes and one or more sets of elements
-M = mergedModel(*[p.feModel() for p in parts])
+M = mergedModel([p.toMesh() for p in parts])
 
 # Create a Formex with the nodes, mostly for drawing
 F = Formex(M.coords)
@@ -80,7 +81,7 @@ def drawModel(M,nodes=True,elems=True,nodenrs=True,elemnrs=True):
     lights(False)
     transparent()
     clear()
-    print F.p
+    print F.prop
     if nodes or nodenrs:
 ##         F = Formex(M.coords)
         if nodes:
@@ -99,7 +100,7 @@ def drawModel(M,nodes=True,elems=True,nodenrs=True,elemnrs=True):
 drawModel(M)
 
 # Transfer the properties from the parts in a global set
-elemprops = concatenate([part.p for part in parts])
+elemprops = concatenate([part.prop for part in parts])
 
 # Create the property database
 P = PropertyDB()
@@ -161,8 +162,8 @@ P.nodeProp(tag='step1',set=lnodes,name='Loaded',cload=[-10.,0.,0.,0.,0.,0.])
 P.nodeProp(tag='step2',set='Loaded',cload=[-10.,10.,0.,0.,0.,0.])
 
 F.setProp(0)
-F.p[bnodes] = pbc
-F.p[lnodes] = pld
+F.prop[bnodes] = pbc
+F.prop[lnodes] = pld
 
 print "Node properties"
 for p in P.getProp('n'):
