@@ -1,7 +1,7 @@
 #!/usr/bin/env python pyformex.py
-# $Id: FeEx.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: FeEx.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,6 +28,8 @@ level = 'advanced'
 topics = ['FEA']
 techniques = ['menu', 'dialog', 'persistence', 'colors'] 
 """
+from pyformex import GUI
+from gui import menu
 
 from simple import rectangle
 from plugins.fe import *
@@ -35,13 +37,14 @@ from plugins.properties import *
 from plugins.fe_abq import *
 from plugins.fe_post import FeResult
 from plugins import postproc_menu
+from plugins import mesh_menu
+from plugins import isopar
 from odict import ODict
 import utils
 
 # global data
 
 parts = None
-femodels = None
 model = None
 PDB = None
 
@@ -65,17 +68,19 @@ feresult_name = utils.NameSequence(name)
 
 
 def resetData():
-    global parts,femodels,model,PDB
+    global parts,model,PDB
     parts = []
-    femodels = []
     model = None
     PDB = None
+    mesh_menu.selection.set([])
+    
     
 def reset():
     clear()
     smoothwire()
     transparent()
-    lights(False)
+    #lights(False)
+    mesh_menu.selection.draw()
 
 def deleteAll():
     resetData()
@@ -84,60 +89,108 @@ def deleteAll():
 ######################## parts ####################
     
 x0,y0 = 0.,0.
-x1,y1 = 1.,1.
+x1,y1 = 1.,0.
+x2,y2 = 1.,1.
+x3,y3 = 0.,1.
 nx,ny = 4,4
 eltype = 'quad'
 
-def createPart(res=None):
+def createRectPart(res=None):
     """Create a rectangular domain from user input"""
-    global x0,y0,x1,y1
+    global x0,y0,x2,y2,nx,ny,eltype
     if model is not None:
         if ask('You have already merged the parts! I can not add new parts anymore.\nYou should first delete everything and recreate the parts.',['Delete','Cancel']) == 'Delete':
             deleteAll()
         else:
             return
     if res is None:
-        res = askItems([('x0',x0),('y0',y0),
-                        ('x1',x1),('y1',y1),
+        res = askItems([('x0',x0,{'tooltip':'The x-value of one of the corners'}),
+                        ('y0',y0),
+                        ('x2',x2),('y2',y2),
                         ('nx',nx),('ny',ny),
                         ('eltype',eltype,'select',['quad','tri-u','tri-d']),
                         ])
     if res:
         globals().update(res)
-        if x0 > x1:
-            x0,x1 = x1,x0
-        if y0 > y1:
-            y0,y1 = y1,y0
+        if x0 > x2:
+            x0,x2 = x2,x0
+        if y0 > y2:
+            y0,y2 = y2,y0
         diag = {'quad':'', 'tri-u':'u', 'tri-d':'d'}[eltype]
-        F = rectangle(nx,ny,x1-x0,y1-y0,diag=diag).trl([x0,y0,0])
-        addPart(F)
-        drawParts()
+        M = rectangle(nx,ny,x2-x0,y2-y0,diag=diag).toMesh().trl([x0,y0,0])
+        addPart(M)
 
 
-def addPart(F):
-    """Add a Formex to the parts list."""
+def createQuadPart(res=None):
+    """Create a quadrilateral domain from user input"""
+    global x0,y0,x1,y1,x2,y2,x3,y3,nx,ny,eltype
+    if model is not None:
+        if ask('You have already merged the parts! I can not add new parts anymore.\nYou should first delete everything and recreate the parts.',['Delete','Cancel']) == 'Delete':
+            deleteAll()
+        else:
+            return
+    if res is None:
+        res = askItems([
+            ('Vertex 0',(x0,y0)),
+            ('Vertex 1',(x1,y1)),
+            ('Vertex 2',(x2,y2)),
+            ('Vertex 3',(x3,y3)),
+            ('nx',nx),
+            ('ny',ny),
+            ('eltype',eltype,'select',['quad','tri-u','tri-d']),
+            ])
+    if res:
+        x0,y0 = res['Vertex 0']
+        x1,y1 = res['Vertex 1']
+        x2,y2 = res['Vertex 2']
+        x3,y3 = res['Vertex 3']
+        nx = res['nx']
+        ny = res['ny']
+        eltype = res['eltype']
+        diag = {'quad':'', 'tri-u':'u', 'tri-d':'d'}[eltype]
+        xold = rectangle(1,1).coords
+        xnew = Coords([[x0,y0],[x1,y1],[x2,y2],[x3,y3]])
+        M = rectangle(nx,ny,1.,1.,diag=diag).toMesh().isopar('quad4',xnew,xold)
+        addPart(M)
+
+
+def addPart(M):
+    """Add a Mesh to the parts list."""
     global parts
     n = len(parts)
-    F.setProp(n)
-    export({'part-%s'%n:F})
-    parts.append(F)
-    femodels.append(F.feModel())
+    part = M.setProp(n)
+    parts.append(part)
+    partname = 'part-%s'%n
+    export({partname:part})
+    mesh_menu.selection.names.append(partname)
+    mesh_menu.selection.draw()
+
+
+def convertQuadratic():
+    """Convert the parts to quadratic"""
+    global parts
+    parts = [ p.convert('quad9') for p in parts ]
+    mesh_menu.selection.changeValues(parts)
+    drawParts()
+    
 
 def drawParts():
     """Draw all parts"""
     clear()
-    draw(parts)
-    [ drawNumbers(p) for p in parts ]
-    [ drawNumbers(Formex(fem[0]),color=red) for fem in femodels ]
-    zoomAll()
+    ## draw(parts)
+    ## [ drawNumbers(p,color=blue) for p in parts ]
+    ## [ drawNumbers(p.coords,color=red) for p in parts ]
+    ## zoomAll()
+    mesh_menu.selection.draw()
 
 
 ######################## the model ####################
+    
 
 def createModel():
     """Merge all the parts into a Finite Element model."""
     global model,PDB
-    model = Model(*mergeModels(femodels))
+    model = Model(*mergeMeshes(parts))
     PDB = PropertyDB()
     drawModel()
 
@@ -259,24 +312,29 @@ def deleteAllBcons():
 
 xload = 0.0
 yload = 0.0
+nloads = 1
+lc = 0
 
 def setCLoad():
     """Pick the points with load condition."""
-    global xload,yload
+    global xload,yload,nloads
     if model is None:
         warn()
         return
     removeHighlights()
-    res = askItems([('x-load',xload),('y-load',yload)])
+    res = askItems([
+        #('load-case',lc),
+        ('x-load',xload),('y-load',yload)])
     if res:
+        #lc = res['load-case']
         xload = res['x-load']
         yload = res['y-load']
         nodeset = pickNodes()
         if len(nodeset) > 0:
             print nodeset
-            if len(nodeset) > 0:
-                print "SETTING CLOAD %s" % [xload,yload,0.,0.,0.,0.]
-                PDB.nodeProp(set=nodeset,cload=[xload,yload,0.,0.,0.,0.])
+            print "SETTING CLOAD %s" % [xload,yload,0.,0.,0.,0.]
+            PDB.nodeProp(set=nodeset,tag="Load-%s"%lc,cload=[xload,yload,0.,0.,0.,0.])
+            nloads = max(nloads,lc+1)
 
 
 def deleteAllCLoads():
@@ -350,6 +408,204 @@ def createAbaqusInput():
     step2 = Step(time=[1.,1.,0.01,1.],nlgeom='no',tags=[2])
 
     AbqData(model,PDB,[step1,step2],out=out,res=res).write(jobname)
+
+############################# Calix ##############################
+def createCalixInput():
+    """Write the Calix input file."""
+
+    checkWorkdir()
+
+    if model is None:
+        warn()
+        return
+    
+    # ask job name from user
+    res = askItems([
+        ('JobName',feresult_name.next()),
+        ('Header','A Calix example'),
+        ])
+    if not res:
+        return
+
+    jobname = res['JobName']
+    header = res['Header']
+    if not jobname:
+        print "No Job Name: writing to sys.stdout"
+        jobname = None
+
+    filnam = jobname+'.dta'
+    fil = open(filnam,'w')
+    
+    nnodes = model.coords.shape[0]
+    nelems = model.celems[-1]
+    nplex = [ e.shape[1] for e in model.elems ]
+    if min(nplex) != max(nplex):
+        print [ e.shape for e in model.elems ]
+        warning("All parts should have same element type")
+        return
+    nodel = nplex[0]
+
+    # Get materials
+    matprops = PDB.getProp(kind='e',attr=['section']) 
+    # E, nu, thickness, rho
+    mats = array([[mat.young_modulus,
+                   mat.poisson_ratio,
+                   mat.thickness,
+                   0.0,      # rho was not defined in material
+                   ] for mat in matprops]) 
+    matnr = zeros(nelems,dtype=int32)
+    for i,mat in enumerate(matprops):  # proces in same order as above!
+        matnr[mat.set] = i+1
+    print matnr
+    nmats = mats.shape[0]
+    nloads = 0
+    # Header
+    fil.write("""; calix data file generated by %s
+; jobname=%s
+start: %s
+;use cmdlog 'calix.log'
+;use for messages cmdlog
+file open 'femodel.tmp' write da 1
+yes cmdlog
+;yes debug
+use for cmdlog output
+femodel elast stress 2
+use for cmdlog cmdlog
+;-----------------------------------------
+; Aantal knopen:   %s
+; Aantal elementen:   %s
+; Aantal materialen:     %s
+; Aantal belastingsgevallen: %s
+"""% (GD.Version,jobname,header,nnodes,nelems,nmats,nloads))
+    # Nodal coordinates
+    fil.write(""";-----------------------------------------
+; Knopen
+;--------
+nodes coord %s 1
+""" % nnodes)
+    fil.write('\n'.join(["%5i%10.2f%10.2f"%(i,x[0],x[1]) for i,x in zip(arange(nnodes)+1,model.coords)]))
+    fil.write('\n\n')
+    # Boundary conditions
+    fil.write(""";-----------------------------------------
+; Verhinderde verplaatsingen
+;-------------------------
+bound bcon
+plane
+""")
+    for p in PDB.getProp(kind='n',attr=['bound']):
+        bnd = "%5i" + "%5i"*2 % (p.bound[0],p.bound[1])
+        if p.set is None:
+            nod = arange(model.nnodes)
+        else:
+            nod = array(p.set)
+        fil.write('\n'.join([ bnd % i for i in nod+1 ]))
+    fil.write('\n\n')
+    fil.write("""print bcon 3
+                           $$$$$$$$$$$$$$$$$$$$$$$
+                           $$      D O F S      $$
+                           $$$$$$$$$$$$$$$$$$$$$$$
+""")
+    
+    # Materials
+    fil.write(""";-----------------------------------------
+; Materialen
+;-----------
+array mat     1 4
+""")
+    fil.write('\n'.join([ "%.4e "*4 % tuple(m) for m in mats]))
+    fil.write('\n\n')
+    fil.write("""print mat 3
+                           $$$$$$$$$$$$$$$$$$$$$$$
+                           $$ M A T E R I A L S $$
+                           $$$$$$$$$$$$$$$$$$$$$$$
+""")
+    
+    # Elements
+    for grp in model.elems:
+        nelems,nplex = grp.shape
+        fil.write(""";-----------------------------------------
+; Elementen
+;----------
+elements nodes matnr  %s %s 1
+""" % (nplex,nelems))
+        fil.write('\n'.join(["%5i"*(nplex+2) % tuple([i,1]+e.tolist()) for i,e in zip(arange(nelems)+1,grp+1)]))
+        fil.write('\n\n')
+        fil.write("""plane plane coord bcon nodes matnr 2 2
+""")
+        
+    # Nodal Loads
+    fil.write("""text 3 1
+                           $$$$$$$$$$$$$$$$$$$$
+                           $$  NODAL  LOADS  $$
+                           $$$$$$$$$$$$$$$$$$$$
+loads f bcon 1
+""")
+    loadcase=1
+    for p in PDB.getProp('n',attr=['cload']):
+        if p.set is None:
+            nodeset = range(calpyModel.nnodes)
+        else:
+            nodeset = p.set
+        F = [0.0,0.0]
+        for i,v in p.cload:
+            if i in [0,1]:
+                F[i] = v
+        fil.write(''.join(["%5i%5i%10.2f%10.2f\n" % (n,loadcase,F[0],F[1]) for n in nodeset]))
+
+    fil.write('\n')
+    
+    # Distributed loads
+    fil.write("""""")
+    fil.write("""no debug
+print f 3
+                           $$$$$$$$$$$$$$$$$$$$
+                           $$  LOAD  VECTOR  $$
+                           $$$$$$$$$$$$$$$$$$$$
+;
+assemble plane mat s 0 0 0 3
+""")
+    # Solve and output
+    fil.write(""";------------------------------------------------solve+output
+flavia mesh '%s.flavia.msh' %s
+flavia nodes coord
+flavia elems nodes matnr  %s
+flavia results '%s.flavia.res'
+solbnd s f
+delete s
+text named 1
+"Displacement" "Elastic Analysis"
+text typed 1
+Vector OnNodes
+text names 1
+"Stress" "Elastic Analysis"
+text types 1
+Matrix OnNodes
+intvar set 1 1
+loop 1
+ displ f bcon displ $1 1
+ tran displ disp
+ flavia result named typed disp $1
+ stress plane mat f stresg $1 1
+ gp2nod plane stresg strese 0 1
+ nodavg plane nodes strese stren nval 1
+ tran stren stre
+ flavia result names types stre $1
+ intvar add 1 1
+next
+stop
+""" % (jobname,nplex,nplex,jobname))
+
+    # Done: Close data file
+    fil.close()
+    showFile(filnam,mono=True)
+
+    if ack("Shall I run the Calix analysis?"):
+        # Run the analysis
+        outfile = utils.changeExt(filnam,'res')
+        cmd = "calix %s %s" % (filnam,outfile)
+        utils.runCommand(cmd)
+        showFile(outfile,mono=True)
+    
     
 
 ##############################################################################
@@ -370,7 +626,16 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
     # Load the needed calpy modules    
     from plugins import calpy_itf
     calpy_itf.check()
+
+    ## # Load development version
+    ## import sys
+    ## sys.path.insert(0,'/home/bene/prj/calpy')
+    ## print sys.path
+
     import calpy
+    reload(calpy)
+    print calpy.__path__
+    
     calpy.options.optimize=True
     from calpy import femodel,fe_util,plane
     from calpy.arrayprint import aprint
@@ -401,7 +666,13 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
     calpyModel = femodel.FeModel(2,"elast","Plane_Stress")
     calpyModel.nnodes = model.coords.shape[0]
     calpyModel.nelems = model.celems[-1]
-    calpyModel.nnodel = 4
+    print [ e.shape for e in model.elems ]
+    nplex = [ e.shape[1] for e in model.elems ]
+    if min(nplex) != max(nplex):
+        warning("All parts should have same element type")
+        return
+    
+    calpyModel.nnodel = nplex[0]
 
     # 2D model in calpy needs 2D coordinates
     coords = model.coords[:,:2]
@@ -443,7 +714,7 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
 
    
     ########### Find number of load cases ############
-    calpyModel.nloads = 1
+    calpyModel.nloads = nloads
     calpyModel.PrintModelData()
     ngp = 2
     nzem = 3
@@ -497,9 +768,10 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
     # Also notice that the indexing inside the bcon array uses numpy
     # convention (starting at 0), thus no adding 1 is needed!
     print "Assembling Concentrated Loads"
-    calpyModel.nloads = 1
     f = zeros((calpyModel.ndof,calpyModel.nloads),float)
     for p in PDB.getProp('n',attr=['cload']):
+        lc = 0
+        #lc = int(p.tag)
         if p.set is None:
             nodeset = range(calpyModel.nnodes)
         else:
@@ -509,7 +781,7 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
             if i in [0,1]:
                 F[i] = v
         for n in nodeset:
-            f[:,0] = fe_util.AssembleVector(f[:,0],F,bcon[n])
+            f[:,lc] = fe_util.AssembleVector(f[:,lc],F,bcon[n])
 
     print "Assembling distributed loads"
     # This is a bit more complex. See Calpy for details
@@ -520,7 +792,7 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
     ngroups = model.ngroups()
     s = [ "" ] * ngroups
     nb = [ 0 ] * ngroups
-    loadcase = 1
+    lc = 1
     for p in PDB.getProp('e',attr=['eload']):
         xload = yload = 0.
         if p.label == 'x':
@@ -534,7 +806,7 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
         g = p.group
         print "Group %s" % g
         for e in p.set:
-            s[g] += "%s %s %s %s %s\n" % (e+1,p.edge+1,loadcase,xload,yload)
+            s[g] += "%s %s %s %s %s\n" % (e+1,p.edge+1,lc,xload,yload)
             nb[g] += 1
     #print s,nb
     for nbi,si,nodes,matnr,Plane in zip(nb,s,NodesGrp,MatnrGrp,PlaneGrp):
@@ -566,22 +838,22 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
         res=flavia.ResultsFile(jobname)
         
     # compute stresses
-    for l in range(calpyModel.nloads):
+    for lc in range(calpyModel.nloads):
         
-        print "Results for load case %d" %(l+1)
+        print "Results for load case %d" %(lc+1)
         print "Displacements"
-        aprint(displ[:,:,l],header=['x','y'],numbering=True)
+        aprint(displ[:,:,lc],header=['x','y'],numbering=True)
 
         if flavia:
-            flavia.WriteResultsHeader(res,'"Displacement" "Elastic Analysis"',l+1,'Vector OnNodes')
-            flavia.WriteResults(res,displ[:,:,l])
+            flavia.WriteResultsHeader(res,'"Displacement" "Elastic Analysis"',lc+1,'Vector OnNodes')
+            flavia.WriteResults(res,displ[:,:,lc])
             
         stresn = count = None
         i = 0
         for e,P in zip(model.elems,PlaneGrp):
             i += 1
             #P.debug = 1
-            stresg = P.StressGP (v[:,l],mats)
+            stresg = P.StressGP (v[:,lc],mats)
             if verbose:
                 print "elem group %d" % i
                 print "GP Stress\n", stresg
@@ -604,7 +876,7 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
             aprint(stresn,header=['sxx','syy','sxy'],numbering=True)
                 
         if flavia:
-            flavia.WriteResultsHeader(res,'"Stress" "Elastic Analysis"',l+1,'Matrix OnNodes')
+            flavia.WriteResultsHeader(res,'"Stress" "Elastic Analysis"',lc+1,'Matrix OnNodes')
             flavia.WriteResults(res,stresn)
 
     
@@ -627,25 +899,61 @@ def runCalpyAnalysis(jobname=None,verbose=False,flavia=False):
     name = feresult_name.next()
     export({name:DB})
     showInfo("The results have been exported as %s\nYou can now use the postproc menu to display results" % name)
+    postproc_menu.selection.set(name)
+    postproc_menu.selectDB(DB)
+    postproc_menu.open_results_dialog()
     
 
-def autoRun():
+def autoRun(quadratic=False):
     clear()
-    createPart(dict(x0=0.,x1=1.,y0=0.,y1=1.,nx=4,ny=4,eltype='quad'))
-    createPart(dict(x0=0.,x1=-1.,y0=0.,y1=1.,nx=4,ny=4,eltype='quad'))
+    if quadratic:
+        nx,ny = 2,2
+    else:
+        nx,ny = 4,4
+    createRectPart(dict(x0=0.,x2=1.,y0=0.,y2=1.,nx=nx,ny=ny,eltype='quad'))
+    createRectPart(dict(x0=0.,x2=-1.,y0=0.,y2=1.,nx=nx,ny=ny,eltype='quad'))
+    if quadratic:
+        convertQuadratic()
     createModel()
     nodenrs = arange(model.coords.shape[0])
     PDB.elemProp(eltype='CPS4',section=ElemSection(section=section))
+    if quadratic:
+        ny *= 2
     PDB.nodeProp(set=nodenrs[:ny+1],bound=[1,1,0,0,0,0])
     PDB.nodeProp(set=nodenrs[-(ny+1):],cload=[10.,0.,0.,0.,0.,0.])
     runCalpyAnalysis('FeEx',verbose=True)
 
+def autoRun2():
+    clear()
+    nx,ny = 1,1
+    createRectPart(dict(x0=0.,x2=1.,y0=0.,y2=1.,nx=nx,ny=ny,eltype='quad'))
+    convertQuadratic()
+    createModel()
+    nodenrs = arange(model.coords.shape[0])
+    xmin,xmax = model.coords.bbox()[:,0]
+    xtol = (xmax-xmin) / 1000.
+    left = model.coords.test(dir=0,min=xmin-xtol,max=xmin+xtol)
+    right = model.coords.test(dir=0,min=xmax-xtol,max=xmax+xtol)
+    leftnrs = where(left)[0]
+    rightnrs = where(right)[0]
+    print leftnrs
+    print rightnrs
+    
+    PDB.elemProp(eltype='CPS4',section=ElemSection(section=section))
+    ny *= 2
+    PDB.nodeProp(set=leftnrs,bound=[1,1,0,0,0,0])
+    PDB.nodeProp(set=rightnrs,cload=[10.,0.,0.,0.,0.,0.])
+
+    print "This example is incomplete."
+    print PDB
+    #runCalpyAnalysis('FeEx',verbose=True)
+
 def autoConv():
     clear()
     res = askItems([('nx',1),('ny',1)])
     nx = res['nx']
     ny = res['ny']
-    createPart(dict(x0=0.,x1=10.,y0=0.,y1=1.,nx=nx,ny=ny,eltype='quad'))
+    createRectPart(dict(x0=0.,x1=10.,y0=0.,y1=1.,nx=nx,ny=ny,eltype='quad'))
     createModel()
     nodenrs = arange(model.coords.shape[0])
     PDB.elemProp(eltype='CPS4',section=ElemSection(section=section))
@@ -667,7 +975,9 @@ def create_menu():
     """Create the FeEx menu."""
     MenuData = [
         ("&Delete All",deleteAll),
-        ("&Create Part",createPart),
+        ("&Create Rectangular Part",createRectPart),
+        ("&Create QuadrilateralPart",createQuadPart),
+        ("&Convert to Quadratic",convertQuadratic),
         ("&Show All",drawParts),
         ("---",None),
         ("&Merge Parts into Model",createModel),
@@ -686,16 +996,18 @@ def create_menu():
         ("&Print property database",printDB),
         ("---",None),
         ("&Create Abaqus input file",createAbaqusInput),
+        ("&Create Calix input file",createCalixInput),
         ("&Run Calpy analysis",runCalpyAnalysis),
         ("---",None),
         ("&Import all",importAll),
         ("&Export all",exportAll),
         ("&Autorun example",autoRun),
+        ("&Autorun quadratic example",autoRun2),
         ("&Autoconv example",autoConv),
         ("---",None),
         ("&Close Menu",close_menu),
         ]
-    return widgets.Menu('FeEx',items=MenuData,parent=GD.GUI.menu,before='help')
+    return menu.Menu('FeEx',items=MenuData,parent=GD.GUI.menu,before='help')
 
  
 def show_menu():
@@ -720,6 +1032,7 @@ def reload_menu():
 
 if __name__ == "draw":
 
+    mesh_menu.show_menu()
     resetData()
     reset()
     reload_menu()
