@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: Hesperia.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Hesperia.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,12 +31,12 @@ techniques = ['menu', 'dialog', 'persistence', 'colors']
 """
 
 import simple,utils
-from connectivity import compactElems
+from connectivity import Connectivity
 from plugins.surface import TriSurface
 from plugins.properties import *
 from plugins.fe_abq import *
 from gui.colorscale import ColorScale,ColorLegend
-import gui.decors
+from gui import menu,decors
 
 import time
 
@@ -210,7 +210,7 @@ def assignProperties():
         sel = pickElements()
         if sel.has_key(0):
             GD.debug("PICKED NUMBERS:%s" % sel)
-            F.p[sel[0]] = p
+            F.prop[sel[0]] = p
         undraw(FA)
         FA = draw(F,view=None,bbox=None)
 
@@ -222,7 +222,7 @@ def exportProperties():
         p = res['Property Name']
         if not p.startswith('prop:'):
             p = "prop:%s" % p
-        export({p:F.p})
+        export({p:F.prop})
 
 
 def selectProperties():
@@ -239,7 +239,7 @@ def saveProperties(fn = None):
     if not fn:
         fn = askNewFilename(dirname,filter="Property files (*.prop)")
     if fn:
-        F.p.tofile(fn,sep=',')
+        F.prop.tofile(fn,sep=',')
 
         
 def readProperties(fn = None):
@@ -286,18 +286,18 @@ def createFrameModel():
     print "The structure has %s nodes, %s edges and %s faces" % (S.ncoords(),S.nedges(),S.nfaces())
 
     # Create the steel structure
-    E = Formex(nodes[S.edges])
+    E = Formex(nodes[S.getEdges()])
     clear()
     draw(E)
     
     # Get the tri elements that are part of a quadrilateral:
-    prop = F.p
-    quadtri = S.faces[prop==6]
+    prop = F.prop
+    quadtri = S.getFaces()[prop==6]
     nquadtri = quadtri.shape[0]
     print "%s triangles are part of quadrilateral faces" % nquadtri
     if nquadtri > 0:
         # Create triangle definitions of the quadtri faces
-        tri = compactElems(S.edges,quadtri)
+        tri = Connectivity.tangle(quadtri,S.getEdges())
         D = Formex(nodes[tri])
         clear()
         flatwire()
@@ -310,14 +310,14 @@ def createFrameModel():
     internal = [ c[0] for c in conn if len(c[1]) > 1 ]
     print "Internal edges in quadrilaterals: %s" % internal
     
-    E = Formex(nodes[S.edges],1)
-    E.p[internal] = 6
+    E = Formex(nodes[S.getEdges()],1)
+    E.prop[internal] = 6
     wireframe()
     clear()
     draw(E)
 
     # Remove internal edges
-    tubes = S.edges[E.p != 6]
+    tubes = S.getEdges()[E.prop != 6]
 
     print "Number of tube elements after removing %s internals: %s" % (len(internal),tubes.shape[0])
 
@@ -649,10 +649,13 @@ def runCalpyAnalysis():
     """
 
     ############################
-    # Load the needed calpy modules    
+    # Load the needed calpy modules
+    # You can prepend your own path here to override the installed calpy
+    # sys.path[0:0] = ['/home/bene/prj/calpy']
     from plugins import calpy_itf
     calpy_itf.check()
     import calpy
+    print calpy
     calpy.options.optimize=True
     from calpy import fe_util,beam3d
     ############################
@@ -788,7 +791,12 @@ def runCalpyAnalysis():
     nlc = 1
     loads = zeros((ndof,nlc),float)
     for p in FE.prop.getProp('n',attr=['cload']):
-        loads[:,0] = fe_util.AssembleVector(loads[:,0],p.cload,bcon[p.set,:])
+        cload = zeros(6)
+        for i,v in p.cload:
+            cload[i] += v
+        print cload
+        print cload.shape
+        loads[:,0] = fe_util.AssembleVector(loads[:,0],cload,bcon[p.set,:])
     if verbose:
         print "Calpy.Loads"
         print loads
@@ -909,7 +917,7 @@ def create_menu():
         ("---",None),
         ("&Close Menu",close_menu),
         ]
-    return widgets.Menu('Hesperia',items=MenuData,parent=GD.GUI.menu,before='help')
+    return menu.Menu('Hesperia',items=MenuData,parent=GD.GUI.menu,before='help')
 
  
 def show_menu():
