@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: Inertia.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Inertia.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -54,7 +54,7 @@ Axes = unitAxes()
 def showPrincipal1(F):
     """Show the principal axes."""
     clear()
-    C,I = inertia.inertia(F.f)
+    C,I = inertia.inertia(F.coords)
     GD.message("Center: %s" % C)
     GD.message("Inertia tensor: %s" % I)
     Iprin,Iaxes = inertia.principal(I)
