@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: Isopar.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Isopar.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -30,7 +30,7 @@ techniques = ['dialog', 'colors','isopar']
 
 """
 
-from plugins.isopar import *
+from plugins import isopar
 import simple
 import elements
 
@@ -109,7 +109,7 @@ if sdim < tdim:
 x1 = x1.scale(sz)
 x2 = x2.scale(sz)
 
-G=isopar(F,eltype,x2.points(),x1.points())
+G=F.isopar(eltype,x2.points(),x1.points())
 G.setProp(1)
 
 message('This is the transformed Formex')
