@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: Light.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Light.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,7 +31,6 @@ techniques = ['dialog', 'colors', 'persistence']
 
 """
 
-from gui.prefMenu import setRender
 
 smooth()
 
@@ -44,7 +43,6 @@ color3 = resize(color2,F.shape())
 draw(F,color=color3)
 
 
-setRender()
 
 for a in [ 'ambient', 'specular', 'emission', 'shininess' ]:
     v = getattr(GD.canvas,a)
