@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: Lustrum.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Lustrum.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,18 +29,19 @@ topics = ['curve','drawing','illustration']
 techniques = ['colors','persistence','curve','lima','import']
 
 """
+reset()
 from pyformex.examples.Lima import *
 from project import Project
 linewidth(2)
 fgcolor(blue)
-grow('Plant1',False,False)
+grow('Plant1',ngen=7,clearing=False,text=False)
 P = Project(os.path.join(GD.cfg['datadir'],'blippo-5.pyf'),create=False)
 P.load()
 curve = P['blippo-5']
+bb = curve.coords.bbox()
+ctr = bb.center()
+siz = bb.sizes()
+curve.coords = curve.coords.trl(0,-ctr[0]).scale(50./siz[0])
 draw(curve,color=pyformex_pink,linewidth=5)
 
-## if ask("This is the pyFormex Lustrum release\n\nClick Remove to no longer show this startup screen\n",choices=['Remove','Keep']) == 'Remove':
-##     GD.cfg['gui/easter_egg'] = None
-    
-
 # End
