@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: Novation.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Novation.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -97,7 +97,7 @@ if export and checkWorkdir():
         G += F.selectNodes([0,i-1,i])
     clear()
     draw(G)
-    surface.write_stla(f,G.f)
+    surface.write_stla(f,G.coords)
     f.close()
 
 # End
