@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: Section2D.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: Section2D.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -34,6 +34,7 @@ techniques = []
 """
 
 from plugins.section2d import *
+from plugins.mesh import Mesh
 import simple,connectivity,mydict
 
 
@@ -55,24 +56,20 @@ def circle_example():
 def close_loop_example():
     # one more example, originally not a closed loop curve
     F = Formex(pattern('11')).replic(2,1,1) + Formex(pattern('2')).replic(2,2,0)
-    nodes,elems = F.feModel()
+    M = F.toMesh()
+    draw(M,color='green')
+    drawNumbers(M,color=red)
+    drawNumbers(M.coords,color=blue)
 
-    FN = Formex(nodes)
-    drawNumbers(FN,color=blue)
-
-    F = Formex(nodes[elems])
-    draw(F,color='green')
-    drawNumbers(F,color=red)
-
-    print "Original elements:",elems
-    ret, sorted = connectivity.closedLoop(elems)
+    print "Original elements:",M.elems
+    ret, sorted = connectivity.closedLoop(M.elems)
     print "Sorted elements:",sorted
 
     showInfo('Click to continue')
     clear()
-    F = Formex(nodes[sorted])
-    drawNumbers(F)
-    return F
+    M = Mesh(M.coords,sorted)
+    drawNumbers(M)
+    return M.toFormex()
 
 
 clear()
