@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
 # $Id: SpaceTrussRoof_fe_abq.py 150 2006-11-01 11:13:34Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -68,24 +68,24 @@ clear();linewidth(1);draw(F,view='myview1')
 #Creating FE-model
 ###################
 
-nodes,elems=F.feModel()
+M = F.toMesh()
 
 ###############
 #Creating elemsets
 ###################
 # Remember: elems are in the same order as elements in F
-topbar = where(F.p==3)[0]
-bottombar = where(F.p==0)[0]
-diabar = where(F.p==1)[0]
+topbar = where(F.prop==3)[0]
+bottombar = where(F.prop==0)[0]
+diabar = where(F.prop==1)[0]
 
 ###############
 #Creating nodesets
 ###################
 
-nnod=nodes.shape[0]
+nnod=M.ncoords()
 nlist=arange(nnod)
 count = zeros(nnod)
-for n in elems.flat:
+for n in M.elems.flat:
     count[n] += 1
 field = nlist[count==8]
 topedge = nlist[count==7]
@@ -138,12 +138,12 @@ out = Output(type='field',variable='preselect')
 res = [ Result(kind='element',keys=['S']),
 	Result(kind='node',keys=['U'])
 	]
-model = Model(nodes,elems)
+model = Model(M.coords,M.elems)
 
 if not checkWorkdir():
     exit()
 
 message("Writing the Abaqus file")
-AbqData(model,P,[step],eprop=F.p,out=[out],res=res).write('SpaceTruss')
+AbqData(model,P,[step],eprop=F.prop,out=[out],res=res).write('SpaceTruss')
 
 # End
