@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
 # $Id: SpaceTrussRoof_calpy.py 154 2006-11-03 19:08:25Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,16 +29,22 @@ topics = ['FEA']
 techniques = ['dialog', 'animation', 'persistence', 'colors'] 
 """
 
+from plugins.mesh import Mesh
 from plugins.properties import *
 ############################
 # Load the needed calpy modules    
 from plugins import calpy_itf
 calpy_itf.check()
-
+import calpy
+calpy.options.optimize = False
 from calpy.fe_util import *
 from calpy.truss3d import *
 
 ############################
+
+if not checkWorkdir():
+    exit()
+
 import time
 
 ####
@@ -77,24 +83,24 @@ clear();linewidth(1);draw(F,view='myview1')
 #Creating FE-model
 ###################
 
-nodes,elems=F.feModel()
+mesh = F.toMesh()
 
 ###############
 #Creating elemsets
 ###################
-# Remember: elems are in the same order as elements in F
-topbar = where(F.p==3)[0]
-bottombar = where(F.p==0)[0]
-diabar = where(F.p==1)[0]
+# Remember: elements in mesh are in the same order as elements in F
+topbar = where(F.prop==3)[0]
+bottombar = where(F.prop==0)[0]
+diabar = where(F.prop==1)[0]
 
 ###############
 #Creating nodesets
 ###################
 
-nnod=nodes.shape[0]
-nlist=arange(nnod)
+nnod = mesh.nnodes()
+nlist = arange(nnod)
 count = zeros(nnod)
-for n in elems.flat:
+for n in mesh.elems.flat:
     count[n] += 1
 field = nlist[count==8]
 topedge = nlist[count==7]
@@ -138,12 +144,11 @@ bcon[support] = [ 1,1,1 ]
 NumberEquations(bcon)
 
 #materials
-nelems = elems.shape[0]
 mats = array([ [p.young_modulus,p.density,p.cross_section] for p in props])
-matnr = zeros_like(F.p)
+matnr = zeros_like(F.prop)
 for i,p in enumerate(props):
     matnr[p.set] = i+1
-matnod = concatenate([matnr.reshape((-1,1)),elems+1],axis=-1)
+matnod = concatenate([matnr.reshape((-1,1)),mesh.elems+1],axis=-1)
 ndof = bcon.max()
 
 # loads
@@ -155,23 +160,30 @@ for n in edge:
     loads[:,0] = AssembleVector(loads[:,0],[ 0.0, 0.0, Q/2 ],bcon[n,:])
 
 message("Performing analysis: this may take some time")
-outfilename = os.path.splitext(os.path.basename(GD.scriptName))[0] + '.out'
 
-if not isWritable(outfilename):
-    tmpdir = '/var/tmp'
-    if isWritable(tmpdir):
-        outfilename = os.path.join(tmpdir,outfilename)
-    else:
-        message("No writeable path: I can not execute the simulation\nCopy the script to a writeable path and try again.")
-        exit()
+# Find a candidate for the output file
+fullname = os.path.splitext(__file__)[0] + '.out'
+basename = os.path.basename(fullname)
+dirname = os.path.dirname(fullname)
+outfilename = None
+for candidate in [dirname,GD.cfg['workdir'],'/var/tmp']:
+    if isWritable(candidate):
+        fullname = os.path.join(candidate,basename)
+        if not os.path.exists(fullname) or isWritable(fullname):
+            outfilename = fullname
+            break
+
+if outfilename is None:
+    error("No writeable path: I can not execute the simulation.\nCopy the script to a writeable path and try running from there.")
+    exit()
 
 outfile = file(outfilename,'w')
-message("Output is written to file '%s' in %s" % (outfilename,os.getcwd()))
+message("Output is written to file '%s'" % os.path.realpath(outfilename))
 stdout_saved = sys.stdout
 sys.stdout = outfile
 print "# File created by pyFormex on %s" % time.ctime()
 print "# Script name: %s" % GD.scriptName
-displ,frc = static(nodes,bcon,mats,matnod,loads,Echo=True)
+displ,frc = static(mesh.coords,bcon,mats,matnod,loads,Echo=True)
 print "# Analysis finished on %s" % time.ctime()
 sys.stdout = stdout_saved
 outfile.close()
@@ -186,10 +198,14 @@ if GD.options.gui:
     from gui.colorscale import *
     import gui.decors
 
+
+    def showOutput():
+        #showText(file(outfilename).read())
+        showFile(outfilename)
+        
+
     def showForces():
-        # Creating a formex for displaying results is fairly easy
-        results = Formex(nodes[elems],range(nelems))
-        # Now give the formex some meaningful colors.
+        # Give the mesh some meaningful colors.
         # The frc array returns element forces and has shape
         #  (nelems,nforcevalues,nloadcases)
         # In this case there is only one resultant force per element (the
@@ -203,7 +219,7 @@ if GD.options.gui:
         #aprint(cval,header=['Red','Green','Blue'])
         clear()
         linewidth(3)
-        draw(results,color=cval)
+        draw(mesh,color=cval)
         drawtext('Normal force in the truss members',300,50,size=24)
         CL = ColorLegend(CS,100)
         CLA = decors.ColorLegend(CL,10,10,30,200,size=24)
@@ -213,10 +229,10 @@ if GD.options.gui:
     # Show a deformed plot
     def deformed_plot(dscale=100.):
         """Shows a deformed plot with deformations scaled with a factor scale."""
-        dnodes = nodes + dscale * displ[:,:,0]
-        deformed = Formex(dnodes[elems],F.p)
         # deformed structure
-        FA = draw(deformed,bbox=None,wait=False)
+        dnodes = mesh.coords + dscale * displ[:,:,0]
+        deformed = Mesh(dnodes,mesh.elems,mesh.prop)
+        FA = draw(deformed,bbox='last',view=None,wait=False)
         TA = drawtext('Deformed geometry (scale %.2f)' % dscale,300,50,size=24)
         return FA,TA
 
@@ -238,7 +254,7 @@ if GD.options.gui:
     def getOptimscale():
         """Determine an optimal scale for displaying the deformation"""
         siz0 = F.sizes()
-        dF = Formex(displ[:,:,0][elems])
+        dF = Formex(displ[:,:,0][mesh.elems])
         #clear(); draw(dF,color=black)
         siz1 = dF.sizes()
         return niceNumber(1./(siz1/siz0).max())
@@ -277,8 +293,8 @@ if GD.options.gui:
 
 
     optimscale = getOptimscale()
-    options = ['Cancel','Member forces','Deformation','Animated deformation']
-    functions = [None,showForces,showDeformation,showAnimatedDeformation]
+    options = ['None','Output File','Member forces','Deformation','Animated deformation']
+    functions = [None,showOutput,showForces,showDeformation,showAnimatedDeformation]
     while True:
         ans = ask("Which results do you want to see?",options)
         ind = options.index(ans)
