@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
 # $Id: Sphere_surface.py 154 2006-11-03 19:08:25Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -96,6 +96,6 @@ if ack("Create Surface model?"):
         if fn:
             from plugins import surface
             f = file(fn,'w')
-            surface.write_stla(f,T.f)
+            surface.write_stla(f,T.coords)
             f.close()
 
