@@ -1,6 +1,6 @@
 #!/usr/bin/env pyformex --gui
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -26,6 +26,8 @@
 level = 'normal'
 topics = ['geometry']
 techniques = ['curve','transform']
+
+See also the Sweep example for a more sophisticated Spirals application
 """
 
 from plugins import curve
@@ -44,14 +46,12 @@ draw(F)
 
 def spiral(X,dir=[0,1,2],rfunc=lambda x:1,zfunc=lambda x:1):
     """Perform a spiral transformation on a coordinate array"""
-    print X.shape
     theta = X[...,dir[0]]
     r = rfunc(theta) + X[...,dir[1]]
     x = r * cos(theta)
     y = r * sin(theta)
     z = zfunc(theta) + X[...,dir[2]]
     X = hstack([x,y,z]).reshape(X.shape)
-    print X.shape
     return Coords(X)
 
 
@@ -64,26 +64,17 @@ a = c*tand(phi)
 b = tand(phi) / tand(alpha2)
  
 
-print "a = %s, b = %s, c = %s" % (a,b,c)
-print c*b/a
-print tand(45.)
-print arctan(c*b/a) / Deg
-
 zf = lambda x : c * exp(b*x)
 rf = lambda x : a * exp(b*x)
 
 
-S = spiral(F.f,[0,1,2],rf)#.rosette(nwires,360./nwires)
+S = spiral(F.coords,[0,1,2],rf)#.rosette(nwires,360./nwires)
 
 PL = curve.PolyLine(S[:,0,:])
 
-## F = Formex(pattern('15263748'))
-## PL = curve.PolyLine(F.f[:,0,:])
-
 clear()
 draw(PL,color=red)
 draw(PL.coords,color=red)
-#drawNumbers(PL.coords)
 
 
 if ack("Spread point evenly?"):
@@ -94,19 +85,5 @@ if ack("Spread point evenly?"):
     draw(PL,color=blue)
     draw(PL.coords,color=blue)
 
-## sweep = ask("Sweep cross section",['None','line','surface'])
-## if sweep == 'line':
-##     CS = Formex(pattern('123'))  # circumference of a square
-## elif sweep == 'surface':
-##     CS = Formex(mpattern('123'))  # a square surface
-## else:
-##     exit()
-
-## # Use a Mesh, because that already has a 'sweep' function
-## CS = CS.swapAxes(0,2).scale(0.2).toMesh()
-## structure = CS.sweep(PL,normal=0,upvector=None,avgdir=True)
-## clear()
-## draw(structure)
-
 # End
 
