@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: SweepBeam.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: SweepBeam.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -76,9 +76,9 @@ method = ask("Choose extrude method:",['Cancel','Sweep','Connect','Extrude'])
 
 if method == 'Sweep':
     L = simple.line([0,0,0],[0,0,l],el)
-    x = concatenate([L.f[:,0],L.f[-1:,1]])
+    x = concatenate([L.coords[:,0],L.coords[-1:,1]])
     path = curve.PolyLine(x)
-    Beam = Section.sweep(path,normal=[0.,0.,1.])
+    Beam = Section.sweep(path,normal=[0.,0.,1.],upvector=[0.,1.,0.])
 
 elif method == 'Connect':
     Section1 = Section.copy()
