@@ -1,6 +1,6 @@
 #!/usr/bin/env python pyformex.py
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,6 +31,8 @@ techniques = ['widgets','dialog','random','colors']
 """
 
 from numpy.random import rand
+from gui.widgets import simpleInputItem as I
+
     
 setDrawOptions({'clear':True, 'bbox':'auto'})
 linewidth(2) # The linewidth option is not working nyet
@@ -90,11 +92,12 @@ nplex = 3
 eltype = 'auto'
 color = 'element'
 pos = None
-items = [('Geometry Model',geom,'radio',geom_mode),
-         ('Plexitude',nplex,'select',plexitude),
-         ('Element Type',eltype,'select',element_type),
-         ('Color Mode',color,'select',color_mode),
-         ]
+items = [
+    I('geom',geom,'radio',choices=geom_mode,text='Geometry Model'),
+    I('nplex',nplex,'select',choices=plexitude,text='Plexitude'),
+    I('eltype',eltype,'select',choices=element_type,text='Element Type'),
+    I('color',color,'select',choices=color_mode,text='Color Mode'),
+    ]
 
 dialog = None
 
@@ -103,12 +106,8 @@ def show():
     """Accept the data and draw according to them"""
     dialog.acceptData()
     res = dialog.results
-    globals().update(dict(
-        geom = res[items[0][0]],
-        nplex = int(res[items[1][0]]),
-        eltype = res[items[2][0]],
-        color = res[items[3][0]],
-        ))
+    res['nplex'] = int(res['nplex'])
+    globals().update(res)
 
     G = select_geom(geom,nplex,eltype)
     print "GEOM: nelems=%s, nplex=%s" % (G.nelems(),G.nplex())
@@ -136,7 +135,7 @@ def timeOut():
     close()
 
 # Create the non-modal dialog widget and show it
-dialog = widgets.InputDialog(items,caption='Drawing parameters',actions = [('Close',close),('Show',show)],default='Show')
+dialog = widgets.NewInputDialog(items,caption='Drawing parameters',actions = [('Close',close),('Show',show)],default='Show')
 dialog.timeout = timeOut
 dialog.show()
         
