@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
 # $Id: WireStent_calpy.py 147 2006-10-13 09:30:49Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,6 +29,8 @@ topics = ['FEA']
 techniques = ['colors'] 
 """
 
+chdir(__file__)
+
 ############################
 # Load the needed calpy modules    
 
@@ -67,12 +69,28 @@ if GD.options.gui:
 ############################################
 # Perform Analysis
 
+# Create output file
+if not checkWorkdir():
+    print "Could not open a file for writing. I have to stop here"
+    exit()
+
+outfilename = 'WireStent_calpy.out'
+outfile = file(outfilename,'w')
+message("Output is written to file '%s' in %s" % (outfilename,os.getcwd()))
+stdout_saved = sys.stdout
+sys.stdout = outfile
+print "# File created by pyFormex on %s" % time.ctime()
+print "# Script name: %s" % GD.scriptName
+
+
+
 nel = stent.nelems()
 print "Number of elements: %s" % nel
 print "Original number of nodes: %s" % stent.nnodes()
 # Create FE model
 message("Creating Finite Element model: this may take some time.")
-nodes,elems = stent.feModel()
+nodes,elems = stent.fuse(nodesperbox=1)
+
 nnod = nodes.shape[0]
 print "Compressed number of nodes: %s" % nnod
 
@@ -87,7 +105,7 @@ print "After adding a node for orientation: %s" % nnod
 # (remember props are 1,2,3, so are OK)
 
 thirdnode = nnod*ones(shape=(nel,1),dtype=int)
-matnr = reshape(stent.p,(nel,1))
+matnr = reshape(stent.prop,(nel,1))
 elements = concatenate([elems+1,thirdnode,matnr],1)
 
 # Create endnode sets (with calpy numbering)
@@ -162,7 +180,11 @@ import calpy
 calpy.options.optimize=True
 print elements
 displ,frc = static(coords,bcon,mats,elements,loads,Echo=True)
-print "Analysis done"
+
+print "# Analysis finished on %s" % time.ctime()
+sys.stdout = stdout_saved
+outfile.close()
+
 
 
 ################################
@@ -220,4 +242,7 @@ if GD.options.gui:
         drawtext('Deformed geometry (scale %.2f)' % dscale,x,70)
 
 
+    if ack("Show the output file?"):
+        showFile(outfilename)
+
 # End
