@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
 # $Id: X_truss_calpy.py 147 2006-10-13 09:30:49Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -68,7 +68,7 @@ truss.top.setProp(0)
 truss.vert.setProp(2)
 truss.dia1.setProp(1)
 truss.dia2.setProp(1)
-for p in [ truss.bot.p, truss.top.p ]:
+for p in [ truss.bot.prop, truss.top.prop ]:
     p[0] = p[n-1] = 3 
 
 # define member properties
@@ -91,8 +91,8 @@ def getmat(key):
 
 # create model for structural analysis
 model = truss.allBars()
-coords,elems = model.feModel()
-props = model.prop()
+coords,elems = model.fuse()
+props = model.prop
 propset = model.propSet()
 
 clear()
@@ -183,5 +183,8 @@ draw(deformed,color=cval)
 drawtext('Normal force in the truss members',400,100,size=14)
 drawtext('Deformed geometry (scale %.2f)' % dscale,400,130,size=12)
 
+if ack("Show the output file?"):
+    showFile(outfilename)
+
 
 # End
