@@ -1,6 +1,6 @@
-# $Id: actors.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: actors.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,8 +31,9 @@ from drawable import *
 from formex import *
 import elements
 
-from connectivity import reverseIndex
+from plugins.mesh import Mesh
 from plugins.surface import TriSurface
+from marks import TextMark
 
 import timer
 
@@ -172,8 +173,9 @@ class BboxActor(Actor):
         if self.linewidth is not None:
             GL.glLineWidth(self.linewidth)
         drawLines(self.vertices,self.edges,color)
-            
 
+
+ 
 class TriadeActor(Actor):
     """An OpenGL actor representing a triade of global axes."""
 
@@ -208,7 +210,7 @@ class TriadeActor(Actor):
         GL.glScalef (self.size,self.size,self.size) 
         # Coord axes of size 1.0
         GL.glBegin(GL.GL_LINES)
-        pts = Formex(pattern('1')).f.reshape(-1,3)
+        pts = Formex(pattern('1')).coords.reshape(-1,3)
         GL.glColor3f(*black)
         for i in range(3):
             #GL.glColor(*self.color[i])
@@ -218,7 +220,7 @@ class TriadeActor(Actor):
         GL.glEnd()
         # Coord plane triangles of size 0.5
         GL.glBegin(GL.GL_TRIANGLES)
-        pts = Formex(mpattern('16')).scale(0.5).f.reshape(-1,3)
+        pts = Formex(mpattern('16')).scale(0.5).coords.reshape(-1,3)
         for i in range(3):
             pts = pts.rollAxes(1)
             GL.glColor(*self.color[i])
@@ -342,6 +344,9 @@ class PlaneActor(Actor):
                 drawGridPlanes(self.x0,self.x1,nx)
 
 
+
+
+
 ###########################################################################
 
 quadratic_curve_ndiv = 8
@@ -391,21 +396,30 @@ class GeomActor(Actor):
         """
         Actor.__init__(self)
 
-        if isinstance(data,GeomActor):
-            self.coords,self.elems,self.eltype = data.coords,data.elems,data.eltype
+        # Store a reference to the drawn object
+        self.object = data
+        
+        if isinstance(data,GeomActor) or isinstance(data,Mesh):
+            self.coords = data.coords
+            self.elems = data.elems
+            self.eltype = data.eltype
+            
         elif isinstance(data,Formex):
-            self.coords = data.f
+            self.coords = data.coords
             self.elems = None
             self.eltype = data.eltype
+
         else:
             self.coords = data
             self.elems = elems
             self.eltype = eltype
             
-        if self.elems is None:
-            self.atype = 'Formex'
-        else:
-            self.atype = 'Mesh'
+        ## if self.elems is None:
+        ##     self.atype = 'Formex'
+        ## else:
+        ##     self.atype = 'Mesh'
+        ##     if isinstance(data,TriSurface):
+        ##         self.atype = 'TriSurface'
             
         self.mode = mode
         self.setLineWidth(linewidth)
@@ -422,6 +436,9 @@ class GeomActor(Actor):
         self.list = None
 
 
+    def getType(self):
+        return self.object.__class__
+
     def nplex(self):
         return self.shape()[1]
 
@@ -437,27 +454,20 @@ class GeomActor(Actor):
     def npoints(self):
         return self.vertices().shape[0]
 
+    def nedges(self):
+        # This is needed to be able to pick edges!!
+        try:
+            return self.object.nedges()
+        except:
+            try:
+                return self.object.getEdges().shape[0]
+            except:
+                return 0
+
     def vertices(self):
         """Return the vertives as a 2-dim array."""
         return self.coords.reshape(-1,3)
 
-    # This should probably go th Mesh
-    def nedges(self):
-        try:
-            el = getattr(elements,self.eltype.capitalize())
-            return self.nelems() * len(el.edges)
-        except:
-            return 0
-
-    def edges(self):
-        try:
-            el = getattr(elements,self.eltype.capitalize())
-            edg = asarray(el.edges)
-            edges = self.elems[:,edg]
-            return edges.reshape(-1,2)
-        except:
-            return None
-
  
     def setColor(self,color=None,colormap=None):
         """Set the color of the Actor."""
@@ -472,8 +482,6 @@ class GeomActor(Actor):
         """Set the Actors alpha value."""
         self.alpha = float(alpha)
         self.trans = self.alpha < 1.0
-        #if self.trans:
-        #    GD.debug("Setting Actor's ALPHA value to %f" % alpha)
 
 
     def setMarkSize(self,marksize):
@@ -528,9 +536,13 @@ class GeomActor(Actor):
 
         if mode.endswith('wire'):
             self.drawGL(mode=mode[:-4],color=color,colormap=colormap,alpha=alpha)
+            # draw the lines without lights
+            save = GD.canvas.hasLight()
+            GD.canvas.glLight(False)
             self.drawGL(mode='wireframe',color=asarray(black),colormap=None)
+            GD.canvas.glLight(save)
             return
-
+                            
         ############# set drawing attributes #########
         if alpha is None:
             alpha = self.alpha
@@ -539,12 +551,14 @@ class GeomActor(Actor):
             color,colormap = self.color,self.colormap
         else:
             color,colormap = saneColorSet(color,colormap,self.nelems())
-        
+
+
         if color is None:  # no color
             pass
         
         elif color.dtype.kind == 'f' and color.ndim == 1:  # single color
             GL.glColor(append(color,alpha))
+                
             color = None
 
         elif color.dtype.kind == 'i': # color index
@@ -556,6 +570,21 @@ class GeomActor(Actor):
         if self.linewidth is not None:
             GL.glLineWidth(self.linewidth)
 
+        if mode.startswith('smooth'):
+            if hasattr(self,'specular'):
+                fill_mode = GL.GL_FRONT
+                import colors
+                if color:
+                    spec = color * self.specular# *  GD.canvas.specular
+                    spec = append(spec,1.)
+                else:
+                    spec = colors.GREY(self.specular)# *  GD.canvas.specular
+                #print self.coords.shape
+                #print "SETTING SPECULAR to %s" % str(spec)
+                GL.glMaterialfv(fill_mode,GL.GL_SPECULAR,spec)
+                GL.glMaterialfv(fill_mode,GL.GL_EMISSION,spec)
+                GL.glMaterialfv(fill_mode,GL.GL_SHININESS,self.specular)
+
         ################## draw the geometry #################
         nplex = self.nplex()
         if nplex == 1:
@@ -566,7 +595,11 @@ class GeomActor(Actor):
                 drawPoints(self.coords,color,alpha,self.marksize)
                 
         elif nplex == 2:
+            #save = GD.canvas.hasLight()
+            #GD.canvas.glLight(False)
             drawLines(self.coords,self.elems,color)
+            #GD.canvas.glLight(save)
+
         
         elif self.eltype == 'curve' and nplex == 3:
             GD.debug("DRAWING WITH drawQuadraticCurves")
@@ -577,6 +610,7 @@ class GeomActor(Actor):
             drawNurbsCurves(self.coords,color)
             
         elif self.eltype is None:
+            # polygons
             if mode=='wireframe' :
                 drawPolyLines(self.coords,self.elems,color)
             else:
@@ -590,8 +624,13 @@ class GeomActor(Actor):
             if mode=='wireframe' :
                 drawEdges(self.coords,self.elems,el.edges,color)    
             else:
-                drawFaces(self.coords,self.elems,el.faces,mode,color,alpha)
-    
+                if hasattr(el,'drawfaces'):
+                    faces = el.drawfaces
+                else:
+                    faces = el.faces
+                drawFaces(self.coords,self.elems,faces,mode,color,alpha)
+
+   
 
     def pickGL(self,mode):
         """ Allow picking of parts of the actor.
@@ -602,7 +641,7 @@ class GeomActor(Actor):
             pickPolygons(self.coords,self.elems)
 
         elif mode == 'edge':
-            edges = self.edges()
+            edges = self.object.getEdges()
             if edges is not None:
                 pickPolygons(self.coords,edges)
                 
@@ -626,105 +665,4 @@ class GeomActor(Actor):
         return GeomActor(x,e,eltype=self.eltype)
 
 
-#############################################################################
-
-
-class TriSurfaceActor(Actor,TriSurface):
-    """Draws a triangulated surface specified by points and connectivity."""
-
-    def __init__(self,S,color=None,colormap=None,bkcolor=None,bkcolormap=None,linewidth=None,alpha=1.0,mode=None):
-        
-        Actor.__init__(self)
-        self.atype = 'TriSurface'
-        TriSurface.__init__(self,S.coords,S.edges,S.faces)
-        
-        self.setLineWidth(linewidth)
-        self.setColor(color,colormap)
-        self.setBkColor(bkcolor,bkcolormap)
-        self.setAlpha(alpha)
-
-        self.list = None
-
-    nelems = TriSurface.nelems
-    npoints = TriSurface.npoints
-
-    def setColor(self,color=None,colormap=None):
-        """Set the color of the Actor."""
-        self.color,self.colormap = saneColorSet(color,colormap,(self.nelems(),3)) 
-
-
-    def setBkColor(self,color=None,colormap=None):
-        """Set the backside color of the Actor."""
-        self.bkcolor,self.bkcolormap = saneColorSet(color,colormap,(self.nelems(),3))
-
-    def setAlpha(self,alpha):
-        """Set the Actors alpha value."""
-        self.alpha = float(alpha)
-        self.trans = self.alpha < 1.0
-
-
-    # override the defaults
-    # (no longer needed as we removed the defaults from Drawable)
-    #bbox = TriSurface.bbox
-    #nelems = TriSurface.nelems
-
-
-    def drawGL(self,mode='wireframe',color=None,colormap=None,alpha=None):
-        """Draw the surface."""
-
-        if mode.endswith('wire'):
-            self.drawGL(mode='wireframe',color=asarray(black),colormap=None)
-            self.drawGL(mode=mode[:-4],color=color,colormap=colormap,alpha=alpha)
-            return
-
-        if alpha is None:
-            alpha = self.alpha           
-
-        if color is None:  
-            color,colormap = self.color,self.colormap
-        else:
-            color,colormap = saneColorSet(color,colormap,self.nelems())
-        
-        if color is None:  # no color
-            pass
-        
-        elif color.dtype.kind == 'f' and color.ndim == 1:  # single color
-            GL.glColor(append(color,alpha))
-            color = None
-
-        elif color.dtype.kind == 'i': # color index
-            color = colormap[color]
-
-        else: # a full color array : use as is
-            pass
-
-        if self.linewidth is not None:
-            GL.glLineWidth(self.linewidth)
-
-        t = timer.Timer()
-        if mode=='wireframe' :
-            rev = reverseIndex(self.faces)
-            if color is not None:
-                color = color[rev[:,-1]]
-            drawLines(self.coords,self.edges,color)
-        else:
-            self.refresh()
-            drawPolygons(self.coords,self.elems,mode,color,alpha)
-        GD.debug("Drawing time: %s seconds" % t.seconds())
-    
-
-    def pickGL(self,mode):
-        """ Allow picking of parts of the actor.
-
-        mode can be 'element', 'edge' or 'point'
-        """
-        if mode == 'element':
-            self.refresh()
-            pickPolygons(self.coords,self.elems)
-        elif mode == 'edge':
-            pickPolygons(self.coords,self.edges)
-        elif mode == 'point':
-            pickPoints(self.coords)
-
-
 # End
