@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: camera.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: camera.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -22,7 +22,7 @@
 ##  You should have received a copy of the GNU General Public License
 ##  along with this program.  If not, see http://www.gnu.org/licenses/.
 ##
-"""camera 0.1 (C) Benedict Verhegghe"""
+"""OpenGL camera handling"""
 
 from numpy import *
 
@@ -129,6 +129,7 @@ class Camera:
 
     def __init__(self,center=[0.,0.,0.], long=0., lat=0., twist=0., dist=1.):
         """Create a new camera at position (0,0,0) looking along the -z axis"""
+        self.locked = False
         self.setCenter(*center)
         self.setRotation(long,lat,twist)
         self.setDist(dist)
@@ -139,6 +140,8 @@ class Camera:
         self.keep_aspect = True
         self.setPerspective(True)
         self.viewChanged = True
+        self.tracking = False
+        self.m = self.p = self.v = None
 
     # Use only these access functions to make implementation changes easier
         
@@ -152,11 +155,16 @@ class Camera:
         """Return the camera distance."""
         return self.dist
 
+    def lock(self,onoff=True):
+        self.locked = onoff
+        print("Camera locked is %s" % self.locked)
+
 
     def setCenter(self,x,y,z):
         """Set the center of the camera in global cartesian coordinates."""
-        self.ctr = [x,y,z]
-        self.viewChanged = True
+        if not self.locked:
+            self.ctr = [x,y,z]
+            self.viewChanged = True
 
 
     def setAngles(self,angles):
@@ -167,28 +175,32 @@ class Camera:
             - a named view corresponding to angles in view_angles
             - None
         """
-        if type(angles) is str:
-            angles = view_angles.get(angles)
-        if angles is None:
-            return
-        self.setRotation(*angles)
+        if not self.locked:
+            if type(angles) is str:
+                angles = view_angles.get(angles)
+            if angles is None:
+                return
+            self.setRotation(*angles)
             
 
     def setRotation(self,long,lat,twist=0):
         """Set the rotation matrix of the camera from three angles."""
-        GL.glMatrixMode(GL.GL_MODELVIEW)
-        GL.glLoadIdentity()
-        GL.glRotatef(-twist % 360, 0.0, 0.0, 1.0)
-        GL.glRotatef(lat % 360, 1.0, 0.0, 0.0)
-        GL.glRotatef(-long % 360, 0.0, 1.0, 0.0)
-        self.rot = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
-        self.viewChanged = True
+        if not self.locked:
+            GL.glMatrixMode(GL.GL_MODELVIEW)
+            GL.glLoadIdentity()
+            GL.glRotatef(-twist % 360, 0.0, 0.0, 1.0)
+            GL.glRotatef(lat % 360, 1.0, 0.0, 0.0)
+            GL.glRotatef(-long % 360, 0.0, 1.0, 0.0)
+            self.rot = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
+            self.viewChanged = True
+
 
     def setDist(self,dist):
         """Set the distance."""
-        if dist > 0.0 and dist != inf:
-            self.dist = dist
-            self.viewChanged = True
+        if not self.locked:
+            if dist > 0.0 and dist != inf:
+                self.dist = dist
+                self.viewChanged = True
 
 
     def report(self):
@@ -215,9 +227,11 @@ class Camera:
         The front and back clipping planes may need adjustment after
         a dolly operation.
         """
-        self.setDist(self.getDist() * val)
-        #print("DIST %s" % self.dist)
-        self.viewChanged = True
+        if not self.locked:
+            self.setDist(self.getDist() * val)
+            #print("DIST %s" % self.dist)
+            self.viewChanged = True
+
         
     def pan(self,val,axis=0):
         """Rotate the camera around axis through its eye. 
@@ -229,14 +243,16 @@ class Camera:
         alias tilt is created.
         For axis = 2 the operation is equivalent to the rotate operation.
         """
-        if axis==0 or axis ==1:
-            pos = self.getPosition()
-            self.eye[axis] = (self.eye[axis] + val) % 360
-            center = diff(pos,sphericalToCartesian(self.eye))
-            self.setCenter(*center)
-        elif axis==2:
-            self.twist = (self.twist + val) % 360
-        self.viewChanged = True
+        if not self.locked:
+            if axis==0 or axis ==1:
+                pos = self.getPosition()
+                self.eye[axis] = (self.eye[axis] + val) % 360
+                center = diff(pos,sphericalToCartesian(self.eye))
+                self.setCenter(*center)
+            elif axis==2:
+                self.twist = (self.twist + val) % 360
+            self.viewChanged = True
+
 
     def tilt(self,val):
         """Rotate the camera up/down around its own horizontal axis.
@@ -246,8 +262,10 @@ class Camera:
         A positive value tilts the camera up, shifting the scene down.
         The value is specified in degrees.
         """
-        self.pan(val,1)
-        self.viewChanged = True
+        if not self.locked:
+            self.pan(val,1)
+            self.viewChanged = True
+
 
     def move(self,dx,dy,dz):
         """Move the camera over translation (dx,dy,dz) in global coordinates.
@@ -255,8 +273,9 @@ class Camera:
         The center of the camera is moved over the specified translation
         vector. This has the effect of moving the scene in opposite direction.
         """
-        x,y,z = self.ctr
-        self.setCenter(x+dx,y+dy,z+dz)
+        if not self.locked:
+            x,y,z = self.ctr
+            self.setCenter(x+dx,y+dy,z+dz)
 
 ##    def truck(self,dx,dy,dz):
 ##        """Move the camera translation vector in local coordinates.
@@ -277,70 +296,83 @@ class Camera:
 ##        self.viewChanged = True
 
 
-        
-    def setMatrix(self):
-        """Set the ModelView matrix from camera parameters.
-
-        These are the transformations applied on the model space.
-        Rotations and translations need be taken negatively.
-        """
-        # The operations on the model space
-        # arguments should be taken negative and applied in backwards order
-        GL.glMatrixMode(GL.GL_MODELVIEW)
-        GL.glLoadIdentity()
-        #printModelviewMatrix("Identity:\n%s")
-        # translate over camera distance
-        GL.glTranslate(0,0,-self.dist)
-        #printModelviewMatrix("Camera distance:\n%s")
-        # rotate
-        GL.glMultMatrixf(self.rot)
-        #printModelviewMatrix("Rotation:\n%s")
-        # translate to center
-        dx,dy,dz = self.getCenter()
-        GL.glTranslatef(-dx,-dy,-dz)
-        #printModelviewMatrix("Translation:\n%s")
-
-
     def lookAt(self,eye,center,up):
-        GL.glMatrixMode(GL.GL_MODELVIEW)
-        GL.glLoadIdentity()
-        GLU.gluLookAt(*concatenate([eye,center,up]))
-        self.saveMatrix()
+        if not self.locked:
+            GL.glMatrixMode(GL.GL_MODELVIEW)
+            GL.glLoadIdentity()
+            GLU.gluLookAt(*concatenate([eye,center,up]))
+            self.saveModelView()
+
 
     def rotate(self,val,vx,vy,vz):
         """Rotate the camera around current camera axes."""
-        GL.glMatrixMode(GL.GL_MODELVIEW)
-        self.saveMatrix()
-        GL.glLoadIdentity()
-        GL.glTranslatef(0,0,-self.dist)
-        GL.glRotatef(val,vx,vy,vz)
-        GL.glMultMatrixf(self.rot)
-        dx,dy,dz = self.getCenter()
-        GL.glTranslatef(-dx,-dy,-dz)
-        self.saveMatrix()
+        if not self.locked:
+            GL.glMatrixMode(GL.GL_MODELVIEW)
+            self.saveModelView()
+            GL.glLoadIdentity()
+            GL.glTranslatef(0,0,-self.dist)
+            GL.glRotatef(val,vx,vy,vz)
+            GL.glMultMatrixf(self.rot)
+            dx,dy,dz = self.getCenter()
+            GL.glTranslatef(-dx,-dy,-dz)
+            self.saveModelView()
 
-    def saveMatrix (self):
+
+    def saveModelView (self):
         """Save the ModelView matrix."""
-        self.m = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
-        self.rot = copy.deepcopy(self.m)
-        self.trl = copy.deepcopy(self.rot[3,0:3])
-        #print("Translation: %s" % self.trl)
-        self.rot[3,0:3] = [0.,0.,0.]
+        if not self.locked:
+            self.m = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
+            self.rot = copy.deepcopy(self.m)
+            self.trl = copy.deepcopy(self.rot[3,0:3])
+            #print("Translation: %s" % self.trl)
+            self.rot[3,0:3] = [0.,0.,0.]
+            #print "Rotation: %s" % self.rot
 
-    def loadMatrix (self):
+
+        
+    def setModelView(self):
+        """Set the ModelView matrix from camera parameters.
+
+        """
+        if not self.locked:
+            # The camera operations are applied on the model space
+            # Arguments should be taken negative and applied in backwards order
+            GL.glMatrixMode(GL.GL_MODELVIEW)
+            GL.glLoadIdentity()
+            # translate over camera distance
+            GL.glTranslate(0,0,-self.dist)
+            # rotate according to current rotation matrix
+            GL.glMultMatrixf(self.rot)
+            # translate to center
+            dx,dy,dz = self.getCenter()
+            GL.glTranslatef(-dx,-dy,-dz)
+
+
+    def loadModelView (self):
         """Load the saved ModelView matrix."""
-        GL.glMatrixMode(GL.GL_MODELVIEW)
-        if self.viewChanged:
-            self.setMatrix()
-            self.saveMatrix()
-            self.viewChanged = False
-        else:
-            GL.glLoadMatrixf(self.m)
+        if not self.locked:
+            GL.glMatrixMode(GL.GL_MODELVIEW)
+            if self.viewChanged:
+                self.setModelView()
+                self.saveModelView()
+                self.viewChanged = False
+            else:
+                GL.glLoadMatrixf(self.m)
+
+
+    def loadCurrentRotation (self):
+        """Load the current ModelView matrix with translations canceled out."""
+        rot = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
+        rot[3,0:3] = [0.,0.,0.]
+        GL.glLoadMatrixf(rot)
+
  
     def translate(self,vx,vy,vz,local=True):
-        if local:
-            vx,vy,vz = self.toWorld([vx,vy,vz,1])
-        self.move(-vx,-vy,-vz)
+        if not self.locked:
+            if local:
+                vx,vy,vz = self.toWorld([vx,vy,vz,1])
+            self.move(-vx,-vy,-vz)
+
       
     def transform(self,v):
         """Transform a vertex using the currently saved Modelview matrix."""
@@ -349,6 +381,7 @@ class Camera:
         v = multiply([v],self.m)[0]
         return [ a/v[3] for a in v[0:3] ]
 
+
     def toWorld(self,v,trl=False):
         """Transform a vertex from camera to world coordinates.
 
@@ -540,6 +573,40 @@ class Camera:
             GL.glMatrixMode(GL.GL_MODELVIEW)     
 
 
+    #### global manipulation ###################
+
+    def set3DMatrices(self):
+        self.loadProjection()
+        self.loadModelView()
+        # this is saved by loadModelView
+        #self.m = GL.glGetDoublev(GL.GL_MODELVIEW_MATRIX)
+        ##!!! self.p and self.v should be saved as we do with self.m
+        self.p = GL.glGetDoublev(GL.GL_PROJECTION_MATRIX)
+        self.v = GL.glGetIntegerv(GL.GL_VIEWPORT)
+
+
+    def project(self,x,y,z):
+        "Map the object coordinates (x,y,z) to window coordinates."""
+        self.set3DMatrices()
+        return GLU.gluProject(x,y,z,self.m,self.p,self.v)
+
+
+    def unProject(self,x,y,z):
+        "Map the window coordinates (x,y,z) to object coordinates."""
+        self.set3DMatrices()
+        return GLU.gluUnProject(x,y,z,self.m,self.p,self.v)
+
+
+    def setTracking(self,onoff=True):
+        """Enable/disable coordinate tracking using the camera"""
+        if onoff:
+            self.tracking = True
+            self.set3DMatrices()
+        else:
+            self.tracking = False
+        
+
+
 if __name__ == "__main__":
     
     from OpenGL.GLUT import *
@@ -555,7 +622,7 @@ if __name__ == "__main__":
         GL.glColor3f (1.0, 1.0, 1.0)
         GL.glLoadIdentity ()             # clear the matrix
         cam.loadProjection()
-        cam.loadMatrix()
+        cam.loadModelView()
         glutWireCube (1.0)
         GL.glFlush ()
 
