@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: cameraMenu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: cameraMenu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -91,6 +91,10 @@ def dollyOut():
     GD.canvas.update()   
 def report():
     print(GD.canvas.camera.report())
+def lock():
+    GD.canvas.camera.lock()
+def unlock():
+    GD.canvas.camera.lock(False)
 
 
 MenuData = [
@@ -121,6 +125,8 @@ MenuData = [
         (_('Rotate &ClockWise'),twistRight),
         (_('Rotate &CCW'),twistLeft),
         ]),
+    (_('&Lock'),lock), 
+    (_('&Unlock'),unlock), 
     ('---',None),
     (_('&Report'),report), 
     ]
