@@ -1,6 +1,6 @@
-# $Id: canvas.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: canvas.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -23,13 +23,14 @@
 ##
 """This implements an OpenGL drawing widget for painting 3D scenes."""
 
-import pyformex as GD
+import pyformex as pf
 from coords import tand
 
 from numpy import *
 from OpenGL import GL,GLU
 
 from formex import length
+from drawable import saneColor
 import colors
 import camera
 import actors
@@ -41,7 +42,7 @@ import utils
 def gl_pickbuffer():
     "Return a list of the 2nd numbers in the openGL pick buffer."
     buf = GL.glRenderMode(GL.GL_RENDER)
-    #GD.debugt("translate getpickbuf")
+    #pf.debugt("translate getpickbuf")
     return asarray([ r[2] for r in buf ])
 
 
@@ -63,12 +64,13 @@ def glFill():
 def glLine():
     GL.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_LINE)
 
-## def glLight(onoff):
-##     """Toggle lights on/off."""
-##     if onoff:
-##         GL.glEnable(GL.GL_LIGHTING)
-##     else:
-##         GL.glDisable(GL.GL_LIGHTING)
+def glLineSmooth(onoff):
+    if onoff is True:
+        GL.glEnable(GL.GL_LINE_SMOOTH)
+        GL.glHint(GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST)
+    elif onoff is False:
+        GL.glDisable(GL.GL_LINE_SMOOTH)
+
 
 def glSmooth():
     """Enable smooth shading"""
@@ -76,21 +78,88 @@ def glSmooth():
 def glFlat():
     """Disable smooth shading"""
     GL.glShadeModel(GL.GL_FLAT)
+    
 
-def glCulling():
-    """Enable culling"""
-    GL.glEnable(GL.GL_CULL_FACE)
+def onOff(onoff):
+    """Convert On/Off strings to a boolean"""
+    if type(onoff) is str:
+        return (onoff.lower() == 'on')
+    else:
+        if onoff:
+            return True
+        else:
+            return False
+
+
+def glEnable(facility,onoff):
+    """Enable/Disable an OpenGL facility, depending on onoff value
+
+    facility is an OpenGL facility.
+    onoff can be True or False to enable, resp. disable the facility, or
+    None to leave it unchanged.
+    """
+    pf.debug("%s: %s" % (facility,onoff))
+    if onOff(onoff):
+        pf.debug("ENABLE")
+        GL.glEnable(facility)
+    else:
+        pf.debug("DISABLE")
+        GL.glDisable(facility)
+        
+
+def glCulling(onoff=True):
+    glEnable(GL.GL_CULL_FACE,onoff)
 def glNoCulling():
-    """Disable culling"""
-    GL.glDisable(GL.GL_CULL_FACE)
+    glCulling(False)
 
+def glLighting(onoff):
+    #print onoff
+    glEnable(GL.GL_LIGHTING,onoff)
+
+
+def glPolygonFillMode(mode):
+    if type(mode) is str:
+        mode = mode.lower()
+        if mode == 'Front and Back':
+            glBothFill()
+        elif mode == 'Front':
+            glFrontFill()
+        elif mode == 'Back':
+            glBackFill()
+
+            
+def glPolygonMode(mode):
+    if type(mode) is str:
+        mode = mode.lower()
+        if mode == 'fill':
+            glFill()
+        elif mode == 'line':
+            glLine()
+    
+
+def glShadeModel(model):
+    if type(model) is str:
+        model = model.lower()
+        if model == 'smooth':
+            glSmooth()
+        elif model == 'flat':
+            glFlat()
+        
+def glSettings(settings):
+    pf.debug("GL SETTINGS: %s" % settings)
+    glCulling(settings.get('Culling',None))
+    glLighting(settings.get('Lighting',None))
+    glShadeModel(settings.get('Shading',None))
+    glLineSmooth(onOff(settings.get('Line Smoothing',None)))
+    glPolygonFillMode(settings.get('Polygon Fill',None))
+    glPolygonMode(settings.get('Polygon Mode',None))
+    pf.canvas.update()
 
     
 class ActorList(list):
 
-    def __init__(self,canvas,atype):
+    def __init__(self,canvas):
         self.canvas = canvas
-        self.atype = atype
         list.__init__(self)
         
     def add(self,actor):
@@ -99,16 +168,8 @@ class ActorList(list):
 
     def delete(self,actor):
         """Remove an actor from an actorlist."""
-##         if self.atype == 'annotation':
-##             print("DELETE")
-##             print(self)
-##             print(actor)
         if actor in self:
-##             if self.atype == 'annotation':
-##                 print("REMOVE %s" % actor)
             self.remove(actor)
-##             if self.atype == 'annotation':
-##                 print(self)
 
     def redraw(self):
         """Redraw (some) actors in the scene.
@@ -130,14 +191,14 @@ class CanvasSettings(object):
     """A collection of settings for an OpenGL Canvas."""
 
     default = dict(
+        pointsize = 4.0,
         linewidth = 1.0,
         bgcolor = colors.mediumgrey,
         bgcolor2 = None,
         fgcolor = colors.black,
-        slcolor = colors.red,     # color for selected items
-        transparency = 1.0,       # opaque
-        # colormap for mapping property values
-        propcolors = [
+        slcolor = colors.magenta,     # highlight color
+        transparency = 0.5,           # half transparent
+        colormap = [
             colors.black, colors.red, colors.green, colors.blue,
             colors.cyan, colors.magenta, colors.yellow, colors.white
             ],
@@ -155,16 +216,27 @@ class CanvasSettings(object):
                     ok[k] = None
                 else:
                     ok[k] = colors.GLColor(v)
-            elif k == 'propcolors':
+            elif k == 'colormap':
                 ok[k] = map(colors.GLColor,v)
             elif k == 'linewidth':
                 ok[k] = float(v)
+            elif k == 'pointsize':
+                ok[k] = float(v)
+            elif k == 'transparency':
+                ok[k] = max(min(float(v),1.0),0.0)
+            elif k == 'colormap':
+                ok[k] = saneColor(colormap)
         return ok
+
         
     def __init__(self,dict={}):
         """Create a new set of CanvasSettings, possibly changing defaults."""
         self.reset(dict)
 
+
+    def items(self):
+        return self.__dict__.items()
+    
     def reset(self,dict={}):
         """Reset to defaults
 
@@ -192,15 +264,18 @@ class CanvasSettings(object):
 
 class Light(object):
 
-    def __init__(self,nr,*args,**kargs):
+    def __init__(self,nr,**kargs):
         self.light = GL.GL_LIGHT0 + (nr % GL.GL_MAX_LIGHTS)
-        self.set(**kargs)
+        self.set(ambient=0.5,diffuse=0.5,specular=0.5,position=[0.,0.,1.,0.],enabled=False)
 
-    def set(self,ambient=0.5,diffuse=0.5,specular=0.5,position=(0.,0.,1.,0.)):
-        self.ambient = colors.GREY(ambient)
-        self.diffuse = colors.GREY(diffuse)
-        self.specular = colors.GREY(specular)
-        self.position = position
+    def set(self,**kargs):
+        for k in kargs:
+            self.set_value(k,kargs[k])
+
+    def set_value(self,key,value):
+        if key in [ 'ambient','diffuse','specular' ]:
+            value = colors.GREY(value)
+        setattr(self,key,value)
 
     def enable(self):
         GL.glLightfv(self.light,GL.GL_POSITION,self.position)
@@ -212,6 +287,42 @@ class Light(object):
     def disable(self):
         GL.glDisable(self.light)
 
+    def __str__(self):
+        return """LIGHT %s:
+    ambient color:  %s
+    diffuse color:  %s
+    specular color: %s
+    position: %s
+""" % (self.light-GL.GL_LIGHT0,self.ambient,self.diffuse,self.specular,self.position)
+    
+
+class Lights(object):
+    """An array of OpenGL lights.
+
+    """
+    def __init__(self,nlights):
+        self.lights = [ Light(i) for i in range(nlights) ]
+
+    def set_value(self,i,key,value):
+        """Set an attribute of light i"""
+        self.lights[i].set_value(key,value)
+        
+    def set(self,i,**kargs):
+        """Set all attributes of light i"""
+        self.lights[i].set(**kargs)
+
+    def enable(self):
+        """Enable the lights"""
+        [ i.enable() for i in self.lights if i.enabled ]
+
+    def disable(self):
+        [ i.disable() for i in self.lights ]
+
+    def __str__(self):
+        return ''.join([i.__str__() for i in self.lights if i.enabled ])
+
+
+
 
 ##################################################################
 #
@@ -221,23 +332,33 @@ class Canvas(object):
     """A canvas for OpenGL rendering."""
 
     rendermodes = ['wireframe','flat','flatwire','smooth','smoothwire',
-                   'smooth-avg']
+                   'smooth_avg']
+
+    light_model = {
+        'ambient': GL.GL_AMBIENT,
+        'diffuse': GL.GL_DIFFUSE,
+        'ambient and diffuse': GL.GL_AMBIENT_AND_DIFFUSE,
+        'emission': GL.GL_EMISSION,
+        'specular': GL.GL_SPECULAR,
+        }
+    
 
     def __init__(self):
         """Initialize an empty canvas with default settings."""
-        self.actors = ActorList(self,'actor')
-        self.highlights = ActorList(self,'highlight')
-        self.annotations = ActorList(self,'annotation')
-        self.decorations = ActorList(self,'decoration')
+        self.actors = ActorList(self)
+        self.highlights = ActorList(self)
+        self.annotations = ActorList(self)
+        self.decorations = ActorList(self)
         self.triade = None
         self.background = None
         self.bbox = None
         self.resetLighting()
+        self.lights = Lights(8)
         self.resetLights()
         self.setBbox()
-        self.settings = CanvasSettings(GD.cfg['canvas'])
+        self.settings = CanvasSettings()
         self.mode2D = False
-        self.rendermode = 'wireframe'
+        self.rendermode = pf.cfg['render/mode']
         self.polygonfill = False
         self.lighting = True
         self.avgnormals = False
@@ -248,32 +369,32 @@ class Canvas(object):
         self.view_angles = camera.view_angles
         self.cursor = None
         self.focus = False
-        GD.debug("Canvas Setting:\n%s"% self.settings)
+        pf.debug("Canvas Setting:\n%s"% self.settings)
 
 
     def Size(self):
         return self.width(),self.height()
     
 
-    def glMatSpec(self):
-        GL.glMaterialfv(fill_mode,GL.GL_SPECULAR,colors.GREY(self.specular))
-        GL.glMaterialfv(fill_mode,GL.GL_EMISSION,colors.GREY(self.emission))
-        GL.glMaterialfv(fill_mode,GL.GL_SHININESS,self.shininess)
-        GL.glColorMaterial(fill_mode,GL.GL_AMBIENT_AND_DIFFUSE)
-        GL.glEnable(GL.GL_COLOR_MATERIAL)
-
     def glLightSpec(self):
         GL.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT,colors.GREY(self.ambient))
         GL.glLightModeli(GL.GL_LIGHT_MODEL_TWO_SIDE, 1)
         GL.glLightModeli(GL.GL_LIGHT_MODEL_LOCAL_VIEWER, 0)
-        if GD.canvas:
-            GL.glMatrixMode(GL.GL_MODELVIEW)
-            GL.glPushMatrix()
-            GL.glLoadIdentity()
-            for l in GD.canvas.lights:
-                l.enable()
-            GL.glPopMatrix()
-        self.glMatSpec()
+        GL.glMatrixMode(GL.GL_MODELVIEW)
+        GL.glPushMatrix()
+        GL.glLoadIdentity()
+        self.lights.enable()
+        GL.glPopMatrix()
+        
+        GL.glMaterialfv(fill_mode,GL.GL_AMBIENT_AND_DIFFUSE,colors.GREY(self.ambient))
+        GL.glMaterialfv(fill_mode,GL.GL_SPECULAR,colors.GREY(self.specular))
+        GL.glMaterialfv(fill_mode,GL.GL_EMISSION,colors.GREY(self.emission))
+        GL.glMaterialfv(fill_mode,GL.GL_SHININESS,self.shininess)
+
+        # What component to drive by color commands
+        GL.glColorMaterial(fill_mode,self.light_model[self.lightmodel])
+        GL.glEnable(GL.GL_COLOR_MATERIAL)
+
 
     def glLight(self,onoff):
         """Toggle lights on/off."""
@@ -283,23 +404,32 @@ class Canvas(object):
         else:
             GL.glDisable(GL.GL_LIGHTING)
 
+
+    def hasLight(self):
+        """Return the status of the lighting."""
+        return GL.glIsEnabled(GL.GL_LIGHTING)
+        
+
     def resetDefaults(self,dict={}):
         """Return all the settings to their default values."""
         self.settings.reset(dict)
+        self.resetLighting()
         self.resetLights()
 
+
     def resetLighting(self):
-        self.ambient = GD.cfg['render/ambient']
-        self.specular = GD.cfg['render/specular']
-        self.emission = GD.cfg['render/emission']
-        self.shininess = GD.cfg['render/shininess']
+        self.lightmodel = pf.cfg['render/lightmodel']
+        self.ambient = pf.cfg['render/material/ambient']
+        self.specular = pf.cfg['render/material/specular']
+        self.emission = pf.cfg['render/material/emission']
+        self.shininess = pf.cfg['render/material/shininess']
+
         
     def resetLights(self):
-        self.lights = []
         for i in range(8):
-            light = GD.cfg.get('render/light%d' % i, None)
+            light = pf.cfg.get('render/light%d' % i, None)
             if light is not None:
-                self.lights.append(Light(i,light))
+                self.lights.set(i,**light)
 
 
     def setRenderMode(self,rm):
@@ -307,12 +437,10 @@ class Canvas(object):
 
         This changes the rendermode and redraws everything with the new mode.
         """
-        #GD.debug("Changing Render Mode to %s" % rm)
         if rm != self.rendermode:
             if rm not in Canvas.rendermodes:
                 rm = Canvas.rendermodes[0]
             self.rendermode = rm
-            #GD.debug("Redrawing with mode %s" % self.rendermode)
             self.glinit(self.rendermode)
             self.redrawAll()
 
@@ -327,10 +455,10 @@ class Canvas(object):
     def setAveragedNormals(self,mode):
         self.avgnormals = mode
         change = (self.rendermode == 'smooth' and self.avgnormals) or \
-                 (self.rendermode == 'smooth-avg' and not self.avgnormals)
+                 (self.rendermode == 'smooth_avg' and not self.avgnormals)
         if change:
             if self.avgnormals:
-                self.rendermode = 'smooth-avg'
+                self.rendermode = 'smooth_avg'
             else:
                 self.rendermode = 'smooth'
             self.actors.redraw()
@@ -341,6 +469,10 @@ class Canvas(object):
         """Set the linewidth for line rendering."""
         self.settings.linewidth = float(lw)
 
+    def setPointSize(self,sz):
+        """Set the size for point drawing."""
+        self.settings.pointsize = float(sz)
+
 
     def setBgColor(self,color1,color2=None):
         """Set the background color.
@@ -351,7 +483,7 @@ class Canvas(object):
         """
         self.settings.bgcolor = colors.GLColor(color1)
         if color2 is None:
-            GD.debug("Clearing twocolor background")
+            pf.debug("Clearing twocolor background")
             self.settings.bgcolor2 = None
             self.background = None
         else:
@@ -400,11 +532,10 @@ class Canvas(object):
                 self.setSlColor(v)
         
         
-
-    
-    def setLight(self,nr,ambient,diffuse,specular,position):
+    def setLightValue(self,nr,key,val):
         """(Re)Define a light on the scene."""
-        self.lights[nr].set(ambient,diffuse,specular,position)
+        self.lights.set_value(nr,key,val)
+        self.lights.enable(nr)
 
     def enableLight(self,nr):
         """Enable an existing light."""
@@ -415,7 +546,7 @@ class Canvas(object):
         self.lights[nr].disable()
 
 
-    def setTriade(self,on=None,size=1.0,pos=[0.0,0.0,0.0]):
+    def setTriade(self,on=None,pos='lb',siz=100):
         """Toggle the display of the global axes on or off.
 
         If on is True, a triade of global axes is displayed, if False it is
@@ -423,21 +554,21 @@ class Canvas(object):
         """
         if on is None:
             on = self.triade is None
-        GD.debug("SETTING TRIADE %s" % on)
+        pf.debug("SETTING TRIADE %s" % on)
         if self.triade:
             self.removeAnnotation(self.triade)
             self.triade = None
         if on:
-            self.triade = actors.TriadeActor(size,pos)
+            self.triade = decors.Triade(pos,siz)
             self.addAnnotation(self.triade)
     
 
     def initCamera(self):
-        #if GD.options.makecurrent:
+        #if pf.options.makecurrent:
         self.makeCurrent()  # we need correct OpenGL context for camera
         self.camera = camera.Camera()
-        #GD.debug("camera.rot = %s" % self.camera.rot)
-        #GD.debug("view angles: %s" % self.view_angles)
+        #pf.debug("camera.rot = %s" % self.camera.rot)
+        #pf.debug("view angles: %s" % self.view_angles)
 
 
     def glinit(self,mode=None):
@@ -462,7 +593,7 @@ class Canvas(object):
                 glSmooth()
                 glFill()
             else:
-                glFlat()
+                #glFlat()
                 glLine()
             self.lighting = False
             self.glLight(False)
@@ -486,6 +617,14 @@ class Canvas(object):
         else:
             raise RuntimeError,"Unknown rendering mode"
 
+
+        if self.rendermode.endswith('wire'):
+            GL.glEnable(GL.GL_POLYGON_OFFSET_FILL)
+            GL.glPolygonOffset(1.0,1.0) 
+        else:
+            GL.glDisable(GL.GL_POLYGON_OFFSET_FILL)
+            
+
     def glupdate(self):
         """Flush all OpenGL commands, making sure the display is updated."""
         GL.glFlush()
@@ -502,6 +641,9 @@ class Canvas(object):
         """Activate the canvas settings in the GL machine."""
         GL.glColor3fv(self.settings.fgcolor)
         GL.glLineWidth(self.settings.linewidth)
+        GL.glPointSize(self.settings.pointsize)
+        if self.rendermode.startswith('smooth'):
+            self.glLightSpec()
 
     
     def setSize (self,w,h):
@@ -522,15 +664,16 @@ class Canvas(object):
         This should e.g. be used when actors are added to the scene,
         or after changing  camera position/orientation or lens.
         """
-        #GD.debugt("UPDATING CURRENT OPENGL CANVAS")
+        #pf.debugt("UPDATING CURRENT OPENGL CANVAS")
         self.makeCurrent()
         self.clear()
         self.glLight(self.lighting)
         
         # decorations are drawn in 2D mode
         self.begin_2D_drawing()
+        
         if self.background:
-            GD.debug("Displaying background")
+            #pf.debug("Displaying background")
             self.background.draw(mode='smooth')
 
         if len(self.decorations) > 0:
@@ -543,18 +686,21 @@ class Canvas(object):
                 ##     self.zoom_2D()
 
         # draw the focus rectangle if more than one viewport
-        if len(GD.GUI.viewports.all) > 1:
+        if len(pf.GUI.viewports.all) > 1 and pf.cfg['gui/showfocus']:
             if self.hasFocus():
                 self.draw_focus_rectangle(2)
             elif self.focus:
                 self.draw_focus_rectangle(1)
+                
         self.end_2D_drawing()
+
+        # start 3D drawing
+        self.camera.set3DMatrices()
         
         # draw the highlighted actors
-        self.camera.loadProjection()
-        self.camera.loadMatrix()
         if self.highlights:
             for actor in self.highlights:
+                self.setDefaults()
                 actor.draw(mode=self.rendermode)
 
         # draw the scene actors
@@ -562,11 +708,13 @@ class Canvas(object):
             opaque = [ a for a in self.actors if not a.trans ]
             transp = [ a for a in self.actors if a.trans ]
             for actor in opaque:
+                self.setDefaults()
                 actor.draw(mode=self.rendermode)
             GL.glEnable (GL.GL_BLEND)
             GL.glDepthMask (GL.GL_FALSE)
             GL.glBlendFunc (GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA)
             for actor in transp:
+                self.setDefaults()
                 actor.draw(mode=self.rendermode)
             GL.glDepthMask (GL.GL_TRUE)
             GL.glDisable (GL.GL_BLEND)
@@ -575,14 +723,11 @@ class Canvas(object):
                 self.setDefaults()
                 actor.draw(mode=self.rendermode)
 
-        # annotations are drawn in 3D space
+        # annotations are decorations drawn in 3D space
         for actor in self.annotations:
             self.setDefaults()
             actor.draw(mode=self.rendermode)
 
-        # decorations are drawn in 2D mode
-        # decoration drawing was moved to the start of the drawing cycle
-
         # make sure canvas is updated
         GL.glFlush()
 
@@ -600,9 +745,9 @@ class Canvas(object):
         It is assumed that you will not try to change/refresh the normal
         3D drawing cycle during this operation.
         """
-        #GD.debug("Start 2D drawing")
+        #pf.debug("Start 2D drawing")
         if self.mode2D:
-            #GD.debug("WARNING: ALREADY IN 2D MODE")
+            #pf.debug("WARNING: ALREADY IN 2D MODE")
             return
         GL.glMatrixMode(GL.GL_MODELVIEW)
         GL.glPushMatrix()
@@ -618,7 +763,7 @@ class Canvas(object):
  
     def end_2D_drawing(self):
         """Cancel the 2D drawing mode initiated by begin_2D_drawing."""
-        #GD.debug("End 2D drawing")
+        #pf.debug("End 2D drawing")
         if self.mode2D:
             GL.glEnable(GL.GL_DEPTH_TEST)    
             GL.glMatrixMode(GL.GL_PROJECTION)
@@ -632,18 +777,19 @@ class Canvas(object):
     def setBbox(self,bbox=None):
         """Set the bounding box of the scene you want to be visible."""
         # TEST: use last actor
-        #GD.debug("BBOX WAS: %s" % self.bbox)
+        #pf.debug("BBOX WAS: %s" % self.bbox)
         if bbox is None:
             if len(self.actors) > 0:
                 bbox = self.actors[-1].bbox()
             else:
                 bbox = [[-1.,-1.,-1.],[1.,1.,1.]]
         bbox = asarray(bbox)
-        if bbox.any() == nan:
-            GD.message("Invalid Bbox: %s" % bbox)
-            return
-        self.bbox = nan_to_num(bbox)
-        #GD.debug("BBOX BECOMES: %s" % self.bbox)
+        try:
+            self.bbox = nan_to_num(bbox)
+        except:
+        # if bbox.any() == nan:
+            pf.message("Invalid Bbox: %s" % bbox)
+        #pf.debug("BBOX BECOMES: %s" % self.bbox)
 
          
     def addActor(self,actor):
@@ -669,8 +815,9 @@ class Canvas(object):
 
     def removeAnnotation(self,actor):
         """Remove an annotation from the 3D scene."""
-##         if actor == self.triade:
-##             self.triade = None
+        if actor == self.triade:
+            pf.debug("REMOVING TRIADE")
+            self.triade = None
         self.annotations.delete(actor)
          
     def addDecoration(self,actor):
@@ -702,8 +849,6 @@ class Canvas(object):
         if actorlist == None:
             actorlist = self.actors[:]
         for actor in actorlist:
-            if actor is self.triade:
-                self.triade == None
             self.removeActor(actor)
         self.setBbox()
         
@@ -717,14 +862,10 @@ class Canvas(object):
 
 
     def removeAnnotations(self,actorlist=None):
-        """Remove all actors in actorlist (default = all) from the scene."""
-        GD.debug("REMOVING ALL ACTORS")
+        """Remove all annotations in actorlist (default = all) from the scene."""
         if actorlist == None:
             actorlist = self.annotations[:]
         for actor in actorlist:
-            if actor == self.triade:
-                GD.debug("REMOVING TRIADE")
-                self.triade = None
             self.removeAnnotation(actor)
 
 
@@ -742,7 +883,6 @@ class Canvas(object):
         self.removeHighlights()
         self.removeAnnotations()
         self.removeDecorations()
-        self.triade = None
 
 
     def redrawAll(self):
@@ -772,8 +912,9 @@ class Canvas(object):
         self.makeCurrent()
         # go to a distance to have a good view with a 45 degree angle lens
         if bbox is not None:
+            pf.debug("SETTING BBOX: %s" % self.bbox)
             self.setBbox(bbox)
-        #GD.debug("USING BBOX: %s" % self.bbox)
+        pf.debug("USING BBOX: %s" % self.bbox)
         X0,X1 = self.bbox
         center = 0.5*(X0+X1)
         # calculating the bounding circle: this is rather conservative
@@ -788,10 +929,10 @@ class Canvas(object):
         # Currently, we keep the default fovy/aspect
         # and change the camera distance to focus
         fovy = self.camera.fovy
-        #GD.debug("FOVY: %s" % fovy)
+        #pf.debug("FOVY: %s" % fovy)
         self.camera.setLens(fovy,self.aspect)
         # Default correction is sqrt(3)
-        correction = float(GD.cfg.get('gui/autozoomfactor',1.732))
+        correction = float(pf.cfg.get('gui/autozoomfactor',1.732))
         tf = tand(fovy/2.)
 
         import simple,coords
@@ -804,7 +945,7 @@ class Canvas(object):
         dist = (vsize/tf + offset) / correction
         
         if dist == nan or dist == inf:
-            GD.debug("DIST: %s" % dist)
+            pf.debug("DIST: %s" % dist)
             return
         if dist <= 0.0:
             dist = 1.0
@@ -819,34 +960,35 @@ class Canvas(object):
             self.camera.dolly(f)
 
 
-    ## def unProject(self,x,y,z):
-    ##     "Map the window coordinates (x,y,z) to object coordinates."""
-    ##     self.makeCurrent()
-    ##     #y = vp.h-y
-    ##     model = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
-    ##     proj = GL.glGetFloatv(GL.GL_PROJECTION_MATRIX)
-    ##     view = GL.glGetIntegerv(GL.GL_VIEWPORT)
-    ##     print("Modelview matrix:",model)
-    ##     print("Projection matrix:",proj)
-    ##     print("Viewport:",view)
-    ##     print("Point:",str((x,y,z)) )
-    ##     objx, objy, objz = GLU.gluUnProject(x,y,z,model,proj,view)
-    ##     print("Coordinates: ",x,y," map to ",objx,objy)
-    ##     return (objx,objy,objz)
+    def project(self,x,y,z,locked=False):
+        "Map the object coordinates (x,y,z) to window coordinates."""
+        locked=False
+        if locked:
+            model,proj,view = self.projection_matrices
+        else:
+            self.makeCurrent()
+            self.camera.loadProjection()
+            model = GL.glGetDoublev(GL.GL_MODELVIEW_MATRIX)
+            proj = GL.glGetDoublev(GL.GL_PROJECTION_MATRIX)
+            view = GL.glGetIntegerv(GL.GL_VIEWPORT)
+        winx,winy,winz = GLU.gluProject(x,y,z,model,proj,view)
+        return winx,winy,winz
+        return self.camera.project(x,y,z)
 
-
-    ## def unProject2(self,x,y,z):
-    ##     "Map the window coordinates (x,y,z) to object coordinates."""
-    ##     model = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
-    ##     proj = GL.glGetFloatv(GL.GL_PROJECTION_MATRIX)
-    ##     view = GL.glGetIntegerv(GL.GL_VIEWPORT)
-    ##     print("Modelview matrix:",model)
-    ##     print("Projection matrix:",proj)
-    ##     print("Viewport:",view)
-    ##     print("Point:",str((x,y,z)) )
-    ##     objx, objy, objz = GLU.gluUnProject(x,y,z,model,proj,view)
-    ##     print("Coordinates: ",x,y," map to ",objx,objy)
-    ##     return (objx,objy,objz)
+    def unProject(self,x,y,z,locked=False):
+        "Map the window coordinates (x,y,z) to object coordinates."""
+        locked=False
+        if locked:
+            model,proj,view = self.projection_matrices
+        else:
+            self.makeCurrent()
+            self.camera.loadProjection()
+            model = GL.glGetDoublev(GL.GL_MODELVIEW_MATRIX)
+            proj = GL.glGetDoublev(GL.GL_PROJECTION_MATRIX)
+            view = GL.glGetIntegerv(GL.GL_VIEWPORT)
+        objx, objy, objz = GLU.gluUnProject(x,y,z,model,proj,view)
+        return (objx,objy,objz)
+        return self.camera.unProject(x,y,z)
 
 
     def zoomRectangle(self,x0,y0,x1,y1):
@@ -889,15 +1031,15 @@ class Canvas(object):
         """draw the cursor"""
         if self.cursor:
             self.removeDecoration(self.cursor)
-        w,h = GD.cfg.get('pick/size',(20,20))
-        col = GD.cfg.get('pick/color','yellow')
+        w,h = pf.cfg.get('pick/size',(20,20))
+        col = pf.cfg.get('pick/color','yellow')
         self.cursor = decors.Grid(x-w/2,y-h/2,x+w/2,y+h/2,color=col,linewidth=1)
         self.addDecoration(self.cursor)
 
     def draw_rectangle(self,x,y):
         if self.cursor:
             self.removeDecoration(self.cursor)
-        col = GD.cfg.get('pick/color','yellow')
+        col = pf.cfg.get('pick/color','yellow')
         self.cursor = decors.Grid(self.statex,self.statey,x,y,color=col,linewidth=1)
         self.addDecoration(self.cursor)
 
