@@ -1,6 +1,6 @@
-# $Id: decors.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: decors.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,7 +28,7 @@ from PyQt4 import QtOpenGL
 
 from drawable import *
 from text import *
-from actors import Actor
+from marks import TextMark
 
 import colors
 import gluttext
@@ -117,6 +117,53 @@ class Decoration(Drawable):
         self.y = int(y)
         Drawable.__init__(self)
 
+
+# Marks database: a dict with mark name and a function to draw
+# the mark. The 
+_marks_ = {
+    'dot':drawDot,
+    }
+
+class Mark(Decoration):
+    """A mark at a fixed position on the canvas."""
+    def __init__(self,x,y,mark='dot',color=None,linewidth=None):
+        Decoration.__init__(self,x,y)
+        self.x = x
+        self.y = y
+        if not mark in _marks_:
+            raise ValueError,"Unknown mark: %s" % mark
+        self.mark = mark
+        self.color = saneColor(color)
+        self.linewidth = saneLineWidth(linewidth)
+
+
+    def drawGL(self,mode='wireframe',color=None):
+        if self.color is not None:
+            GL.glColor3fv(self.color)
+        if self.linewidth is not None:
+            GL.glLineWidth(self.linewidth)
+        _marks_[self.mark](self.x,self.y)
+
+
+class Line(Decoration):
+    """A straight line on the canvas."""
+    def __init__(self,x1,y1,x2,y2,color=None,linewidth=None):
+        Decoration.__init__(self,x1,y1)
+        self.x1 = x1
+        self.y1 = y1
+        self.x2 = x2
+        self.y2 = y2
+        self.color = saneColor(color)
+        self.linewidth = saneLineWidth(linewidth)
+
+
+    def drawGL(self,mode='wireframe',color=None):
+        if self.color is not None:
+            GL.glColor3fv(self.color)
+        if self.linewidth is not None:
+            GL.glLineWidth(self.linewidth)
+        drawLine(self.x1,self.y1,self.x2,self.y2)
+
         
 ## class QText(Decoration):
 ##     """A viewport decoration showing a text."""
@@ -144,23 +191,22 @@ class Decoration(Drawable):
 
 
 class GlutText(Decoration):
-    """A viewport decoration showing a text."""
+    """A viewport decoration showing a text string.
 
-    def __init__(self,text,x,y,font='9x15',size=None,gravity=None,adjust=None,color=None,zoom=None):
+    - text: a simple string, a multiline string or a list of strings. If it is
+      a string, it will be splitted on the occurrence of '\n' characters.
+    - x,y: insertion position on the canvas
+    - gravity: a string that determines the adjusting of the text with
+      respect to the insert position. It can be a combination of one of the
+      characters 'N or 'S' to specify the vertical positon, and 'W' or 'E'
+      for the horizontal. The default(empty) string will center the text.
+    """
+
+    def __init__(self,text,x,y,font='9x15',size=None,gravity=None,color=None,zoom=None):
         """Create a text actor"""
         Decoration.__init__(self,x,y)
         self.text = str(text)
         self.font = gluttext.glutSelectFont(font,size)
-        if adjust is not None:
-            import warnings
-            warnings.warn("The 'adjust' parameter should no longer be used. Use 'gravity' intead.", DeprecationWarning)
-
-            gravity = { 'left':'W',
-                        'right':'E',
-                        'center':'C',
-                        'under':'N',
-                        'above':'S',
-                        }[adjust]
 
         if gravity is None:
             gravity = 'E'
@@ -174,7 +220,7 @@ class GlutText(Decoration):
         ##     GD.canvas.zoom_2D(self.zoom)
         if self.color is not None: 
             GL.glColor3fv(self.color)
-        gluttext.glutDrawText(self.text,self.x,self.y,self.font,gravity=self.gravity)
+        gluttext.glutDrawText(self.text,self.x,self.y,font=self.font,gravity=self.gravity)
         ## if self.zoom:
         ##     GD.canvas.zoom_2D()
 
@@ -183,7 +229,7 @@ Text = GlutText
 
 class ColorLegend(Decoration):
     """A viewport decoration showing a colorscale legend."""
-    def __init__(self,colorlegend,x,y,w,h,font=None,size=None,dec=2,scale=0,grid=0,linewidth=None,lefttext=False):
+    def __init__(self,colorlegend,x,y,w,h,font=None,size=None,dec=2,scale=0,grid=0,linewidth=None,lefttext=False,nlabels=-1):
         Decoration.__init__(self,x,y)
         self.cl = colorlegend
         self.w = int(w)
@@ -201,13 +247,13 @@ class ColorLegend(Decoration):
     def drawGL(self,mode='wireframe',color=None):
         #from draw import drawText
         n = len(self.cl.colors)
+        GD.debug("NUMBER OF COLORS: %s" % n)
         x1 = float(self.x)
         x2 = float(self.x+self.w)
         y0 = float(self.y)
         dy = float(self.h)/n
         # colors
         y1 = y0
-        GD.debug("START COLORS AT %s" % y0)
         GL.glLineWidth(1.5)
         for i,c in enumerate(self.cl.colors):
             y2 = y0 + (i+1)*dy
@@ -225,18 +271,22 @@ class ColorLegend(Decoration):
         GD.debug("FONT HEIGHT %s" % fh)
         dh = fh + self.ygap # vert. distance between successive labels
         #y0 -= 0.25*fh  # 0.5*fh seems more logic, but character pos is biased
-        GD.debug("FIRST TEXT AT %s" % y0)
         GL.glColor3f(*colors.black)
         self.decorations = []
+
+        # FOR 3-VALUE SCALES THIS SHOULD BE DONE IN TWO PARTS,
+        # FROM THE CENTER OUTWARDS, AND THEN ADDING THE
+        # MIN AND MAX VALUES
         for i,v in enumerate(self.cl.limits):
             y2 = y0 + i*dy
-            GD.debug("next y = %s" % y2)
-            if y2 >= y1 or i == 0:
-                GD.debug("drawing at %s" % y2)
+            if y2 >= y1 or i == 0 or i == len(self.cl.limits)-1:
+                if y2 >= self.y+self.h-dh and i < len(self.cl.limits)-1:
+                    continue
                 t = Text(("%%.%df" % self.dec) % (v*self.scale),x1,y2,font=self.font,gravity=gravity)
                 self.decorations.append(t)
                 t.drawGL(mode,color)
                 y1 = y2 + dh
+
         # grid: after values, to be on top
         if self.linewidth is not None:
             GL.glLineWidth(self.linewidth)
@@ -286,26 +336,6 @@ class Grid(Decoration):
         drawGrid(self.x1,self.y1,self.x2,self.y2,self.nx,self.ny)
 
 
-class Line(Decoration):
-    """A straight line on the canvas."""
-    def __init__(self,x1,y1,x2,y2,color=None,linewidth=None):
-        Decoration.__init__(self,x1,y1)
-        self.x1 = x1
-        self.y1 = y1
-        self.x2 = x2
-        self.y2 = y2
-        self.color = saneColor(color)
-        self.linewidth = saneLineWidth(linewidth)
-
-
-    def drawGL(self,mode='wireframe',color=None):
-        if self.color is not None:
-            GL.glColor3fv(self.color)
-        if self.linewidth is not None:
-            GL.glLineWidth(self.linewidth)
-        drawLine(self.x1,self.y1,self.x2,self.y2)
-
-
 class LineDrawing(Decoration):
     """A collection of straight lines on the canvas."""
     def __init__(self,data,color=None,linewidth=None):
@@ -337,4 +367,122 @@ class LineDrawing(Decoration):
         GL.glEnd()
 
 
+# Not really a decoration, though it could be made into one
+
+class Triade(Drawable):
+    """An OpenGL actor representing a triade of global axes.
+
+    - `pos`: position on the canvas: two charaters, of which first sets
+       horizontal position ('l', 'c' or 'r') and second sets vertical
+       position ('b', 'c' or 't').
+    - `size`: size in pixels of the zone displaying the triade.
+    - `pat`: shape to be drawn in the coordinate planes. Default is a square.
+      '16' givec a triangle. '' disables the planes.
+    - `legend`: text symbols to plot at the end of the axes. A 3-character
+      string or a tuple of 3 strings.
+    """
+
+    def __init__(self,pos='lb',siz=100,pat='12-34',legend='xyz',color=[red,green,blue,cyan,magenta,yellow]):
+        Drawable.__init__(self)
+        self.pos = pos
+        self.siz = siz
+        self.pat = pat
+        self.legend = legend
+        self.color = color
+
+ 
+    def _draw_me(self):
+        """Draw the triade components."""
+        GL.glBegin(GL.GL_LINES)
+        pts = Formex(pattern('1')).coords.reshape(-1,3)
+        GL.glColor3f(*black)
+        for i in range(3):
+            #GL.glColor(*self.color[i])
+            for x in pts:
+                GL.glVertex3f(*x)
+            pts = pts.rollAxes(1)
+        GL.glEnd()
+        # Coord planes
+        if self.pat:
+            GL.glBegin(GL.GL_TRIANGLES)
+            pts = Formex(mpattern(self.pat))
+            #pts += pts.reverse()
+            pts = pts.scale(0.5).coords.reshape(-1,3)
+            for i in range(3):
+                pts = pts.rollAxes(1)
+                GL.glColor3f(*self.color[i])
+                for x in pts:
+                    GL.glVertex3f(*x)
+            GL.glEnd()
+        # Coord axes denomination
+        for i,x in enumerate(self.legend):
+            p = unitVector(i)*1.1
+            t = TextMark(p,x)
+            t.drawGL()
+
+ 
+    def _draw_relative(self):
+        """Draw the triade in the origin and with the size of the 3D space."""
+        GL.glMatrixMode(GL.GL_MODELVIEW)
+        GL.glPushMatrix()
+        GL.glTranslatef (*self.pos) 
+        GL.glScalef (self.size,self.size,self.size)
+        self._draw_me()
+        GL.glMatrixMode(GL.GL_MODELVIEW)
+        GL.glPopMatrix()
+
+
+    def _draw_absolute(self):
+        """Draw the triade in the lower left corner."""
+        GL.glMatrixMode(GL.GL_MODELVIEW)
+        GL.glPushMatrix()
+        # Cancel the translations
+        rot = GL.glGetFloatv(GL.GL_MODELVIEW_MATRIX)
+        rot[3,0:3] = [0.,0.,0.]
+        GL.glLoadMatrixf(rot)
+        vp = GL.glGetIntegerv(GL.GL_VIEWPORT)
+        x,y,w,h = vp
+        w0,h0 = self.siz,self.siz # we force aspect ratio 1
+        if self.pos[0] == 'l':
+            x0 = x
+        elif self.pos[0] =='r':
+            x0 = x + w-w0
+        else:
+            x0 = x + (w-w0)/2     
+        if self.pos[1] == 'b':
+            y0 = y
+        elif self.pos[1] =='t':
+            y0 = y + h-h0
+        else:
+            y0 = y + (h-h0)/2
+        GL.glViewport(x0,y0,w0,h0)
+        GL.glMatrixMode(GL.GL_PROJECTION)
+        GL.glPushMatrix()
+        GL.glLoadIdentity()
+        fovy = 45.
+        fv = tand(fovy*0.5)
+        fv *= 4.
+        fh = fv
+        # BEWARE: near/far should be larger than size, but not very large
+        # or the depth sort will fail
+        frustum = (-fh,fh,-fv,fv,-3.,100.)
+        GL.glOrtho(*frustum)
+        GL.glDisable(GL.GL_LIGHTING)
+        GL.glDisable (GL.GL_BLEND)
+        GL.glPolygonMode(GL.GL_FRONT_AND_BACK,GL.GL_FILL)
+        GL.glDisable(GL.GL_CULL_FACE)
+        GL.glClearDepth(1.0)
+        GL.glDepthMask (GL.GL_TRUE)
+        GL.glDepthFunc(GL.GL_LESS)
+        GL.glEnable(GL.GL_DEPTH_TEST)
+        self._draw_me()
+        GL.glViewport(*vp)
+        GL.glMatrixMode(GL.GL_PROJECTION)
+        GL.glPopMatrix()
+        GL.glMatrixMode(GL.GL_MODELVIEW)
+        GL.glPopMatrix()
+
+    def draw(self,mode='wireframe',color=None):
+        self._draw_absolute()
+
 # End
