@@ -1,6 +1,6 @@
-# $Id: drawable.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: drawable.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -115,7 +115,8 @@ def drawPolygons(x,e,mode,color=None,alpha=1.0,normals=None,objtype=-1):
     n = None
     if mode.startswith('smooth') and objtype==-1:
         if normals is None:
-            if mode == 'smooth-avg' and e is not None:
+            GD.debug("Computing normals")
+            if mode == 'smooth_avg' and e is not None:
                 n = interpolateNormals(x,e,treshold=GD.cfg['render/avgnormaltreshold'])
                 mode = 'smooth'
             else:
@@ -152,11 +153,13 @@ def drawPolygons(x,e,mode,color=None,alpha=1.0,normals=None,objtype=-1):
 
 def drawPolyLines(x,e,color):
     """Draw the circumference of polygons."""
+    GD.debug("drawPolyLines")
     drawPolygons(x,e,mode='wireframe',color=color,alpha=1.0,objtype=GL.GL_LINE_LOOP)
 
 
 def drawLines(x,e,color):
     """Draw straight line segments."""
+    GD.debug("drawLines")
     drawPolygons(x,e,mode='wireframe',color=color,alpha=1.0)
 
 
@@ -248,6 +251,10 @@ def drawFaces(x,e,faces,mode,color=None,alpha=1.0):
     """
     GD.debug("drawFaces")
     # We may have faces with different plexitudes!
+    # We collect them according to plexitude.
+    # But first convert to a list, so that we can call this function
+    # with an array too (in case of a single plexitude)
+    faces = list(faces)
     for fac in olist.collectOnLength(faces).itervalues():
         fa = asarray(fac)
         nplex = fa.shape[1]
@@ -390,7 +397,6 @@ def interpolateNormals(coords,elems,atNodes=False,treshold=None):
     return normalize(n)
 
 
-
 def drawQuadraticCurves(x,color=None,n=8):
     """Draw a collection of curves.
 
@@ -405,7 +411,7 @@ def drawQuadraticCurves(x,color=None,n=8):
         if color is not None:
             GL.glColor3fv(color[i])
         P = dot(H,x[i])
-        GD,debug("P.shape=%s"%str(P.shape))
+        GD.debug("P.shape=%s"%str(P.shape))
         GL.glBegin(GL.GL_LINE_STRIP)
         for p in P:
             GL.glVertex3fv(p)
@@ -415,17 +421,19 @@ def drawQuadraticCurves(x,color=None,n=8):
 def drawNurbsCurves(x,color=None):
     """Draw a collection of curves.
 
-    x is a (nlines,3,3) shaped array of coordinates.
+    x is an (nlines,4,3) or (nlines,3,3) shaped array of coordinates.
 
     If color is given it is an (nlines,3) array of RGB values.
     """
     nurb = GLU.gluNewNurbsRenderer()
+    if not nurb:
+        raise RuntimeError,"Could not create a new NURBS renderer"
+        return
+    
     if x.shape[1] == 4:
         knots = array([0.,0.,0.,0.,1.0,1.0,1.0,1.0])
     if x.shape[1] == 3:
         knots = array([0.,0.,0.,1.0,1.0,1.0])
-    if not nurb:
-        return
     for i,xi in enumerate(x):
         if color is not None:
             GL.glColor3fv(color[i])
@@ -661,7 +669,7 @@ def saneColorSet(color=None,colormap=None,shape=(1,)):
         if color.dtype.kind == 'i':
             ncolors = color.max()+1
             if colormap is None:
-                colormap = GD.canvas.settings.propcolors
+                colormap = GD.canvas.settings.colormap
             colormap = saneColor(colormap)
             colormap = saneColorArray(colormap,(ncolors,))
         else:
@@ -703,7 +711,7 @@ class Drawable(object):
     def __init__(self):
         self.trans = False
         self.list = None
-        self.atype = 'unknown'
+        ## self.atype = 'unknown'
 
     def drawGL(self,**kargs):
         """Perform the OpenGL drawing functions to display the actor."""
