@@ -1,6 +1,6 @@
-# $Id: drawgl.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: drawgl.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -83,12 +83,10 @@ def draw_polygons(x,n,c,alpha,objtype):
         n = n.astype(float32)
     if c is not None:
         c = c.astype(float32)
-
     if objtype < 0:
         objtype = glObjType(nplex)
         
-    if nplex <= 4:
-
+    if nplex <= 4 and glObjType(nplex) == objtype:
         GL.glBegin(objtype)
         if c is None:
             if n is None:
