@@ -1,6 +1,6 @@
-## $Id: drawlock.py 1252 2009-12-09 11:05:19Z bverheg $
+## $Id: drawlock.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -45,8 +45,8 @@ class DrawLock(object):
         """
         if self.allowed:
             while self.locked:
-                GD.app.processEvents()
                 GD.canvas.update()
+                GD.app.processEvents()
 
 
     def lock(self,time=None):
@@ -54,15 +54,18 @@ class DrawLock(object):
 
         If a no time is specified, a global value is used.
         """
+        #print self.allowed,self.locked
         if self.allowed and not self.locked:
             if time is None:
                 time = GD.GUI.drawwait
             if time > 0:
+                GD.debug('STARTING TIMER')
                 self.locked = True
                 self.timer = threading.Timer(time,self.release)
                 self.timer.start()
 
 
+    # ?? SHOULD block and release only be activated if self.allowed ??
     def block(self):
         """Lock the drawing function indefinitely."""
         if self.timer:
@@ -75,6 +78,7 @@ class DrawLock(object):
 
         If a timer is running, cancel it.
         """
+        #print "RELEASING LOCK"
         self.locked = False
         if self.timer:
             self.timer.cancel()
@@ -93,4 +97,43 @@ class DrawLock(object):
         """
         self.allowed = True
 
+
+def repeat(func,duration=-1,maxcount=-1,*args,**kargs):
+    """Repeatedly execute a function.
+
+    func(*args,**kargs) is repeatedly executed until one of the following
+    conditions is met:
+    - the function returns a value that evaluates to False
+    - duration >= 0 and duration seconds have elapsed
+    - maxcount >=0 and maxcount executions have been reached
+    The default will indefinitely execute the function until it returns False.
+    This may cause the system to freeze if it never does so!
+    """
+    GD.debug("REPEAT: %s, %s" % (duration,maxcount))
+    global _repeat_timed_out
+    _repeat_timed_out = False
+    _repeat_count_reached = False
+    
+    def timeOut():
+        global _repeat_timed_out
+        _repeat_timed_out = True
+        
+    if duration >= 0:
+        timer = threading.Timer(duration,timeOut)
+        timer.start()
+    
+    count = 0
+
+    while True:
+        GD.app.processEvents()
+        res = func(*args,**kargs)
+        _exit_requested = not(res)
+        count += 1
+        if maxcount >= 0:
+             _repeat_count_reached = count >= maxcount
+        if _exit_requested or _repeat_timed_out or _repeat_count_reached:
+            GD.debug("Count: %s, TimeOut: %s" % (count,_repeat_timed_out))
+            break
+            
+
 #### End
