@@ -1,6 +1,6 @@
-# $Id: fileMenu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: fileMenu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,11 +29,12 @@ import widgets
 import utils
 import project
 import draw
-from script import processArgs
+from script import processArgs,play
 import image
-from plugins import surface_menu,formex_menu,mesh_menu,tools_menu,postproc_menu,jobs_menu
+import plugins
 
 from gettext import gettext as _
+from prefMenu import updateSettings
 
 
 ##################### handle project files ##########################
@@ -41,7 +42,7 @@ from gettext import gettext as _
 the_project = None
 
 
-def createProject(create=True,compression=0,addGlobals=None):
+def createProject(create=True,compression=0,addGlobals=None,makeDefault=True):
     """Open a file selection dialog and let the user select a project.
 
     The default will let the user create new project files as well as open
@@ -54,6 +55,13 @@ def createProject(create=True,compression=0,addGlobals=None):
 
     Only one pyFormex project can be open at any time. The open project
     owns all the global data created and exported by any script.
+
+    If makeDefault is True, an already open project will be closed and
+    the opened project becomes the current project.
+    If makeDefault is False, the project data are imported into GD.PF
+    and the current project does not change. This means that if a project was
+    open, the imported data will be added to it.
+    
     If addGlobals is None, the user is asked whether the current globals
     should be added to the project. Set True or False to force or reject
     the adding without asking.
@@ -64,13 +72,14 @@ def createProject(create=True,compression=0,addGlobals=None):
     if the_project is None:
         cur = GD.cfg.get('workdir','.')
     else:
-        options = ['Cancel','Close without saving','Save and Close']
-        ans = draw.ask("Another project is still open. Shall I close it first?",
-                    options)
-        if ans == 'Cancel':
-            return
-        if ans == options[2]:
-            the_project.save()
+        if makeDefault:
+            options = ['Cancel','Close without saving','Save and Close']
+            ans = draw.ask("Another project is still open. Shall I close it first?",
+                        options)
+            if ans == 'Cancel':
+                return
+            if ans == options[2]:
+                the_project.save()
         cur = the_project.filename
     typ = utils.fileDescription(['pyf','all'])
     res = widgets.ProjectSelection(cur,typ,exist=not create).getResult()
@@ -106,37 +115,93 @@ def createProject(create=True,compression=0,addGlobals=None):
 
     # OK, we have all data, now create/open the project
         
-    GD.cfg['workdir'] = os.path.dirname(fn)
+    updateSettings({'workdir':os.path.dirname(fn)},save=True)
     sig = GD.Version[:GD.Version.rfind('-')]
     if ignoresig:
         sig = ''
 
-    # Loading the project make take a long while; attent user
-    GD.GUI.setBusy()
-    try:
-        the_project = project.Project(fn,create=create,signature=sig,compression=compression,legacy=legacy)
+    proj = _open_project(fn,create,sig,compression,legacy)
+        
+    GD.message("Project contents: %s" % proj.keys())
+    
+    if hasattr(proj,'_autoscript_'):
+        _ignore = "Ignore it!"
+        _show = "Show it"
+        _edit = "Load it in the editor"
+        _exec = "Execute it"
+        res = draw.ask("There is an autoscript stored inside the project.\nIf you received this project file from an untrusted source, you should probably not execute it.",[_ignore,_show,_edit,_exec])
+        if res == _show:
+            res = draw.showText(proj._autoscript_)#,actions=[_ignore,_edit,_show])
+            return
+        if res == _exec:
+            draw.playScript(proj._autoscript_)
+        elif res == _edit:
+            fn = "_autoscript_.py"
+            draw.checkWorkdir()
+            f = file(fn,'w')
+            f.write(proj._autoscript_)
+            f.close()
+            openScript(fn)
+            editScript(fn)
+
+    if hasattr(proj,'autofile') and draw.ack("The project has an autofile attribute: %s\nShall I execute this script?" % proj.autofile):
+        processArgs([proj.autofile])
+
+    if makeDefault:
+        the_project = proj
         if GD.PF and addGlobals:
             the_project.update(GD.PF)
-    finally:
-        GD.GUI.setBusy(False)
-        
-    GD.PF = the_project
-    GD.GUI.setcurproj(fn)
-    GD.message("Project contents: %s" % the_project.keys())
-    if hasattr(the_project,'autofile') and draw.ack("The project has an autofile attribute: %s\nShall I execute this script?" % the_project.autofile):
-        processArgs([the_project.autofile])
+        GD.PF = the_project
+        GD.GUI.setcurproj(fn)
+
+    else:
+        # Just import the data into current project
+        GD.PF.update(proj)
+
+    GD.message("Exported symbols: %s" % GD.PF.keys())
 
 
 def openProject():
-    """Create a new project file, uncompressed by default.
+    """Open an existing project.
 
-    If a compression level (1..9) is specified, the stored data will be
-    gzipped. This may substantially reduce the size of large databases.
-
-    This internal compression is easier to the user than externally
-    compressing the resulting files.
+    Ask the user to select an existing project file, and then open it.
     """
     createProject(create=False)
+
+
+def importProject():
+    """Import an existing project.
+
+    Ask the user to select an existing project file, and then import
+    its data into the current project.
+    """
+    createProject(create=False,addGlobals=False,makeDefault=False)
+
+
+def _open_project(fn,create,signature,compression,legacy):
+    """Open a project in the GUI
+
+    This is a low level function not intended for the user.
+    It is equivalent to creating a project instance, but has
+    exception trapping
+    """
+    # Loading the project may take a long while; attent user
+    GD.GUI.setBusy()
+    try:
+        proj = project.Project(fn,create,signature,compression,legacy)
+    except:
+        proj = None
+        raise
+    finally:
+        GD.GUI.setBusy(False)
+    return proj
+    
+
+def setAutoScript():
+    """Set the current script as autoScript in the project"""
+    global the_project
+    if the_project is not None and GD.cfg['curfile'] and GD.GUI.canPlay:
+        the_project._autoscript_ = file(GD.cfg['curfile']).read()
  
 
 def setAutoFile():
@@ -145,6 +210,13 @@ def setAutoFile():
     if the_project is not None and GD.cfg['curfile'] and GD.GUI.canPlay:
         the_project.autofile = GD.cfg['curfile']
             
+def removeAutoScript():
+    global the_project
+    delattr(the_project,'_autoscript_')
+            
+def removeAutoFile():
+    global the_project
+    delattr(the_project,'autofile')
 
 def saveProject():
     if the_project is not None:
@@ -179,7 +251,8 @@ def closeProject(save=True):
         GD.PF.update(the_project)
         GD.GUI.setcurproj('None')
     the_project = None
-
+        
+    
 
 def askCloseProject():
     if the_project is not None:
@@ -210,7 +283,11 @@ def openScript(fn=None,exist=True,create=False):
     openScript() to open existing scripts.
     """
     if fn is None:
-        cur = GD.cfg.get('curfile',GD.cfg.get('workdir','.'))
+        cur = GD.cfg['curfile']
+        if cur is None:
+            cur = GD.cfg['workdir']
+        if cur is None:
+            cur  = '.'
         typ = utils.fileDescription('pyformex')
         fn = widgets.FileSelection(cur,typ,exist=exist).getFilename()
     if fn:
@@ -220,7 +297,7 @@ def openScript(fn=None,exist=True,create=False):
             template = GD.cfg['scripttemplate']
             if (os.path.exists(template)):
                 shutil.copyfile(template,fn)
-        GD.cfg['workdir'] = os.path.dirname(fn)
+        updateSettings({'workdir':os.path.dirname(fn)},save=True)
         GD.GUI.setcurfile(fn)
         GD.GUI.history.add(fn)
         if create:
@@ -261,8 +338,14 @@ def saveImage(multi=False):
     dia = widgets.SaveImageDialog(GD.cfg['workdir'],pat,multi=multi)
     opt = dia.getResult()
     if opt:
-        GD.cfg['workdir'] = os.path.dirname(opt.fn)
+        if opt.fm == 'From Extension':
+            opt.fm = None
+        if opt.qu < 0:
+            opt.qu = -1
+        updateSettings({'workdir':os.path.dirname(opt.fn)},save=True)
         image.save(filename=opt.fn,
+                   format=opt.fm,
+                   quality=opt.qu,
                    window=opt.wi,
                    multi=opt.mu,
                    hotkey=opt.hk,
@@ -300,25 +383,16 @@ def showImage():
     if fn:
         viewer = ImageViewer(GD.app,fn)
         viewer.show()
-        
-
-def setOptions():
-    options = ['test','debug','uselib','safelib','fastencode']
-    options = [ o for o in options if hasattr(GD.options,o) ]
-    items = [ (o,getattr(GD.options,o)) for o in options ]
-    res = draw.askItems(items)
-    if res:
-        for o in options:
-            setattr(GD.options,o,res[o])
-            print(GD.options)
-            ## if o == 'debug':
-            ##     GD.setDebugFunc()
 
 
 MenuData = [
     (_('&Start new project'),createProject),
     (_('&Open existing project'),openProject),
+    (_('&Import another project'),importProject),
+    (_('&Set current script as AutoScript'),setAutoScript),
+    (_('&Remove the AutoScript'),removeAutoScript),
     (_('&Set current script as AutoFile'),setAutoFile),
+    (_('&Remove the AutoFile'),removeAutoFile),
     (_('&Save project'),saveProject),
     (_('&Save project As'),saveAsProject),
     (_('&Close project without saving'),closeProjectWithoutSaving),
@@ -326,7 +400,7 @@ MenuData = [
     ('---',None),
     (_('&Create new script'),createScript),
     (_('&Open existing script'),openScript),
-    (_('&Play script'),draw.play),
+    (_('&Play script'),play),
     (_('&Edit script'),editScript),
     (_('&Change workdir'),draw.askDirname),
     (_('---1'),None),
@@ -338,19 +412,10 @@ MenuData = [
     (_('&Save as Icon'),saveIcon),
     (_('&Show Image'),showImage),
     (_('---2'),None),
-    (_('Load &Plugins'),[
-        (_('Surface menu'),surface_menu.show_menu),
-        (_('Formex menu'),formex_menu.show_menu),
-        (_('Mesh menu'),mesh_menu.show_menu),
-        (_('Tools menu'),tools_menu.show_menu),
-        (_('Jobs menu'),jobs_menu.show_menu),
-        (_('Postproc menu'),postproc_menu.show_menu),
-        ]),
-    (_('&Options'),setOptions),
-    (_('---3'),None),
     (_('E&xit'),draw.closeGui),
 ]
 
+
 #onExit(closeProject)
 
 # End
