@@ -1,6 +1,6 @@
-# $Id: gluttext.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: gluttext.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -22,7 +22,11 @@
 ##  along with this program.  If not, see http://www.gnu.org/licenses/.
 ##
 
-"""2D text decorations using GLUT fonts"""
+"""2D text decorations using GLUT fonts
+
+This module provides the basic functions for using the GLUT library in the
+rendering of text on an OpenGL canvas.
+"""
 
 import OpenGL.GL as GL
 import OpenGL.GLU as GLU
@@ -55,11 +59,12 @@ GLUTFONTALIAS = {
     'sans'  : ('helvetica','hv10','hv12','hv18'),
     }
 
+
 def glutSelectFont(font=None,size=None):
     """Select one of the glut fonts using a font + size description.
 
-    font is one of: 'fixed', 'serif', 'sans'
-    size is an int that will be rounded to the nearest available size.
+    - font: 'fixed', 'serif' or 'sans'
+    - size: an int that will be rounded to the nearest available size.
 
     The return value is a 4-character string representing one of the
     GLUT fonts.
@@ -102,7 +107,7 @@ def glutFont(font):
     - times-roman: 'tr10', 'tr24'
     - helvetica:   'hv10', 'hv12',  'hv18'
     
-    If an unrecognized string is  given, the default is 'hv18'.
+    If an unrecognized string is given, the default is 'hv18'.
     """
     return GLUTFONTS.get(font,GLUTFONTS['hv18'])
 
@@ -145,32 +150,47 @@ def glutBitmapLength(font, text):
     return len
 
 
-def glutDrawText(text,x,y,font='hv18',gravity=''):
-    """Draw the given text at given 2D position in window.
+def glutDrawText(text,x,y,font='hv18',gravity='',spacing=1.0):
+    """Draw a text at given 2D position in window.
 
-    If adjust == 'center', the text will be horizontally centered on
-    the insertion point.
-    If adjust == 'right', the text will be right aligned on the point.
-    Any other setting will align the text left.
-    Default is to center.
+    - text: a simple string, a multiline string or a list of strings. If it is
+      a string, it will be splitted on the occurrence of '\n' characters.
+    - x,y: insertion position on the canvas
+    - gravity: a string that determines the adjusting of the text with
+      respect to the insert position. It can be a combination of one of the
+      characters 'N or 'S' to specify the vertical positon, and 'W' or 'E'
+      for the horizontal. The default(empty) string will center the text.
+      
     """
-    # !!! Do not use GLUT.glutBitmapLength(font, text)
-    width = glutBitmapLength(font, text)
-    height = glutFontHeight(font)
-    w2,h2 = 0.5*width,0.4*height
-    x -= w2
-    y -= h2
-    if 'E' in gravity:
-        x += w2
-    elif 'W' in gravity:
-        x -= w2
-    if 'N' in gravity:
-        y += h2
-    elif 'S' in gravity:
-        y -= h2
-    GL.glRasterPos2f(float(x),float(y));
-    #GD.debug("RENDERING WITH FONT %s" % font) 
-    glutRenderText(text,font)
+    if type(text) is str:
+        text = text.split('\n')
+    nlines = len(text)
+    widths = [ glutBitmapLength(font, t) for t in text ]
+    fontheight = glutFontHeight(font)
+    spacing *= fontheight
+    width = max(widths)
+    height = spacing*nlines 
+
+    x,y = float(x),float(y)
+    if 'S' in gravity:
+        yi = y
+    elif 'N' in gravity:
+        yi = y + height
+    else:
+        yi = y + height/2
+
+    
+    for t,w in zip(text,widths):
+        if 'E' in gravity:
+            xi = x
+        elif 'W' in gravity:
+            xi = x - w
+        else:
+            xi = x - w/2
+        yi -= spacing
+        GL.glRasterPos2f(float(xi),float(yi))
+        glutRenderText(t,font)
+
 
 
 # End
