@@ -1,6 +1,6 @@
-# $Id: signals.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: signals.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,11 +28,12 @@ anywhere, causing attached functions to be executed.
 """
 import pyformex as GD
 
-from PyQt4.QtCore import Qt,QObject,SIGNAL
+from PyQt4.QtCore import Qt,QObject,SIGNAL,QThread
 
 # signals
 CANCEL = SIGNAL("Cancel")   # cancel the operation, undoing it
 DONE   = SIGNAL("Done")     # accept and finish the operation
+REDRAW = SIGNAL("Redraw")   # redraw a preview state
 WAKEUP = SIGNAL("Wakeup")   # wake up from a sleep state
 TIMEOUT = SIGNAL("Timeout") # terminate what was going on
 SAVE = SIGNAL("Save")       # 
@@ -53,8 +54,6 @@ def offSignal(signal,function,widget=None):
     if widget is None:
         widget = GD.GUI
     QObject.disconnect(widget,signal,function)
-
-
     
 # End
 
