@@ -1,6 +1,6 @@
-# $Id: toolbar.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: toolbar.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -21,7 +21,10 @@
 ##  You should have received a copy of the GNU General Public License
 ##  along with this program.  If not, see http://www.gnu.org/licenses/.
 ##
-"""Toolbars for pyformex GUI."""
+"""Toolbars for the pyFormex GUI.
+
+This module defines the functions for creating the pyFormex window toolbars.
+"""
 
 import pyformex as GD
 import os
@@ -31,7 +34,7 @@ import widgets
 import draw
 import utils
 
-
+ 
 ################### Script action toolbar ###########
 def addActionButtons(toolbar):
     """Add the script action buttons to the toolbar."""
@@ -50,18 +53,19 @@ def addActionButtons(toolbar):
 
 ################### General Button Functions ###########
 
-def addButton(toolbar,text,icon,func,repeat=False,toggle=False,checked=False,icon0=None):
+def addButton(toolbar,tooltip,icon,func,repeat=False,toggle=False,checked=False,icon0=None):
     """Add a button to a toolbar.
 
-    toolbar is where the button will be added, 
-    text appears as tooltip,
-    icon is the name of the icon to be displayed on the button,
-    func is called when button is pressed,
-
-    repeat == True: func will repeatedly be called if button is held down.
-    toggle == True: button is a toggle (stays in pressed state).
-    If the button is a toggle, checked is the initial state and icon1 may
-    specify an icon that will be displayed when button is not checked.
+    - `toolbar`:  the toolbar where the button will be added
+    - `tooltip`: the text to appears as tooltip
+    - `icon`: name of the icon to be displayed on the button,
+    - `func`: function to be called when the button is pressed,
+    - `repeat`: if True, the `func` will repeatedly be called if button is
+      held down.
+    - `toggle`: if True, the button is a toggle and stays in depressed state
+      until pressed again.
+    - `checked`: initial state for a toggle buton.
+    - `icon1`: for a toggle button, icon to display when button is not checked.
     """
     iconset = QtGui.QIcon()
     icon_on = QtGui.QPixmap(utils.findIcon(icon))
@@ -70,9 +74,9 @@ def addButton(toolbar,text,icon,func,repeat=False,toggle=False,checked=False,ico
         icon_off = QtGui.QPixmap(utils.findIcon(icon0))
         iconset.addPixmap(icon_off,QtGui.QIcon.Normal,QtGui.QIcon.Off)
                                  
-    a = toolbar.addAction(iconset,text,func)
-    b =  toolbar.children()[-1] # Get the QToolButton for the last action
-
+    a = toolbar.addAction(iconset,tooltip,func)
+    b = toolbar.widgetForAction(a)
+    
     if repeat:
         b.setAutoRepeat(True)
         b.setAutoRepeatDelay(500)
@@ -83,10 +87,14 @@ def addButton(toolbar,text,icon,func,repeat=False,toggle=False,checked=False,ico
         b.connect(b,QtCore.SIGNAL("clicked()"),QtCore.SLOT("toggle()"))
         b.setChecked(checked)
 
-    b.setToolTip(text)
+    b.setToolTip(tooltip)
 
     return b
-    
+
+
+def removeButton(toolbar,button):
+    """Remove a button from a toolbar."""
+    toolbar.removeAction(button)
 
 
 ################# Camera action toolbar ###############
@@ -188,9 +196,9 @@ def setLight(mode=True):
 
 normals_button = None
 
-def toggleNormals(): 
-    mode = not GD.canvas.avgnormals
-    GD.canvas.setAveragedNormals(mode)
+def toggleNormals():
+    state = not GD.canvas.avgnormals
+    GD.canvas.setAveragedNormals(state)
     GD.canvas.update()
     GD.app.processEvents()
 
@@ -200,12 +208,12 @@ def addNormalsButton(toolbar):
                                'normals-avg',toggleNormals,icon0='normals-ind',
                                toggle=True,checked=False)    
 
-def setNormals(mode=True):
-    """Set the normals mode on or off."""
-    GD.canvas.setAveragedNormals(mode)
+def setNormals(state=True):
+    """Set the normals mode to averaged or not."""
+    GD.canvas.setAveragedNormals(state)
     GD.canvas.update()
     if normals_button:
-        normals_button.setChecked(mode)
+        normals_button.setChecked(state)
     GD.app.processEvents()
 
 
@@ -267,28 +275,46 @@ def setShrink(mode):
 
 ################# Timeout Button ###############
 
-timeout_button = None # the toggle timeout button
+timeout_button = None # the timeout toggle button
 
 def toggleTimeout(onoff=None):
-    #print "TOGGLETIMEOUT"
     if onoff is None:
         onoff = widgets.input_timeout < 0
     if onoff:
-        widgets.setInputTimeout(GD.cfg.get('input/timeout',-1))
+        timeout = GD.cfg.get('gui/timeoutvalue',-1)
     else:
-        widgets.setInputTimeout(-1)
+        timeout = -1
+
+    widgets.setInputTimeout(timeout)
     onoff = widgets.input_timeout > 0
-    #print "TIMEOUT IS NOW %s" % widgets.input_timeout
+    if onoff:
+        # THIS SUSPENDS ALL WAITING! WE SHOULD IMPLEMENT A TIMEOUT!
+        # BY FORCING ALL INDEFINITE PAUSES TO A WAIT TIME EQUAL TO
+        # WIDGET INPUT TIMEOUT
+        GD.debug("FREEING the draw lock")
+        GD.GUI.drawlock.free()
+    else:
+        GD.debug("ALLOWING the draw lock")
+        GD.GUI.drawlock.allow()
     return onoff
 
 
 def addTimeoutButton(toolbar):
+    """Add or remove the timeout button,depending on cfg."""
     global timeout_button
-    timeout_button = addButton(toolbar,'Toggle Timeout','clock',toggleTimeout,
-                               toggle=True,checked=False)
+    if GD.cfg['gui/timeoutbutton']:
+        if timeout_button is None:
+            timeout_button = addButton(toolbar,'Toggle Timeout','clock',toggleTimeout,toggle=True,checked=False)
+    else:
+        if timeout_button is not None:
+            removeButton(toolbar,timeout_button)
+            timeout_button = None
+            
 
 def timeout(onoff=None):
-    timeout_button.setChecked( toggleTimeout(onoff) )
+    """Programmatically toggle the timeout button"""
+    if timeout_button is not None:
+        timeout_button.setChecked(toggleTimeout(onoff))
 
 
 
