@@ -1,6 +1,6 @@
-## $Id: viewport.py 1252 2009-12-09 11:05:19Z bverheg $
+## $Id: viewport.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -286,6 +286,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             self.setMouse(RIGHT,self.dynazoom,mod)
         self.selection_mode = None
         self.selection = Collection()
+        self.trackfunc = None
         self.pick_func = {
             'actor'  : self.pick_actors,
             'element': self.pick_elements,
@@ -293,7 +294,10 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             'edge'   : self.pick_edges,
             'number' : self.pick_numbers,
             }
+        self.pickable = None
+        self.drawmode = None
         self.drawing_mode = None
+        self.drawing = None
         # Drawing options
         self.resetOptions()
 
@@ -320,7 +324,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
     def setCursorShapeFromFunc(self,func):
         """Set the cursor shape to shape"""
-        if func in [ self.mouse_rectangle_zoom,self.mouse_pick,self.mouse_draw ]:
+        if func in [ self.mouse_rectangle_zoom,self.mouse_pick ]:
             shape = 'pick'
         else:
             shape = 'default'
@@ -359,17 +363,23 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
 
     def mouse_rectangle_zoom(self,x,y,action):
-        """Process mouse events during interactive picking.
+        """Process mouse events during interactive rectangle zooming.
 
         On PRESS, record the mouse position.
-        On MOVE, create a rectangular picking window.
-        On RELEASE, pick the objects inside the rectangle.
+        On MOVE, create a rectangular zoom window.
+        On RELEASE, zoom to the picked rectangle.
         """
         if action == PRESS:
             self.makeCurrent()
             self.update()
+            if self.trackfunc:
+                print "PRESS",self.trackfunc,GD.canvas.camera.ctr
+                GD.canvas.camera.setTracking(True)
+                x,y,z = GD.canvas.camera.ctr
+                self.zplane = GD.canvas.project(x,y,z,True)[2]
+                print 'ZPLANE',self.zplane
+                self.trackfunc(x,y,self.zplane)
             self.begin_2D_drawing()
-            #self.swapBuffers()
             GL.glEnable(GL.GL_COLOR_LOGIC_OP)
             # An alternative is GL_XOR #
             GL.glLogicOp(GL.GL_INVERT)        
@@ -378,6 +388,10 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             self.swapBuffers()
 
         elif action == MOVE:
+            if self.trackfunc:
+                print "MOVE",self.trackfunc
+                print 'ZPLANE',self.zplane
+                self.trackfunc(x,y,self.zplane)
             # Remove old rectangle
             self.swapBuffers()
             self.draw_state_rect(*self.state)
@@ -387,7 +401,6 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
         elif action == RELEASE:
             GL.glDisable(GL.GL_COLOR_LOGIC_OP)
-            #self.swapBuffers()
             self.end_2D_drawing()
             x0 = min(self.statex,x)
             y0 = min(self.statey,y)
@@ -396,6 +409,15 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             self.zoomRectangle(x0,y0,x1,y1)
             self.finish_rectangle_zoom()
 
+####################### INTERACTIVE PICKING ############################
+
+    def setPickable(self,nrs=None):
+        """Set the list of pickable actors"""
+        if nrs is None:
+            self.pickable = None
+        else:
+            self.pickable = [ self.actors[i] for i in nrs if i in range(len(self.actors))]
+        
 
     def start_selection(self,mode,filtr):
         """Start an interactive picking mode.
@@ -404,7 +426,6 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
         this can be used to change the filter method.
         """
         if self.selection_mode is None:
-            GD.debug("START SELECTION MODE: %s" % mode)
             self.setMouse(LEFT,self.mouse_pick)
             self.setMouse(LEFT,self.mouse_pick,SHIFT)
             self.setMouse(LEFT,self.mouse_pick,CTRL)
@@ -412,13 +433,11 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             self.setMouse(RIGHT,self.emit_cancel,SHIFT)
             self.connect(self,DONE,self.accept_selection)
             self.connect(self,CANCEL,self.cancel_selection)
-            #self.setCursorShape('pick')
             self.selection_mode = mode
             self.selection_front = None
 
         if filtr == 'none':
             filtr = None
-        GD.debug("SET SELECTION FILTER: %s" % filtr)
         self.selection_filter = filtr
         if filtr is None:
             self.selection_front = None
@@ -436,7 +455,6 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
     def finish_selection(self):
         """End an interactive picking mode."""
-        GD.debug("END SELECTION MODE")
         self.resetMouse(LEFT)
         self.resetMouse(LEFT,SHIFT)
         self.resetMouse(LEFT,CTRL)
@@ -451,7 +469,6 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
         If clear == True, the current selection is cleared.
         """
-        GD.debug("CANCEL SELECTION MODE")
         self.selection_accepted = True
         if clear:
             self.selection.clear()
@@ -532,7 +549,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
                         elif self.mod == CTRL:
                             self.selection.remove(self.picked)
                         if self.mod == NONE or self.mod == SHIFT:
-                            conn_elems = self.actors[closest_actor].connectedElements(closest_elem,self.selection.get(closest_actor))
+                            conn_elems = self.actors[closest_actor].object.connectedElements(closest_elem,self.selection.get(closest_actor))
                             self.selection.set(conn_elems,closest_actor)
                     if func:
                         func(self.selection)
@@ -549,11 +566,135 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
         """Go into number picking mode and return the selection."""
         return self.pick('numbers',*args,**kargs)
 
+#################### Interactive drawing ####################################
+
+    def idraw(self,mode='point',npoints=-1,zplane=0.,func=None,coords=None,preview=False):
+        """Interactively draw on the canvas.
+
+        This function allows the user to interactively create points in 3D
+        space and collects the subsequent points in a Coords object. The
+        interpretation of these points is left to the caller.
+        
+        - `mode`: one of the drawing modes, specifying the kind of objects you
+          want to draw. This is passed to the specified `func`.
+        - `npoints`: If -1, the user can create any number of points. When >=0,
+          the function will return when the total number of points in the
+          collection reaches the specified value.
+        - `zplane`: the depth of the z-plane on which the 2D drawing is done.
+        - `func`: a function that is called after each atomic drawing
+          operation. It is typically used to draw a preview using the current
+          set of points. The function is passed the current Coords and the
+          `mode` as arguments.
+        - `coords`: an initial set of coordinates to which the newly created
+          points should be added. If specified, `npoints` also counts these
+          initial points.
+        - `preview`: **Experimental** If True, the preview funcion will also
+          be called during mouse movement with a pressed button, allowing to
+          preview the result before a point is created.
+        
+        The drawing operation is finished when the number of requested points
+        has been reached, or when the user clicks the right mouse button or
+        hits 'ENTER'.
+        The return value is a (n,3) shaped Coords array.
+        """
+        self.draw_canceled = False
+        self.start_draw(mode,zplane,coords)
+        try:
+            if preview:
+                self.previewfunc = func
+            else:
+                self.previewfunc = None
+
+            while not self.draw_canceled:
+                self.wait_selection()
+                if not self.draw_canceled:
+                    self.drawn = Coords(self.drawn).reshape(-1,3)
+                    self.drawing = Coords.concatenate([self.drawing,self.drawn])
+                    if func:
+                        func(self.drawing,self.drawmode)
+                if npoints > 0 and len(self.drawing) >= npoints:
+                    self.accept_draw()                
+            if func and not self.draw_accepted:
+                func(self.drawing,self.drawmode)
+        finally:
+            self.finish_draw()
+        return self.drawing
+
+
+    def start_draw(self,mode,zplane,coords):
+        """Start an interactive drawing mode."""
+        self.setMouse(LEFT,self.mouse_draw)
+        self.setMouse(RIGHT,self.emit_done)
+        self.setMouse(RIGHT,self.emit_cancel,SHIFT)
+        self.connect(self,DONE,self.accept_draw)
+        self.connect(self,CANCEL,self.cancel_draw)
+        self.drawmode = mode
+        self.zplane = zplane
+        self.drawing = Coords(coords)
+
+    def finish_draw(self):
+        """End an interactive drawing mode."""
+        self.resetMouse(LEFT)
+        self.resetMouse(RIGHT)
+        self.resetMouse(RIGHT,SHIFT)
+        self.disconnect(self,DONE,self.accept_selection)
+        self.disconnect(self,CANCEL,self.cancel_selection)
+        self.drawmode = None
+
+    def accept_draw(self,clear=False):
+        """Cancel an interactive drawing mode.
+
+        If clear == True, the current drawing is cleared.
+        """
+        self.draw_accepted = True
+        if clear:
+            self.drawing = Coords()
+            self.draw_accepted = False
+        self.draw_canceled = True
+        self.selection_busy = False
+
+    def cancel_draw(self):
+        """Cancel an interactive drawing mode and clear the drawing."""
+        self.accept_draw(clear=True)
+
+
+    def mouse_draw(self,x,y,action):
+        """Process mouse events during interactive drawing.
+
+        On PRESS, do nothing.
+        On MOVE, do nothing.
+        On RELEASE, add the point to the point list.
+        """
+        if action == PRESS:
+            self.makeCurrent()
+            self.update()
+            if self.trackfunc:
+                print "ENABLE TRACKING"
+                GD.canvas.camera.setTracking(True)
+
+        elif action == MOVE:
+            if GD.app.hasPendingEvents():
+                return
+            if self.trackfunc:
+                self.trackfunc(x,y,self.zplane)
+                #GD.app.processEvents()
+            if self.previewfunc:
+                self.swapBuffers()
+                self.drawn = self.unProject(x,y,self.zplane)
+                self.drawn = Coords(self.drawn).reshape(-1,3)
+                self.previewfunc(Coords.concatenate([self.drawing,self.drawn]),self.drawmode)
+                self.swapBuffers()
+            
+        elif action == RELEASE:
+            self.drawn = self.unProject(x,y,self.zplane)
+            self.selection_busy = False
+
+##########################################################################
 
     def start_drawing(self,mode):
-        """Start an interactive drawing mode."""
+        """Start an interactive line drawing mode."""
         GD.debug("START DRAWING MODE")
-        self.setMouse(LEFT,self.mouse_draw)
+        self.setMouse(LEFT,self.mouse_draw_line)
         self.setMouse(RIGHT,self.emit_done)
         self.setMouse(RIGHT,self.emit_cancel,SHIFT)
         self.connect(self,DONE,self.accept_drawing)
@@ -608,12 +749,13 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
         """Interactively draw lines on the canvas.
 
         - single: if True, the function returns as soon as the user ends
-        a drawing operation. The default is to let the user
-        draw multiple lines and only to return after an explicit
-        cancel (ESC or right mouse button).
+          a drawing operation. The default is to let the user
+          draw multiple lines and only to return after an explicit
+          cancel (ESC or right mouse button).
         - func: if specified, this function will be called after each
-        atomic drawing operation. The current drawing is passed as
-        an argument. This can e.g. be used to show the drawing.
+          atomic drawing operation. The current drawing is passed as
+          an argument. This can e.g. be used to show the drawing.
+
         When the drawing operation is finished, the drawing is returned.
         The return value is a (n,2,2) shaped array.
         """
@@ -677,7 +819,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
     # their values.
     # On a mouse button RELEASE, self.button is cleared, to avoid further
     # move actions.
-    # Functions that change the camera settings should call saveMatrix()
+    # Functions that change the camera settings should call saveModelView()
     # when they are done.
     # ATTENTION! The y argument is positive upwards, as in normal OpenGL
     # operations!
@@ -725,7 +867,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
         elif action == RELEASE:
             self.update()
-            self.camera.saveMatrix()
+            self.camera.saveModelView()
 
             
     def dynapan(self,x,y,action):
@@ -746,7 +888,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
         elif action == RELEASE:
             self.update()
-            self.camera.saveMatrix()          
+            self.camera.saveModelView()          
 
             
     def dynazoom(self,x,y,action):
@@ -776,7 +918,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
 
         elif action == RELEASE:
             self.update()
-            self.camera.saveMatrix()
+            self.camera.saveModelView()
 
     def wheel_zoom(self,delta):
         """Zoom by rotating a wheel over an angle delta"""
@@ -851,10 +993,10 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             self.selection_busy = False
 
 
-    def pick_actors(self,store_closest=False):
+    def pick_actors(self):
         """Set the list of actors inside the pick_window."""
         self.camera.loadProjection(pick=self.pick_window)
-        self.camera.loadMatrix()
+        self.camera.loadModelView()
         stackdepth = 1
         npickable = len(self.actors)
         selbuf = GL.glSelectBuffer(npickable*(3+stackdepth))
@@ -864,21 +1006,16 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             GL.glPushName(i)
             GL.glCallList(a.list)
             GL.glPopName()
-##         buf = GL.glRenderMode(GL.GL_RENDER)
-##         self.picked = [ r[2][0] for r in buf]
-##         if store_closest and len(buf) > 0:
-##             d = asarray([ r[0] for r in buf ])
-##             dmin = d.min()
-##             w = where(d == dmin)[0][0]
-##             self.closest_pick = (self.picked[w], dmin)
         libGL.glRenderMode(GL.GL_RENDER)
         # Read the selection buffer
+        store_closest = self.selection_filter == 'single' or \
+                        self.selection_filter == 'closest'
         self.picked = []
         if selbuf[0] > 0:
             buf = asarray(selbuf).reshape(-1,3+selbuf[0])
             buf = buf[buf[:,0] > 0]
             self.picked = buf[:,3]
-            if store_closest and len(buf) > 0:
+            if store_closest:
                 w = buf[:,1].argmin()
                 self.closest_pick = (self.picked[w], buf[w,1])
 
@@ -893,18 +1030,27 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
         The picked object numbers are stored in self.picked.
         If store_closest==True, the closest picked object is stored in as a
         tuple ( [actor,object] ,distance) in self.picked_closest
+
+        A list of actors from which can be picked may be given.
+        If so, the resulting keys are indices in this list.
+        By default, the full actor list is used.
         """
         self.picked = []
+        GD.debug('PICK_PARTS %s %s %s' % (obj_type,max_objects,store_closest))
         if max_objects <= 0:
             GD.message("No such objects to be picked!")
             return
         self.camera.loadProjection(pick=self.pick_window)
-        self.camera.loadMatrix()
+        self.camera.loadModelView()
         stackdepth = 2
         selbuf = GL.glSelectBuffer(max_objects*(3+stackdepth))
         GL.glRenderMode(GL.GL_SELECT)
         GL.glInitNames()
-        for i,a in enumerate(self.actors):
+        if self.pickable is None:
+            pickable = self.actors
+        else:
+            pickable = self.pickable
+        for i,a in enumerate(pickable):
             GL.glPushName(i)
             a.pickGL(obj_type)  # this will push the number of the part
             GL.glPopName()
@@ -940,7 +1086,7 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
             npickable += a.npoints()
         self.pick_parts('point',npickable,store_closest=\
                         self.selection_filter == 'single' or\
-                        self.selection_filter == 'closest'
+                        self.selection_filter == 'closest',
                         )
 
 
@@ -952,14 +1098,14 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
                 npickable += a.nedges()
         self.pick_parts('edge',npickable,store_closest=\
                         self.selection_filter == 'single' or\
-                        self.selection_filter == 'closest'
+                        self.selection_filter == 'closest',
                         )
 
 
     def pick_numbers(self):
         """Return the numbers inside the pick_window."""
         self.camera.loadProjection(pick=self.pick_window)
-        self.camera.loadMatrix()
+        self.camera.loadModelView()
         self.picked = [0,1,2,3]
         if self.numbers:
             self.picked = self.numbers.drawpick()
@@ -968,11 +1114,10 @@ class QtCanvas(QtOpenGL.QGLWidget,canvas.Canvas):
     def draw_state_line(self,x,y):
         """Store the pos and draw a line to it."""
         self.state = x,y
-        #GD.debug("Rect (%s,%s) - (%s,%s)" % (self.statex,self.statey,x,y))
         decors.drawLine(self.statex,self.statey,x,y)
 
 
-    def mouse_draw(self,x,y,action):
+    def mouse_draw_line(self,x,y,action):
         """Process mouse events during interactive drawing.
 
         On PRESS, record the mouse position.
@@ -1116,7 +1261,6 @@ class MultiCanvas(FramedGridLayout):
     def __init__(self,parent=None):
         """Initialize the multicanvas."""
         FramedGridLayout.__init__(self)
-#        QtGui.QGridLayout.__init__(self)
         self.all = []
         self.active = []
         self.current = None
@@ -1125,11 +1269,6 @@ class MultiCanvas(FramedGridLayout):
         self.parent = parent
 
 
-    def setDefaults(self,dict):
-        """Update the default settings of the canvas class."""
-        GD.debug("Setting canvas defaults:\n%s" % dict)
-        canvas.CanvasSettings.default.update(canvas.CanvasSettings.checkDict(dict))
-
     def newView(self,shared=None):
         "Create a new viewport"
         canv = QtCanvas(self.parent,shared)
@@ -1139,6 +1278,9 @@ class MultiCanvas(FramedGridLayout):
     def addView(self):
         """Add a new viewport to the widget"""
         canv = self.newView()
+        if len(self.all) > 0:
+            # copy default settings from previous
+            canv.resetDefaults(self.all[-1].settings)
         self.all.append(canv)
         self.active.append(canv)
         self.showWidget(canv)
@@ -1253,9 +1395,16 @@ Viewport %s;  Active:%s;  Current:%s;  Settings:
         columns and nrows is ignored. If ncols is None and nrows is an int,
         viewports are laid out columnwise over nrows rows.
         """
+        if GD.options.fpbug:
+            print "CHANGE LAYOUT: %s (%s)" % (nvps,type(nvps))
+            print self.all
         if type(nvps) == int:
+            if GD.options.fpbug:
+                print "removeView"
             while len(self.all) > nvps:
                 self.removeView()
+            if GD.options.fpbug:
+                print "addView"
             while len(self.all) < nvps:
                 self.addView()
 
@@ -1267,10 +1416,14 @@ Viewport %s;  Active:%s;  Current:%s;  Settings:
         else:
             return
 
+        if GD.options.fpbug:
+            print "removeWidget"
         for w in self.all:
             self.removeWidget(w)
         self.ncols = ncols
         self.rowwise = rowwise
+        if GD.options.fpbug:
+            print "showWidget"
         for w in self.all:
             self.showWidget(w)
 
