@@ -1,6 +1,6 @@
-# $Id: viewportMenu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: viewportMenu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -29,43 +29,23 @@ import widgets
 import draw
 from gettext import gettext as _
 
-        
-
-def set_near_clip(v):
-    dist = GD.canvas.camera.getDist()
-    GD.canvas.camera.setClip(10**v*dist,100.*dist)
-    GD.canvas.update()
-    
-    
-def setClip():
-    items = [
-        ('near',-2.0,'slider',{'min':-100,'max':100,'scale':0.01,'func': set_near_clip}),
-        ]
-    res = draw.askItems(items)
-    ## if res:
-    ##     updateSettings(res,GD.cfg)
+from widgets import compatInputItem as C
 
 
-def setTriadeParams():
+def setTriade():
     try:
-        size = GD.canvas.triade.size
-        pos = GD.canvas.triade.pos.tolist()
+        pos = GD.canvas.triade.pos
+        siz = GD.canvas.triade.siz
     except:
-        size = 1.0
-        pos = [0.,0.,0.]
-    res = draw.askItems([('size',size),('pos',pos)])
+        pos = 'lb'
+        siz = 100
+    res = draw.askItems([
+        ('triade',True),
+        ('pos',pos,'select',{'choices':['lt','lc','lb','ct','cc','cb','rt','rc','rb']}),
+        ('size',siz),
+        ])
     if res:
-        draw.setTriade(True,res['size'],res['pos'])
-        
-
-def setRenderMode():
-    """Change the rendering mode."""
-    mode = GD.canvas.rendermode
-    modes = canvas.Canvas.rendermodes
-    itemlist = [('Render Mode', mode, 'select', modes)]
-    res = widgets.InputDialog(itemlist,'Select Render Mode').getResult()
-    if res:
-        GD.canvas.setRenderMode(res['Render Mode'])
+        draw.setTriade(res['triade'],res['pos'],res['size'])
 
 
 _the_dialog = None
@@ -101,10 +81,10 @@ def setBgColor2():
     color2 = GD.canvas.settings.bgcolor2
     if color2 is None:
         color2 = color
-    itemlist = [('top',color,'color',{'text':'Top background color'}),
-                ('bottom',color2,'color',{'text':'Bottom background color'}),
+    itemlist = [C('top',color,'color',{'text':'Top background color'}),
+                C('bottom',color2,'color',{'text':'Bottom background color'}),
                 ]
-    _the_dialog = widgets.InputDialog(itemlist,'Config Dialog')
+    _the_dialog = widgets.NewInputDialog(itemlist,'Config Dialog')
     res = _the_dialog.getResult()
     GD.debug(res)
     if res:
@@ -133,22 +113,19 @@ def setSlColor():
 def setLineWidth():
     """Change the default line width."""
     lw = GD.canvas.settings.linewidth
-    itemlist = [('Line Width', lw, 'float')]
-    res = widgets.InputDialog(itemlist,'Choose default line width').getResult()
+    itemlist = [C('Line Width', lw, 'float')]
+    res = widgets.NewInputDialog(itemlist,'Choose default line width').getResult()
     if res:
         GD.canvas.setLineWidth(res['Line Width'])
     
 def setCanvasSize():
     """Save the current viewport size"""
-    itemlist = [('w',GD.canvas.width()),('h',GD.canvas.height())]
-    res = widgets.InputDialog(itemlist,'Set Canvas Size').getResult()
+    itemlist = [C('w',GD.canvas.width()),('h',GD.canvas.height())]
+    res = widgets.NewInputDialog(itemlist,'Set Canvas Size').getResult()
     if res:
         GD.canvas.resize(int(res['w']),int(res['h']))
 
 
-
-
-
 def viewportSettings():
     """Interactively set the viewport settings."""
     mode = GD.canvas.rendermode
@@ -156,15 +133,15 @@ def viewportSettings():
     s = GD.canvas.settings
     if s.bgcolor2 is None:
         s.bgcolor2 = s.bgcolor
-    itemlist = [('rendermode', mode, 'select', modes),
-                ('linewidth', s.linewidth, 'float'),
-                ('bgcolor', s.bgcolor, 'color'),
-                ('bgcolor2', s.bgcolor2, 'color'),
-                ('fgcolor', s.fgcolor, 'color'),
-                ('slcolor', s.slcolor, 'color'),
-                ('Store these settings as defaults', False),
+    itemlist = [C('rendermode', mode, 'select', modes),
+                C('linewidth', s.linewidth, 'float'),
+                C('bgcolor', s.bgcolor, 'color'),
+                C('bgcolor2', s.bgcolor2, 'color'),
+                C('fgcolor', s.fgcolor, 'color'),
+                C('slcolor', s.slcolor, 'color'),
+                C('Store these settings as defaults', False),
                 ]
-    res = widgets.InputDialog(itemlist,'Config Dialog').getResult()
+    res = widgets.NewInputDialog(itemlist,'Config Dialog').getResult()
     if res:
         GD.debug(res)
         GD.canvas.updateSettings(res)
@@ -183,11 +160,11 @@ def viewportLayout():
         current = directions[0]
     else:
         current = directions[1]
-    itemlist = [('Number of viewports',len(GD.GUI.viewports.all)),
-                ('Viewport layout direction',current,'select',directions),
-                ('Number of viewports per row/column',GD.GUI.viewports.ncols),
+    itemlist = [C('Number of viewports',len(GD.GUI.viewports.all)),
+                C('Viewport layout direction',current,'select',{'choices':directions}),
+                C('Number of viewports per row/column',GD.GUI.viewports.ncols),
                 ]
-    res = widgets.InputDialog(itemlist,'Config Dialog').getResult()
+    res = widgets.NewInputDialog(itemlist,'Config Dialog').getResult()
     if res:
         GD.debug(res)
         nvps = res['Number of viewports']
@@ -202,12 +179,63 @@ def viewportLayout():
 #        if res['Store these settings as defaults']:
 #            GD.cfg.update()
 
+
+def canvasSettings():
+    dia = None
+    def apply_():
+        dia.acceptData()
+        set_near_clip(dia.results['near'])
+    def close():
+        dia.close()
+        
+    def set_near_clip(v):
+        dist = GD.canvas.camera.getDist()
+        GD.canvas.camera.setClip(10**v*dist,10.*dist)
+        GD.canvas.update()
+        
+    dia = widgets.NewInputDialog(
+        caption='Canvas Settings',
+        items=[
+            C('near',-1.0,'slider',{'min':-100,'max':100,'scale':0.01,'func': set_near_clip,'text':'Near clipping plane'}),
+            ],
+        actions=[('Done',close),('Apply',apply_)]
+        )
+    dia.show()
+
+
+def openglSettings():
+    dia = None
+    def apply_():
+        dia.acceptData()
+        canvas.glSettings(dia.results)
+    def close():
+        dia.close()
+        
+    dia = widgets.NewInputDialog(
+        caption='OpenGL Settings',
+        items=[
+            C('Line Smoothing','Off','radio',{'choices':['On','Off']}),
+            C('Polygon Mode',None,'radio',{'choices':['Fill','Line']}),
+            C('Polygon Fill',None,'radio',{'choices':['Front and Back','Front','Back']}),
+            C('Culling','Off','radio',{'choices':['On','Off']}),
+# These are currently set by the render mode
+#            ('Shading',None,'radio',{'choices':['Smooth','Flat']}),
+#            ('Lighting',None,'radio',{'choices':['On','Off']}),
+            ],
+        actions=[('Done',close),('Apply',apply_)]
+        )
+    dia.show()
+
+def lineSmoothOn():
+    canvas.glLineSmooth(True)
+
+def lineSmoothOff():
+    canvas.glLineSmooth(False)
+
 MenuData = [
     (_('&Viewport'),[
         (_('&Clear'),draw.clear),
-        (_('Toggle &Axes Triade'),draw.setTriade), 
-        (_('Set &Axes Triade Properties'),setTriadeParams), 
-        (_('Set Near and Far Clipping Planes'),setClip), 
+        (_('&Axes Triade'),setTriade), 
 #        (_('&Transparency'),setOpacity), 
         (_('&Background Color'),setBgColor), 
         (_('&Background 2Color'),setBgColor2), 
@@ -217,17 +245,21 @@ MenuData = [
         (_('&Canvas Size'),setCanvasSize), 
         (_('&All Viewport Settings'),viewportSettings),
         (_('&Global Draw Options'),draw.askDrawOptions),
-        ('&OpenGL Settings',
-         [('&Flat',canvas.glFlat),
-          ('&Smooth',canvas.glSmooth),
-          ('&Culling',canvas.glCulling),
-          ('&No Culling',canvas.glNoCulling),
-          ('&Polygon Line',canvas.glLine),
-          ('&Polygon Fill',canvas.glFill),
-          ('&Polygon Front Fill',canvas.glFrontFill),
-          ('&Polygon Back Fill',canvas.glBackFill),
-          ('&Polygon Front and Back Fill',canvas.glBothFill),
-          ]),
+        (_('&Canvas Settings'),canvasSettings),
+        (_('&OpenGL Settings'),openglSettings),
+        ## ('&OpenGL Settings',
+        ##  [('&Flat',canvas.glFlat),
+        ##   ('&Smooth',canvas.glSmooth),
+        ##   ('&Culling',canvas.glCulling),
+        ##   ('&No Culling',canvas.glNoCulling),
+        ##   ('&Line Smoothing On',lineSmoothOn),
+        ##   ('&Line Smoothing Off',lineSmoothOff),
+        ##   ('&Polygon Line',canvas.glLine),
+        ##   ('&Polygon Fill',canvas.glFill),
+        ##   ('&Polygon Front Fill',canvas.glFrontFill),
+        ##   ('&Polygon Back Fill',canvas.glBackFill),
+        ##   ('&Polygon Front and Back Fill',canvas.glBothFill),
+        ##   ]),
         (_('&Redraw'),draw.redraw),
         (_('&Reset'),draw.reset),
         (_('&Change viewport layout'),viewportLayout), 
