@@ -1,6 +1,6 @@
-/* $Id: miscmodule.c 1252 2009-12-09 11:05:19Z bverheg $ */
+/* $Id: miscmodule.c 1497 2010-06-05 08:50:48Z bverheg $ */
 //
-//  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+//  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 //  pyFormex is a tool for generating, manipulating and transforming 3D
 //  geometrical models by sequences of mathematical operations.
 //  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -22,13 +22,13 @@
 //  along with this program.  If not, see http://www.gnu.org/licenses/.
 //
 
-/*
-  A collection of accelerated pyFormex functions.
-*/
-
 #include <Python.h>
 #include <numpy/arrayobject.h>
 
+static char __doc__[] = "_misc module\n\
+\n\
+This module provides accelerated versions of some pyFormex functions.\n\
+\n";
 
 /**************************************************** isclose ****/
 /* Check if two floats are equal within a given tolerance */
@@ -44,7 +44,7 @@
 
 
 /**************************************************** coords.fuse ****/
-/* Fuse nodes */
+/* Fuse nodes : new and much faster algorithm */
 /* args:  x, val, flag, sel, tol
      x   : (nnod,3) coordinates
      val : (nnod) gives the point a code, such that only points with equal val
@@ -53,8 +53,82 @@
      sel : (nnod) 
      tol : tolerance for defining equality of coordinates
 */  
-static PyObject *
-coords_fuse(PyObject *dummy, PyObject *args)
+static PyObject * coords_fuse2(PyObject *dummy, PyObject *args)
+{
+  PyObject *arg1=NULL, *arg2=NULL, *arg3=NULL, *arg4=NULL;
+  PyObject *arr1=NULL, *arr2=NULL, *arr3=NULL, *arr4=NULL;
+  float *x;
+  int *flag;
+  int *val,*sel;
+  float tol;
+  if (!PyArg_ParseTuple(args, "OOOOf", &arg1, &arg2, &arg3, &arg4, &tol)) return NULL;
+  arr1 = PyArray_FROM_OTF(arg1, NPY_FLOAT, NPY_IN_ARRAY);
+  if (arr1 == NULL) return NULL;
+  arr2 = PyArray_FROM_OTF(arg2, NPY_INT, NPY_IN_ARRAY);
+  if (arr2 == NULL) goto fail;
+  arr3 = PyArray_FROM_OTF(arg3, NPY_INT, NPY_INOUT_ARRAY);
+  if (arr3 == NULL) goto fail;
+  arr4 = PyArray_FROM_OTF(arg4, NPY_INT, NPY_INOUT_ARRAY);
+  if (arr4 == NULL) goto fail;
+  /* We suppose the dimensions are correct*/
+  npy_intp * dims;
+  dims = PyArray_DIMS(arg1);
+  int nnod;
+  nnod = dims[0];
+  x = (float *)PyArray_DATA(arr1);
+  val = (int *)PyArray_DATA(arr2);
+  flag = (int *)PyArray_DATA(arr3);
+  sel = (int *)PyArray_DATA(arr4);
+  int i,j,ki,kj,nexti;
+
+  nexti = 1;
+  for (i=1; i<nnod; i++) {
+    j = i-1;
+    ki = 3*i;
+    while (j >= 0 && val[i]==val[j]) {
+      kj = 3*j;
+      if ( fabs(x[ki]-x[kj]) < tol &&
+	   fabs(x[ki+1]-x[kj+1]) < tol &&
+	   fabs(x[ki+2]-x[kj+2]) < tol ) {
+	flag[i] = 0;
+	sel[i] = sel[j];
+	break;
+      }
+      --j;
+    }
+    if (flag[i]) {
+      sel[i] = nexti;
+      ++nexti;
+    }
+  }
+  /* Clean up and return */
+  Py_DECREF(arr1);
+  Py_DECREF(arr2);
+  Py_DECREF(arr3);
+  Py_DECREF(arr4);
+  Py_INCREF(Py_None);
+  return Py_None;
+ fail:
+  printf("Error Cleanup\n");
+  Py_XDECREF(arr1);
+  Py_XDECREF(arr2);
+  Py_XDECREF(arr3);
+  Py_XDECREF(arr4);
+  return NULL;
+}
+
+
+/**************************************************** coords.fuse ****/
+/* Fuse nodes : new and much faster algorithm */
+/* args:  x, val, flag, sel, tol
+     x   : (nnod,3) coordinates
+     val : (nnod) gives the point a code, such that only points with equal val
+         are close. points in x are sorted according to val.
+     flag: (nnod) initially 1, set to 0 if point is fused with another.
+     sel : (nnod) 
+     tol : tolerance for defining equality of coordinates
+*/  
+static PyObject * coords_fuse(PyObject *dummy, PyObject *args)
 {
   PyObject *arg1=NULL, *arg2=NULL, *arg3=NULL, *arg4=NULL;
   PyObject *arr1=NULL, *arr2=NULL, *arr3=NULL, *arr4=NULL;
@@ -81,23 +155,15 @@ coords_fuse(PyObject *dummy, PyObject *args)
   flag = (int *)PyArray_DATA(arr3);
   sel = (int *)PyArray_DATA(arr4);
   int i,j,ki,kj;
-/*   printf("Fast Fuse Data:\n"); */
-/*   for (i=0; i<nnod; i++) { */
-/*     ki = 3*i; */
-/*     printf(" node %d: %12.8f, %12.8f, %12.8f; %lld; %d; %lld\n",i,x[ki],x[ki+1],x[ki+2],val[i],flag[i],sel[i]); */
-/*   } */
+
   for (i=0; i<nnod; i++) {
     ki = 3*i;
-/*     printf(" node %d: %12.8f, %12.8f, %12.8f; %lld; %d; %lld\n",i,x[ki],x[ki+1],x[ki+2],val[i],flag[i],sel[i]); */
-  
     j = i-1;
     while (j >= 0 && val[i]==val[j]) {
-/*       printf("Compare %d and %d\n",i,j); */
       kj = 3*j;
       if ( fabs(x[ki]-x[kj]) < tol && \
 	   fabs(x[ki+1]-x[kj+1]) < tol && \
 	   fabs(x[ki+2]-x[kj+2]) < tol ) {
-/* 	printf("Nodes %d and %d are close\n",i,j);	\ */
 	flag[i] = 0;
 	sel[i] = sel[j];
 	j = i+1;
@@ -107,6 +173,7 @@ coords_fuse(PyObject *dummy, PyObject *args)
       j = j-1;
     }
   }
+
   /* Clean up and return */
   Py_DECREF(arr1);
   Py_DECREF(arr2);
@@ -115,7 +182,7 @@ coords_fuse(PyObject *dummy, PyObject *args)
   Py_INCREF(Py_None);
   return Py_None;
  fail:
-  printf("Error Cleanup\n");
+  //printf("Error Cleanup\n");
   Py_XDECREF(arr1);
   Py_XDECREF(arr2);
   Py_XDECREF(arr3);
@@ -137,8 +204,7 @@ coords_fuse(PyObject *dummy, PyObject *args)
     (CURRENTLY NOT IMPLEMENTED)
     The operations are done in-place. The return value is None.
 */  
-static PyObject *
-nodal_sum(PyObject *dummy, PyObject *args)
+static PyObject * nodal_sum(PyObject *dummy, PyObject *args)
 {
   PyObject *arg1=NULL, *arg2=NULL, *arg3=NULL;
   PyObject *arr1=NULL, *arr2=NULL, *arr3=NULL;
@@ -205,8 +271,7 @@ nodal_sum(PyObject *dummy, PyObject *args)
     
     The operations are done in-place. The return value is None.
 */  
-static PyObject *
-nodal_sum_1(PyObject *dummy, PyObject *args)
+static PyObject * nodal_sum_1(PyObject *dummy, PyObject *args)
 {
   PyObject *arg1=NULL, *arg2=NULL, *arg3=NULL, *arg4=NULL;
   PyObject *arr1=NULL, *arr2=NULL, *arr3=NULL, *arr4=NULL;
@@ -270,17 +335,17 @@ nodal_sum_1(PyObject *dummy, PyObject *args)
 
 
 /* The methods defined in this module */
-static PyMethodDef Methods[] = {
-    {"fuse", coords_fuse, METH_VARARGS, "Fuse nodes."},
+static PyMethodDef __methods__[] = {
+    {"_fuse2", coords_fuse2, METH_VARARGS, "Fuse nodes."},
+    {"_fuse", coords_fuse, METH_VARARGS, "Fuse nodes."},
     {"nodalSum", nodal_sum, METH_VARARGS, "Nodal sum."},
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
 /* Initialize the module */
-PyMODINIT_FUNC
-initmisc(void)
+PyMODINIT_FUNC initmisc(void)
 {
-    (void) Py_InitModule("misc", Methods);
+  (void) Py_InitModule3("misc", __methods__, __doc__);
     import_array(); /* Get access to numpy array API */
 }
 
