@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: main.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: main.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -26,24 +26,48 @@
 import sys,os
 pyformexdir = sys.path[0]
 svnversion = os.path.exists(os.path.join(pyformexdir,'.svn'))
+startup_warnings = ''
+
+if svnversion:
+    libdir = os.path.join(pyformexdir,'lib')
+    libraries = [ 'miscmodule','drawglmodule' ]
+    for lib in libraries:
+        src = os.path.join(libdir,lib+'.c')
+        obj = os.path.join(libdir,lib+'.so')
+        if not os.path.exists(obj) or os.path.getmtime(obj) < os.path.getmtime(src):
+            startup_warnings += "\nThe compiled library '%s' is not up to date!" % lib
+    if startup_warnings:
+        startup_warnings += """
+        
+You should probably rebuild the pyFormex library first.
+
+Do 'make lib' in %s
+""" % pyformexdir
 
 import pyformex
-from pyformex import *
+import utils
+from config import Config
 
 import warnings
 warnings.filterwarnings('ignore','.*return_index.*',UserWarning,'numpy')
 
 
-import utils
-from config import Config
-
 ###########################  main  ################################
 
 def refLookup(key):
     """Lookup a key in the reference configuration."""
-    return pyformex.refcfg[key]
+    try:
+        return pyformex.refcfg[key]
+    except:
+        pyformex.debug("!There is no key '%s' in the reference config!"%key)
+        return None
 
 
+def prefLookup(key):
+    """Lookup a key in the reference configuration."""
+    return pyformex.prefcfg[key]
+    
+
 def printcfg(key):
     try:
         print("!! refcfg[%s] = %s" % (key,pyformex.refcfg[key]))
@@ -58,7 +82,6 @@ def setRevision():
         pyformex.__revision__ = "$Rev: %s $" % out.strip()
 
 
-
 def remove_pyFormex(pyformexdir,scriptdir):
     """Remove the pyFormex installation."""
     print("""
@@ -100,21 +123,61 @@ def savePreferences():
     if pyformex.preffile is None:
         return
     
-    del pyformex.cfg['__ref__']
+    del pyformex.prefcfg['__ref__']
 
-    # Dangerous to set permanently!
-    del pyformex.cfg['input/timeout']
-    
-    debug("!!!Saving config:\n%s" % pyformex.cfg)
+    # Currently erroroneously processed, therefore not saved
+    del pyformex.prefcfg['render']['light0']
+    del pyformex.prefcfg['render']['light1']
+    del pyformex.prefcfg['render']['light2']
+    del pyformex.prefcfg['render']['light3']
+
+    pyformex.options.debug = 1
+    pyformex.debug("="*60)
+    pyformex.debug("!!!Saving config:\n%s" % pyformex.prefcfg)
 
     try:
-        fil = file(pyformex.preffile,'w')
-        fil.write("%s" % pyformex.cfg)
-        fil.close()
+        pyformex.prefcfg.write(pyformex.preffile)
         res = "Saved"
     except:
         res = "Could not save"
-    debug("%s preferences to file %s" % (res,pyformex.preffile))
+    pyformex.debug("%s preferences to file %s" % (res,pyformex.preffile))
+
+
+def apply_config_changes(cfg):
+    """Apply incompatible changes in the configuration
+
+    cfg is the user configuration that is to be saved.
+    """
+    # Adhoc changes
+    if type(cfg['gui/dynazoom']) is str:
+        cfg['gui/dynazoom'] = [ cfg['gui/dynazoom'], '' ]
+
+    for i in range(8):
+        t = "render/light%s"%i
+        try:
+            cfg[t] = dict(cfg[t])
+        except:
+            pass
+
+    # Rename settings
+    for old,new in [
+        ('history','gui/history'),
+        ]:
+        if old in cfg.keys():
+            if new not in cfg.keys():
+                cfg[new] = cfg[old]
+            del cfg[old]
+
+    # Delete settings
+    for key in [
+        'input/timeout',
+        'render/ambient','render/diffuse','render/specular','render/emission',
+        ]:
+        if key in cfg.keys():
+            # Can we put GD.debug here?
+            print("DELETING CONFIG VARIABLE %s" % key)
+            del cfg[key]
+
 
 ###########################  app  ################################
 
@@ -155,10 +218,16 @@ def run(argv=[]):
     pyformex.cfg.read(defaults)
     
     # Process options
-    from optparse import OptionParser,make_option as MO
-    parser = OptionParser(
-        usage = "usage: %prog [<options>] [ --  <Qapp-options> ] [[ scriptname [scriptargs]] ...]",
+    import optparse
+    from optparse import make_option as MO
+    parser = optparse.OptionParser(
+        # THE Qapp options are removed, because it does not seem to work !!!
+        # SEE the comments in the gui.startGUI function  
+        # usage = "usage: %prog [<options>] [ --  <Qapp-options> ] [[ scriptname [scriptargs]] ...]",
+        usage = "usage: %prog [<options>] [[ scriptname [scriptargs]] ...]",
         version = pyformex.Version,
+        description = pyformex.Description,
+        formatter = optparse.TitledHelpFormatter(),
         option_list=[
         MO("--gui",
            action="store_true", dest="gui", default=None,
@@ -212,38 +281,42 @@ def run(argv=[]):
            action="store_true", dest="redirect", default=False,
            help="Redirect standard output to the message board (ignored with --nogui)",
            ),
-        MO("--detect",
-           action="store_true", dest="detect", default=False,
-           help="Detect helper software and print report.",
-           ),
         MO("--debug",
-           action="store_true", dest="debug", default=False,
+           action="store_const", dest="debug", const=-1,
            help="display debugging info to sys.stdout",
            ),
-        MO("--classify",
-           action="store_true", dest="classify", default=False,
-           help="classify the examples in categories",
-           ),
-        MO("--whereami",
-           action="store_true", dest="whereami", default=False,
-           help="show where the pyformex package is located",
-           ),
-        MO("--remove",
-           action="store_true", dest="remove", default=False,
-           help="remove the pyformex installation",
+        MO("--debuglevel",
+           action="store", dest="debug", type="int", default=0,
+           help="display debugging info to sys.stdout",
            ),
+        ## MO("--classify",
+        ##    action="store_true", dest="classify", default=False,
+        ##    help="classify the examples in categories",
+        ##    ),
         MO("--test",
            action="store_true", dest="test", default=False,
            help="testing mode: only for developers!",
            ),
-        MO("--testhighlight",
-           action="store_true", dest="testhighlight", default=False,
-           help="highlight testing mode: only for developers!",
+        MO("--testfpbug",
+           action="store_true", dest="fpbug", default=False,
+           help="testing mode: only for developers!",
            ),
-        MO("--executor",
+        MO("--testexecutor",
            action="store_true", dest="executor", default=False,
            help="test alternate executor: only for developers!",
            ),
+        MO("--remove",
+           action="store_true", dest="remove", default=False,
+           help="remove the pyformex installation and exit",
+           ),
+        MO("--whereami",
+           action="store_true", dest="whereami", default=False,
+           help="show where the pyformex package is installed and exit",
+           ),
+        MO("--detect",
+           action="store_true", dest="detect", default=False,
+           help="show detected helper software and exit",
+           ),
         ])
     pyformex.options, args = parser.parse_args(argv)
     pyformex.print_help = parser.print_help
@@ -254,23 +327,23 @@ def run(argv=[]):
         print("\nInvalid options: --nodefaultconfig but no --config option\nDo pyformex --help for help on options.\n")
         sys.exit()
 
-
-    if pyformex.options.whereami:
-        print("Script started from %s" % pyformex.scriptdir)
-        print("I found pyFormex in %s " %  pyformexdir)
-        print("Current Python sys.path: %s" % sys.path)
-        sys.exit()
-        
+    pyformex.debug("Options: %s" % pyformex.options)
+   
     if pyformex.options.remove:
         remove_pyFormex(pyformexdir,pyformex.scriptdir)
         
-    if pyformex.options.detect:
+    if pyformex.options.whereami or pyformex.options.debug :
+        print("Script started from %s" % pyformex.scriptdir)
+        print("I found pyFormex in %s " %  pyformexdir)
+        print("Current Python sys.path: %s" % sys.path)
+
+    if pyformex.options.detect or pyformex.options.debug :
         print("Detecting all installed helper software")
         utils.checkExternal()
         print(utils.reportDetected())
+
+    if pyformex.options.whereami or pyformex.options.detect:
         sys.exit()
-        
-    pyformex.debug("Options: %s" % pyformex.options)
 
     ########### Read the config files  ####################
 
@@ -300,7 +373,6 @@ def run(argv=[]):
         if os.path.exists(pyformex.cfg.localprefs):
             userprefs.append(pyformex.cfg.localprefs)
 
-
     if pyformex.options.config:
         userprefs.append(pyformex.options.config)
 
@@ -315,14 +387,22 @@ def run(argv=[]):
         pyformex.debug("Reading config file %s" % f)
         pyformex.cfg.read(f)
     pyformex.refcfg = pyformex.cfg
+    pyformex.debug("="*60)
     pyformex.debug("RefConfig: %s" % pyformex.refcfg)
 
     # Use the last as place to save preferences
-    pyformex.cfg = Config(default=refLookup)
+    pyformex.prefcfg = Config(default=refLookup)
     if os.path.exists(pyformex.preffile):
         pyformex.debug("Reading config file %s" % pyformex.preffile)
-        pyformex.cfg.read(pyformex.preffile)
-    pyformex.debug("Config: %s" % pyformex.cfg)
+        pyformex.prefcfg.read(pyformex.preffile)
+    pyformex.debug("="*60)
+    pyformex.debug("Config: %s" % pyformex.prefcfg)
+
+    # Fix incompatible changes in configuration
+    apply_config_changes(pyformex.prefcfg)
+
+    # Create an empty one for the session settings
+    pyformex.cfg = Config(default=prefLookup)
 
     # Set option from config if it was not explicitely given
     if pyformex.options.uselib is None:
@@ -335,23 +415,35 @@ def run(argv=[]):
     if pyformex.options.gui:
         pyformex.options.interactive = True
 
-    # Set Revision if we run from an SVN version
+    # Set Revision and run svnclean if we run from an SVN version
     if svnversion:
         setRevision()
-
-    pyformex.debug(utils.reportDetected())
-
-    #print("pyformex",utils.subDict(pyformex.cfg,'canvas/'))
-    #print("DEFAULT",self.default)
-    #print(pyformex.cfg.keys())
-    #print(pyformex.refcfg.keys())
-    
+        svnclean = './pyformex/svnclean'
+        if os.path.exists(svnclean):
+            try:
+                utils.runCommand(svnclean)
+            except:
+                print("Error while executing %s, we ignore it and continue" % svnclean)
+                
     # Start the GUI if needed
     # Importing the gui should be done after the config is set !!
     if pyformex.options.gui:
+        from gui import guimain
         pyformex.debug("GUI version")
-        from gui import gui
-        gui.startGUI(args)
+        res = guimain.startGUI(args)
+        if res != 0:
+            print("Could not start the pyFormex GUI: %s" % res)
+            return res # EXIT
+        
+    # Display the startup warnings
+    if startup_warnings:
+        pyformex.warning(startup_warnings)
+
+    pyformex.debug(utils.reportDetected())
+ 
+    #print(pyformex.cfg.keys())
+    #print(pyformex.refcfg.keys())
+      
     #
     # Qt4 may have changed the locale.
     # Since a LC_NUMERIC setting other than C may cause lots of troubles
@@ -386,7 +478,7 @@ def run(argv=[]):
             if pyformex.options.gui:
                 pyformex.message("There was an error while executing a script")
             else:
-                return res
+                return res # EXIT
                 
     else:
         pyformex.debug("stdin is a tty: %s" % sys.stdin.isatty())
@@ -398,7 +490,7 @@ def run(argv=[]):
 
     # after processing all args, go into interactive mode
     if pyformex.options.gui:
-        res = gui.runGUI()
+        res = guimain.runGUI()
 
     ## elif pyformex.options.interactive:
     ##     print("Enter your script and end with CTRL-D")
