@@ -1,6 +1,6 @@
-# $Id: misc.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: misc.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,6 +31,49 @@ external functions in the compiled library.
 # There should be no other imports here than array
 from pyformex.arraytools import *
 
+
+def _fuse2(x,val,flag,sel,tol):
+    """Fusing nodes.
+
+    This is a low level function performing the internal loop of
+    the fuse operation. It is not intended to be called by the user.
+    """
+    nnod = val.shape[0]
+    nexti = 1
+    for i in range(1,nnod):
+        j = i-1
+        while j>=0 and val[i]==val[j]:
+            if abs(x[i]-x[j]).max() < tol:
+                # node i is same as previous node j
+                flag[i] = 0
+                sel[i] = sel[j]
+                break
+            j = j-1
+        if flag[i]:
+            # node i is a new node
+            sel[i] = nexti
+            nexti += 1
+
+
+def _fuse(x,val,flag,sel,tol):
+    """Fusing nodes.
+
+    This is a low level function performing the internal loop of
+    the fuse operation. It is not intended to be called by the user.
+    """
+    nnod = val.shape[0]
+    for i in range(nnod):
+        j = i-1
+        while j>=0 and val[i]==val[j]:
+            if allclose(x[i],x[j],rtol=tol,atol=tol):
+                # node i is same as node j
+                flag[i] = 0
+                sel[i] = sel[j]
+                sel[i+1:nnod] -= 1
+                break
+            j = j-1
+
+
 def nodalSum(val,elems,work,avg):
     """Compute the nodal sum of values defined on elements.
 
