@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: odict.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: odict.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,6 +28,10 @@
 import olist
 
 
+def __newobj__(cls, *args):
+    return cls.__new__(cls, *args)
+
+
 class ODict(dict):
     """**An ordered dictionary.**
 
@@ -41,13 +45,46 @@ class ODict(dict):
     """
     def __init__(self,data={}):
         """Create a new ODict instance."""
-        if type(data) is list or type(data) is tuple:
-            dict.__init__(self,data)
-            self._order = [ i[0] for i in data ]
-        else:
-            dict.__init__(self,{})
+        dict.__init__(self,data)
+
+        if isinstance(data,ODict):
+            # keep order
+            self._order = data._order
+            
+        elif type(data) is list or type(data) is tuple:
+            # preserve the order
             self._order = []
-            self.update(data)
+            self._add_keys([i[0] for i in data])
+            
+        elif type(data) is dict:
+            # order is undefined
+            self._order = data.keys()
+
+        else:
+            raise ValueError,"Unexpected initialization value for ODict"
+  
+
+    def _add_keys(self,keys):
+        """Add a list of keys to the ordered list, removing existing keys."""
+        for k in keys:
+            if k in self._order:
+                self._order.remove(k)
+        self._order += keys
+        
+        
+    def update(self,data={}):
+        """Add a dictionary to the ODict object.
+
+        The new keys will be appended to the existing, but the order of the
+        added keys is undetemined if data is a dict object. If data is an ODict
+        its order will be respected.. 
+        """
+        dict.update(self,data)
+        self._add_keys(ODict(data)._order)
+
+
+    def __iter__(self):
+        return list.__iter__(self._order)
 
 
     def __repr__(self):
@@ -79,24 +116,6 @@ class ODict(dict):
         dict.__delitem__(self,key)
         self._order.remove(key)
 
-        
-    def update(self,data={}):
-        """Add a dictionary to the ODict object.
-
-        The new keys will be appended to the existing, but the order of the
-        added keys is undetemined if data is a dict object. If data is an ODict
-        its order will be respected.. 
-        """
-        dict.update(self,data)
-        if type(data) is ODict:
-            for k in data._order:
-                if k in self._order:
-                    self._order.remove(k)
-            self._order += data._order
-        else:
-            #print(data)
-            self._order.extend(data.keys())
-
 
     def __add__(self,data):
         """Add two ODicts's together, returning the result."""
@@ -127,7 +146,17 @@ class ODict(dict):
     def items(self):
         """Return the key,value pairs in order of the keys."""
         return [(k,self[k]) for k in self._order]
- 
+
+
+    def pos(self,key):
+        """Return the position of the specified key.
+
+        If the key is not in the ODict, None is returned"""
+        try:
+            return self._order.index(key)
+        except ValueError:
+            return None
+        
 
     def __reduce__(self):
         state = (dict(self), self.__dict__)
@@ -144,9 +173,6 @@ class ODict(dict):
             self.update(state)
 
 
-def __newobj__(cls, *args):
-    return cls.__new__(cls, *args)
-
 
 class KeyedList(ODict):
     """A named item list.
@@ -179,10 +205,12 @@ class KeyedList(ODict):
 
 if __name__ == "__main__":
 
-    d = ODict({'a':1,'b':2,'c':3})
+    d = ODict({'a':1,'b':2,'c':3,'a':3})
     print(d)
     d.sort(['a','b','c'])
     print(d)
+    d = ODict([('a',1),('b',2),('c',3),('a',3)])
+    print d
     d['d'] = 4
     d['e'] = 5
     d['f'] = 6
