@@ -1,6 +1,7 @@
 #!/usr/bin/env python
+# $Id$
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -249,7 +250,7 @@ def removeTriangles(elems):
     Triangles appearing in the centerline are removed by this function.
     Both input and output are the connectivity of the centerline.
     """
-    rev = connectivity.reverseIndex(elems)
+    rev = connectivity(elems).inverse()
     if rev.shape[1] > 2:
         w =  where(rev[:,-3] != -1)[0]
         for i in w:
