@@ -1,7 +1,6 @@
-#!/usr/bin/env pyformex --gui
-# $Id: curve.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: curve.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -38,6 +37,8 @@ but special cases may be created for handling plane curves.
 #from pyformex import debug
 from numpy import *
 from formex import *
+from plugins.geomtools import triangleCircumCircle
+from plugins.mesh import Mesh
 
 
 ##############################################################################
@@ -52,10 +53,13 @@ from formex import *
 # methods:
 #    subPoints(t,j): returns points with parameter values t of part j
 #    points(ndiv,extend=[0.,0.]): returns points obtained by dividing each
-#           part in ndiv sections at equal parameter distance.             
+#           part in ndiv sections at equal parameter distance.
+#    pointsOn(): the defining points situated on the curve
+#    pointsOff(): the defining points situated off the curve (control points)
 
+from geometry import Geometry
 
-class Curve(object):
+class Curve(Geometry):
     """Base class for curve type classes.
 
     This is a virtual class intended to be subclassed.
@@ -66,6 +70,18 @@ class Curve(object):
     """
 
     N_approx = 10
+
+    def pointsOn(self):
+        return self.coords
+
+    def pointsOff(self):
+        return Coords()
+
+    def ncoords(self):
+        return self.coords.shape[0]
+
+    def npoints(self):
+        return self.pointsOn().shape[0]
     
     def sub_points(self,t,j):
         """Return the points at values t in part j
@@ -156,18 +172,41 @@ class Curve(object):
         return self.lengths().sum()
 
 
-    def approx(self,ndiv=N_approx):
-        """Return an approximate PolyLine"""
-        return PolyLine(self.subPoints(ndiv),closed=self.closed)
+    def approx(self,ndiv=None,ntot=None):
+        """Return a PolyLine approximation of the curve
+
+        If no `ntot` is given, the curve is approximated by `ndiv`
+        straight segments over each part of the curve.
+        If `ntot` is given, the curve is approximated by `ntot`
+        straight segments over the total curve. This is based on a
+        first approximation with ndiv segments over each part.
+        """
+        if ndiv is None:
+            ndiv = self.N_approx
+        PL = PolyLine(self.subPoints(ndiv),closed=self.closed)
+        if ntot is not None:
+            at = PL.atLength(ntot)
+            X = PL.pointsAt(at)
+            PL = PolyLine(X,closed=PL.closed)
+        return PL
 
 
-    # This allows us to draw approximations of curves that do not specify
-    # their own (hopefully better) Formex representation
-    def toFormex(self):
-        return self.approx().toFormex()
-        
+    def toFormex(self,*args,**kargs):
+        """Convert a curve to a Formex.
+
+        This creates a polyline approximation as a plex-2 Formex.
+        This is mainly used for drawing curves that do not implement
+        their own drawing routines.
+
+        The method can be passed the same arguments as the `approx` method.
+        """
+        return self.approx(*args,**kargs).toFormex()
   
 
+##############################################################################
+#
+#  Curves that can be transformed by Coords Transforms
+#
 ##############################################################################
 #
 class PolyLine(Curve):
@@ -181,7 +220,17 @@ class PolyLine(Curve):
         If closed == True, the polyline is closed by connecting the last
         point to the first. This does not change the vertex data.
         """
-        coords = Coords(coords)
+        if isinstance(coords,Formex):
+            if coords.nplex() == 1:
+                coords = coords.coords
+            elif coords.nplex() == 2:
+                coords = Coords.concatenate([coords.coords[:,0,:],coords.coords[-1,1,:]])
+            else:
+                raise ValueError,"Only Formices with plexitude 1 or 2 can be converted to PolyLine"
+
+        else:
+            coords = Coords(coords)
+            
         if coords.ndim != 2 or coords.shape[1] != 3:
             raise ValueError,"Expected an (npoints,3) coordinate array"
         self.coords = coords
@@ -189,6 +238,10 @@ class PolyLine(Curve):
         if not closed:
             self.nparts -= 1
         self.closed = closed
+
+
+    def nelems(self):
+        return self.nparts
     
 
     def toFormex(self):
@@ -197,6 +250,20 @@ class PolyLine(Curve):
         F = connect([x,x],bias=[0,1],loop=self.closed)
         return F
 
+    
+    def toMesh(self):
+        """Convert the polyLine to a plex-2 Mesh.
+
+        This returned mesh is equivalent with the PolyLine, but does
+        not guarantee the sequential order.
+        """
+        e1 = arange(self.ncoords())
+        elems = column_stack([e1,roll(e1,-1)])
+        if not self.closed:
+            elems = elems[:-1]
+        return Mesh(self.coords,elems,eltype='line2')
+
+
     def sub_points(self,t,j):
         """Return the points at values t in part j"""
         j = int(j)
@@ -220,40 +287,92 @@ class PolyLine(Curve):
 
 
     def vectors(self):
-        """Return the vectors of the points to the next one.
+        """Return the vectors of each point to the next one.
 
         The vectors are returned as a Coords object.
-        If not closed, this returns one less vectors than the number of points.
+        If the curve is not closed, the number of vectors returned is
+        one less than the number of points.
         """
         x = self.coords
-        y = roll(x,-1,axis=0)
-        if not self.closed:
-            n = self.coords.shape[0] - 1
-            x = x[:n]
-            y = y[:n]
-        return y-x
-
-
-    def directions(self):
-        """Returns unit vectors in the direction of the next point."""
-        return normalize(self.vectors())
-
-
-    def avgDirections(self,normalized=True):
-        """Returns average directions at the inner nodes.
-
-        If open, the number of directions returned is 2 less than the
-        number of points.
-        """
-        d = self.directions()
         if self.closed:
-            d1 = d
-            d2 = roll(d,1,axis=0)
+            x1 = x
+            x2 = roll(x,-1,axis=0) # NEXT POINT
         else:
-            d1 = d[:-1]
-            d2 = d[1:]
-        d = 0.5*(d1+d2)
-        return d
+            x1 = x[:-1]
+            x2 = x[1:]
+        return x2-x1
+        
+
+    def directions(self,return_doubles=False):
+        """Returns unit vectors in the direction of the next point.
+
+        This directions are returned as a Coords object with the same
+        number of elements as the point set.
+        
+        If two subsequent points are identical, the first one gets
+        the direction of the previous segment. If more than two subsequent
+        points are equal, an invalid direction (NaN) will result.
+
+        If the curve is not closed, the last direction is set equal to the
+        penultimate.
+
+        If return_doubles is True, the return value is a tuple of the direction
+        and an index of the points that are identical with their follower.
+        """
+        import warnings
+        warnings.warn('PolyLine.directions() now always returns the same number of directions as there are points. The last direction of an open PolyLine appears twice.')
+        d = normalize(self.vectors())
+        w = where(isnan(d).any(axis=-1))[0]
+        #print "DOUBLES:%s" % w
+        d[w] = d[w-1]  
+        if not self.closed:
+            d = concatenate([d,d[-1:]],axis=0)
+        #print "Directions",d
+        if return_doubles:
+            return d,w
+        else:
+            return d
+    
+
+    def avgDirections(self,return_doubles=False):
+        """Returns the average directions at points.
+
+        For each point the returned direction is the average of the direction
+        from the preceding point to the current, and the direction from the
+        current to the next point.
+        
+        If the curve is open, the first and last direction are equal to the
+        direction of the first, resp. last segment.
+
+        Where two subsequent points are identical, the average directions
+        are set equal to those of the segment ending in the first and the
+        segment starting from the last.
+        """
+        import warnings
+        warnings.warn('PolyLine.avgDirections() now always returns the same number of directions as there are points. The first and last direction are those of the end segment.')
+        d,w = self.directions(True)
+        #if self.closed:
+        d1 = d
+        d2 = roll(d,1,axis=0) # PREVIOUS DIRECTION
+        #else:
+        #    d1 = d[:-1]
+        #    d2 = d[1:]
+        ## if not self.closed:
+        ##     d = concatenate([d[:1],d],axis=0)
+        #print "expanded",d
+        w = concatenate([w,w+1])
+        if not self.closed:
+            w = concatenate([[0,self.npoints()-1],w])
+        #print "Not averaging at %s" % w
+        w = setdiff1d(arange(self.npoints()),w)
+        #d[w] = dd[w]
+        #print "Averaging at %s" % w
+        d[w] = 0.5 * (d1[w]+d2[w])
+        #print "AVG Directions",d
+        if return_doubles:
+            return d,w
+        else:
+            return d
     
 
     def lengths(self):
@@ -265,7 +384,7 @@ class PolyLine(Curve):
         """Returns the parameter values for relative curve lengths div.
         
         ``div`` is a list of relative curve lengths (from 0.0 to 1.0).
-        As a convenience, an single integer value may be specified,
+        As a convenience, a single integer value may be specified,
         in which case the relative curve lengths are found by dividing
         the interval [0.0,1.0] in the specified number of subintervals.
 
@@ -287,9 +406,88 @@ class PolyLine(Curve):
         at[wi] = ai
         return at
 
+
     def reverse(self):
         return PolyLine(reverseAxis(self.coords,axis=0),closed=self.closed)
-        
+
+
+    def split(self,i):
+        """Split the curve at point i.
+
+        Returns a list of open PolyLines: one, if the PolyLine is closed or
+        i is one of the endpoints of an open PolyLine, two in other cases.
+        """
+        res = []
+        if self.closed:
+            res.append(PolyLine(roll(self.coords,-i,axis=0)))
+        else:
+            if i > 0:
+                res.append(PolyLine(self.coords[:i+1]))
+            if i < len(self.coords):
+                res.append(PolyLine(self.coords[i:]))
+        return res
+
+
+    def cutWithPlane(self,p,n,side=''):
+        """Return the parts of the polyline at one or both sides of a plane.
+
+        If side is '+' or '-', return a list of PolyLines with the parts at
+        the positive or negative side of the plane.
+
+        For any other value, returns a tuple of two lists of PolyLines,
+        the first one being the parts at the positive side.
+
+        p is a point specified by 3 coordinates.
+        n is the normal vector to a plane, specified by 3 components.
+        """
+        n = asarray(n)
+        p = asarray(p)
+
+        d = self.coords.distanceFromPlane(p,n)
+        t = d > 0.0
+        cut = t != roll(t,-1)
+        if not self.closed:
+            cut = cut[:-1]
+        w = where(cut)[0]
+
+        res = [[],[]]
+        i = 0
+        if t[0]:
+            sid = 0
+        else:
+            sid = 1
+        Q = Coords()
+
+        for j in w:
+            #print "%s -- %s" % (i,j)
+            P = Formex([self.coords[j:j+2]]).intersectionPointsWithPlane(p,n)
+            P = P.coords.reshape(1,3)
+            #print "%s -- %s cuts at %s" % (j,j+1,P)
+            x = Coords.concatenate([Q,self.coords[i:j+1],P])
+            #print "%s + %s + %s = %s" % (Q.shape[0],j-i,P.shape[0],x.shape[0])
+            res[sid].append(PolyLine(x))
+            sid = 1-sid
+            i = j+1
+            Q = P
+
+        x = Coords.concatenate([Q,self.coords[i:]])
+        #print "%s + %s = %s" % (Q.shape[0],j-i,x.shape[0])
+        res[sid].append(PolyLine(x))
+        if self.closed:
+            if len(res[sid]) > 1:
+                x = Coords.concatenate([res[sid][-1].coords,res[sid][0].coords])
+                res[sid] = res[sid][1:-1]
+                res[sid].append(PolyLine(x))
+            #print [len(r) for r in res]
+            if len(res[sid]) == 1 and len(res[1-sid]) == 0:
+                res[sid][0].closed = True
+
+        # DO not use side in '+-', because '' in '+-' returns True
+        if side in ['+','-']:
+            return res['+-'.index(side)]
+        else:
+            return res
+
 
 ##############################################################################
 #
@@ -300,13 +498,10 @@ class Polygon(PolyLine):
         PolyLine.__init__(self,coords,closed=True)
         
 
-
-
 ##############################################################################
 #
-
 class BezierSpline(Curve):
-    """A class representing a Bezier spline curve."""
+    """A class representing a cubic Bezier spline curve."""
     coeffs = matrix([[-1.,  3., -3., 1.],
                      [ 3., -6.,  3., 0.],
                      [-3.,  3.,  0., 0.],
@@ -316,58 +511,100 @@ class BezierSpline(Curve):
     def __init__(self,coords,deriv=None,curl=0.5,control=None,closed=False):
         """Create a cubic spline curve through the given points.
 
-        The curve is defined by the points and the directions at these points.
-        If no directions are specified, the average of the segments ending
-        in that point is used, and in the end points of an open curve, the
-        direction of the end segment.
-        The curl parameter can be set to influence the curliness of the curve.
-        curl=0.0 results in straight segments.
+        - coords: an (npoints,3) array with ordered points on the curve
+        - deriv: an (npoints,3) or (2,3) array with the directions at all
+          the points or at the two endpoints only.
+          For points where the direction is left unspecified or where the
+          specified direction contains a `NaN` value, the direction
+          is calculated as the average direction of the two
+          line segments ending in the point. This will also be used
+          for points where the specified direction contains a value `NaN`.
+          In the two endpoints of an open curve however, this average
+          direction can not be calculated: the two control points in these
+          parts are set coincident.
+          
+          The curl parameter can be set to influence the curliness of the curve
+          in between two subsequent points. A value curl=0.0 results in
+          straight segments.
         
-        The control points can also be specified directly. If they are, they
-        override the deriv and curl parameters. Since each segment of the curve
-        needs two control points, the control array has shape (npts-1, 2, 3).
+        - control: an (nparts,2,3) array of control points, two for each
+          curve segemetn between two consecutive points. For a closed curve,
+          nparts = npoints, while for an open curve nparts = npoints-1.
+          If the control points are specified directly, they override
+          the deriv and curl parameters.
+        - closed: if True, the curve will be continued from the last point
+          to the first to create a closed curve.
         """
         coords = Coords(coords)
-        self.coords = coords
-        self.nparts = self.coords.shape[0]
+        ncoords = nparts = coords.shape[0]
         if not closed:
-            self.nparts -= 1
-            
+            nparts -= 1
+
         if control is None:
-            if self.nparts < 2:
-                control = self.coords
+            P = PolyLine(coords,closed=closed)
+            ampl = P.lengths().reshape(-1,1)
+            if deriv is None:
+                deriv = array([[nan,nan,nan]]*ncoords)
             else:
-                P = PolyLine(coords,closed=closed)
-                if deriv is None:
-                    deriv = P.avgDirections()
-                ampl = P.lengths().reshape(-1,1)
-                if not closed:
-                    coords = coords[1:-1]
-                if not closed:
-                    p1 = coords + deriv*curl*ampl[1:]
-                    p2 = coords - deriv*curl*ampl[:-1]
-                else:
-                    p1 = coords + deriv*curl*ampl
-                    p2 = coords - deriv*curl*roll(ampl,1,axis=0)
-                if not closed:
-                    p1 = concatenate([p2[:1],p1],axis=0)
-                    p2 = concatenate([p2,p1[-1:]],axis=0)
-                else:
-                    p2 = roll(p2,-1,axis=0)
-                control = concatenate([p1,p2],axis=1)
+                deriv = Coords(deriv)
+                nderiv = deriv.shape[0]
+                if nderiv < ncoords:
+                    if nderiv != 2:
+                        raise ValueError,"Either all or 2 directions expected (got %s)" % nderiv
+                    deriv = concatenate([
+                        deriv[:1],
+                        [[nan,nan,nan]]*(ncoords-2),
+                        deriv[-1:]])
+
+            undefined = isnan(deriv).any(axis=-1)
+            if undefined.any():
+                deriv[undefined] = P.avgDirections()[undefined]
+                
+            if closed:
+                p1 = coords + deriv*curl*ampl
+                p2 = coords - deriv*curl*roll(ampl,1,axis=0)
+                p2 = roll(p2,-1,axis=0)
+            else:
+                p1 = coords[:-1] + deriv[:-1]*curl*ampl
+                p2 = coords[1:] - deriv[1:]*curl*ampl
+                if isnan(p1[0]).any():
+                    p1[0] = p2[0]
+                if isnan(p2[-1]).any():
+                    p2[-1] = p1[-1]
+            control = concatenate([p1,p2],axis=1)
+                
         control = asarray(control).reshape(-1,2,3)
-        self.control = Coords(control)
-        if self.control.shape != (self.nparts,2,3):
-            print("coords array has shape %s" % str(self.coords.shape))
-            print("control array has shape %s" % str(self.control.shape))
-            raise ValueError,"Invalid control points for Bezier Spline"
+        control = Coords(control)
+        if control.shape != (nparts,2,3):
+            print("coords array has shape %s" % str(coords.shape))
+            print("control array has shape %s" % str(control.shape))
+            raise ValueError,"Invalid control points for BezierSpline"
+
+        # To enable easy transforms, we join the coords and controls
+        # in a single array
+        if not closed:
+            dummy = Coords([[[0.,0.,0.],[0.,0.,0.]]])
+            control = Coords.concatenate([control,dummy],axis=0)
+        coords = coords[:,newaxis,:]
+        self.coords = Coords.concatenate([coords,control],axis=1)
+        self.nparts = nparts
         self.closed = closed
 
 
+    def pointsOn(self):
+        """Return the points on the curve""" 
+        return self.coords[:,0,:]
+
+
+    def pointsOff(self):
+        """Return the points off the curve (the control points)""" 
+        return self.coords[:self.nparts,1:,:]
+
+
     def sub_points(self,t,j):
         j1 = (j+1) % self.coords.shape[0]
-        P = self.coords[[j,j1]]
-        D = self.control[j]
+        P = self.pointsOn()[[j,j1]]
+        D = self.pointsOff()[j]
         P = concatenate([ P[0],D[0],D[1],P[1] ],axis=0).reshape(-1,3)
         C = self.coeffs * P
         U = column_stack([t**3., t**2., t, ones_like(t)])
@@ -376,77 +613,83 @@ class BezierSpline(Curve):
 
 ##############################################################################
 #
-
 class QuadBezierSpline(Curve):
-    """A class representing a Bezier spline curve."""
+    """A class representing a Quadratic Bezier spline curve."""
     coeffs = matrix([[ 1., -2.,  1.],
                      [-2.,  2.,  0.],
                      [ 1.,  0.,  0.]]
                     )
 
-    def __init__(self,coords,deriv=None,control=None,closed=False):
-        """Create a cubic spline curve through the given points.
+    def __init__(self,coords,control=None,closed=False):
+        """Create a quadratic spline curve through the given points.
 
-        The curve is defined by the points and the directions at these points.
-        If no directions are specified, the average of the segments ending
-        in that point is used, and in the end points of an open curve, the
-        direction of the end segment.
-        The curl parameter can be set to influence the curliness of the curve.
-        curl=0.0 results in straight segments. curl=1.0 
-        The control points can also be specified directly. If they are, they
-        override the deriv and curl parameters. Since each segment of the curve
-        needs two control points, the control array has shape (npts-1, 2, 3).
         """
         coords = Coords(coords)
-        self.coords = coords
-        self.nparts = self.coords.shape[0]
+        nparts = coords.shape[0]
         if not closed:
-            self.nparts -= 1
+            nparts -= 1
             
         if control is None:
-            if self.nparts < 2:
-                control = self.coords
+            if nparts < 2:
+                control = 0.5*(coords[:-1] + coords[1:])
             else:
-                P = PolyLine(coords,closed=closed)
-                if deriv is None:
-                    deriv = P.avgDirections()
-                ampl = P.lengths().reshape(-1,1)
-                if not closed:
-                    coords = coords[1:-1]
-                if not closed:
-                    p1 = coords + deriv*curl*ampl[1:]
-                    p2 = coords - deriv*curl*ampl[:-1]
+                if closed:
+                    P0 = 0.5 * (roll(coords,1,axis=0) + roll(coords,-1,axis=0))
+                    P1 = 2*coords - P0
+                    Q0 = 0.5*(roll(coords,1,axis=0) + P1)
+                    Q1 = 0.5*(roll(coords,-1,axis=0) + P1)
+                    Q = 0.5*(roll(Q0,-1,axis=0)+Q1)
+                    control = Q
                 else:
-                    p1 = coords + deriv*curl*ampl
-                    p2 = coords - deriv*curl*roll(ampl,1,axis=0)
-                if not closed:
-                    p1 = concatenate([p2[:1],p1],axis=0)
-                    p2 = concatenate([p2,p1[-1:]],axis=0)
-                else:
-                    p2 = roll(p2,-1,axis=0)
-                control = concatenate([p1,p2],axis=1)
+                    P0 = 0.5 * (coords[:-2] + coords[2:])
+                    P1 = 2*coords[1:-1] - P0
+                    Q0 = 0.5*(coords[:-2] + P1)
+                    Q1 = 0.5*(coords[2:] + P1)
+                    Q = 0.5*(Q0[1:]+Q1[:-1])
+                    control = Coords.concatenate([Q0[:1],Q,Q1[-1:]],axis=0)
+
         control = asarray(control).reshape(-1,3)
-        self.control = Coords(control)
-        if self.control.shape != (self.nparts,3):
-            print("coords array has shape %s" % str(self.coords.shape))
-            print("control array has shape %s" % str(self.control.shape))
-            raise ValueError,"Invalid control points for Bezier Spline"
+        control = Coords(control)
+        if control.shape != (nparts,3):
+            print("coords array has shape %s" % str(coords.shape))
+            print("control array has shape %s" % str(control.shape))
+            raise ValueError,"Invalid control points for QuadBezierSpline"
+
+        # To enable easy transforms, we join the coords and controls
+        # in a single array
+        if not closed:
+            dummy = Coords([[0.,0.,0.]])
+            control = Coords.concatenate([control,dummy],axis=0)
+        coords = coords[:,newaxis,:]
+        control = control[:,newaxis,:]
+        self.coords = Coords.concatenate([coords,control],axis=1)
+        self.nparts = nparts
         self.closed = closed
 
 
+    def pointsOn(self):
+        """Return the points on the curve""" 
+        return self.coords[:,0,:]
+
+
+    def pointsOff(self):
+        """Return the points off the curve (the control points)""" 
+        return self.coords[:self.nparts,1,:]
+
+
     def sub_points(self,t,j):
         j1 = (j+1) % self.coords.shape[0]
-        P = self.coords[[j,j1]]
-        D = self.control[j]
+        P = self.pointsOn()[[j,j1]]
+        D = self.pointsOff()[j]
         P = concatenate([ P[0],D,P[1] ],axis=0).reshape(-1,3)
         C = self.coeffs * P
         U = column_stack([t**2., t, ones_like(t)])
         X = dot(U,C)
         return X
 
+
 ##############################################################################
 #
-
 class CardinalSpline(BezierSpline):
     """A class representing a cardinal spline.
 
@@ -464,7 +707,14 @@ class CardinalSpline(BezierSpline):
         BezierSpline.__init__(self,coords,curl=(1.-tension)/3.,closed=closed)
 
 
-class CardinalSpline2(BezierSpline):
+
+##############################################################################
+#
+#  Curves that can NOT be transformed by Coords Transforms
+#
+##############################################################################
+
+class CardinalSpline2(Curve):
     """A class representing a cardinal spline."""
 
     def __init__(self,coords,tension=0.0,closed=False):
@@ -473,8 +723,6 @@ class CardinalSpline2(BezierSpline):
         This is a direct implementation of the Cardinal Spline.
         For open curves, it misses the interpolation in the two end
         intervals of the point set.
-        It is retained here because the implementation may some day
-        replace the BezierSpline implementation.
         """
         coords = Coords(coords)
         self.coords = coords
@@ -588,8 +836,6 @@ class NaturalSpline(Curve):
 
 ##############################################################################
 
-from plugins.geometry import triangleCircumCircle
-
 def vectorPairAngle(v1,v2):
     """Return the angle between the vectors v1 and v2."""
     v1 = asarray(v1)
@@ -666,4 +912,29 @@ class Spiral(Curve):
         self.nparts = nparts
         self.closed = False
 
+
+##############################################################################
+# Other functions
+
+def convertFormexToCurve(self,closed=False):
+    """Convert a Formex to a Curve.
+
+    The following Formices can be converted to a Curve:
+    - plex 2 : to PolyLine
+    - plex 3 : to QuadBezierSpline
+    - plex 4 : to BezierSpline
+    """
+    points = Coords.concatenate([self.coords[:,0,:],self.coords[-1:,-1,:]],axis=0)
+    if self.nplex() == 2:
+        curve = PolyLine(points,closed=closed)
+    elif self.nplex() == 3:
+        control = self.coords[:,1,:]
+        curve = QuadBezierSpline(points,control=control,closed=closed)
+    elif self.nplex() == 4:
+        control = self.coords[:,1:3,:]
+        curve = BezierSpline(points,control=control,closed=closed)
+    return curve
+
+Formex.toCurve = convertFormexToCurve
+
 # End
