@@ -1,6 +1,6 @@
-# $Id: fe.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: fe.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,25 +31,13 @@ from coords import *
 from connectivity import *
 from mydict import Dict
 from numpy import *
+from plugins.mesh import Mesh,mergeMeshes
+from utils import deprecation
+import warnings
 
 
-def mergeNodes(nodes):
-    """Merge all the nodes of a list of node sets.
-
-    Each item in nodes is a Coords array.
-    The return value is a tuple with:
-    
-    - the coordinates of all unique nodes,
-    - a list of indices translating the old node numbers to the new.
-    """
-    coords = Coords(concatenate([x for x in nodes],axis=0))
-    coords,index = coords.fuse()
-    n = array([0] + [ x.npoints() for x in nodes ]).cumsum()
-    ind = [ index[f:t] for f,t in zip(n[:-1],n[1:]) ]
-    return coords,ind
-
-
-def mergeModels(femodels):
+@deprecation("\nUse Mesh instances and mesh.mergeMeshes instead.")
+def mergeModels(femodels,**kargs):
     """Merge all the nodes of a list of FE models.
 
     Each item in femodels is a (coords,elems) tuple.
@@ -58,34 +46,11 @@ def mergeModels(femodels):
     - the coordinates of all unique nodes,
     - a list of elems corresponding to the input list,
       but with numbers referring to the new coordinates.
-    """
-    coords = [ x for x,e in femodels ]
-    elems = [ e for x,e in femodels ]
-    coords,index = mergeNodes(coords)
-    return coords,[i[e] for i,e in zip(index,elems)]
-              
 
-def checkUniqueNumbers(nrs,nmin=0,nmax=None,error=None):
-    """Check that an array contains a set of unique integers in range.
-
-    nrs is an integer array with any shape.
-    All integers should be unique and in the range(nmin,nmax).
-    Beware: this means that    nmin <= i < nmax  !
-    Default nmax is unlimited. Set nmin to None to
-    error is the value to return if the tests are not passed.
-    By default, a ValueError is raised.
-    On success, None is returned
+    The merging operation can be tuned by specifying extra arguments
+    that will be passed to :meth:`Coords:fuse`.
     """
-    nrs = asarray(nrs)
-    new = unique1d(nrs)
-    if new.size != nrs.size or \
-           (nmin is not None and new.min() < nmin) or \
-           (nmax is not None and new.max() > nmax):
-        if error is None:
-            raise ValueError,"Values not unique or not in range"
-        else:
-            return error
-    return None
+    return mergeMeshes([Mesh(m) for m in femodels])
 
 
 ######################## Finite Element Model ##########################
@@ -230,13 +195,13 @@ class Model(Dict):
         return oldnew,newold
 
         
-def mergedModel(*args):
+def mergedModel(meshes,**kargs):
     """Returns the fe Model obtained from merging individual models.
 
     The input arguments are (coords,elems) tuples.
     The return value is a merged fe Model.
     """
-    return Model(*mergeModels(args))
+    return Model(*mergeMeshes(meshes,**kargs))
 
 
 if __name__ == "__main__":
