@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: fe_abq.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: fe_abq.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -90,6 +90,12 @@ def esetName(p):
 ## and should be written to file by the caller.
 ###############################################
 
+
+def fmtCmd(cmd='*'):
+    """Format a command."""
+    return '*'+cmd+'\n'
+
+
 def fmtHeading(text=''):
     """Format the heading of the Abaqus input file."""
     out = """**  Abaqus input file created by %s (%s)
@@ -99,6 +105,14 @@ def fmtHeading(text=''):
 """ % (GD.Version,GD.Url,text)
     return out
 
+def fmtPart(name='Part-1'):
+    """Start a new Part."""
+    out = """**  Abaqus input file created by %s (%s)
+**
+*PART
+""" % (name)
+    return out
+
 
 materialswritten=[]
 def fmtMaterial(mat):
@@ -110,16 +124,41 @@ def fmtMaterial(mat):
     """
     if mat.name is None or mat.name in materialswritten:
         return ""
-
-    if mat.poisson_ratio is None and mat.shear_modulus is not None:
-        mat.poisson_ratio = 0.5 * mat.young_modulus / mat.shear_modulus - 1.0
-
-    out ="""*MATERIAL, NAME=%s
-*ELASTIC
-%s,%s
-""" % (mat.name, float(mat.young_modulus), float(mat.poisson_ratio))
+    
+    out ="*MATERIAL, NAME=%s\n" % mat.name
     materialswritten.append(mat.name)
+    
+    if not mat.has_key('elasticity') or mat.elasticity == 'linear':
+    
+        if mat.poisson_ratio is None and mat.shear_modulus is not None:
+            mat.poisson_ratio = 0.5 * mat.young_modulus / mat.shear_modulus - 1.0
 
+        out += """*ELASTIC
+%s,%s
+""" % (float(mat.young_modulus), float(mat.poisson_ratio))
+    
+    elif mat.elasticity == 'hyperelastic':
+        out += "*HYPERELASTIC, %s" % mat.type
+        if mat.type == 'ogden' or mat.type == 'polynomial' or mat.type == 'reduced polynomial':
+            
+            ord = 1
+            if mat.has_key('order'):
+                ord = mat.order
+            out += ", N=%s\n" % ord
+    
+    elif mat.elasticity == 'anisotropic hyperelastic':
+        out += "*ANISOTROPIC HYPERELASTIC, HOLZAPFEL\n"
+        #TO DO: add possibility to define local orientations!!!"
+            
+    elif mat.elasticity == 'user':
+        out += "*USER MATERIAL, CONSTANTS=%s\n" % len(mat.constants)
+            
+    if mat.has_key('constants'):
+        for p in range(len(mat.constants)):
+            out += "%s, " % mat.constants[p]
+            if p%8==7 or p==len(mat.constants)-1:
+                out = out[:-2]+"\n"
+    
     if mat.density is not None:
         out += "*DENSITY\n%s\n" % float(mat.density)
 
@@ -1249,12 +1288,14 @@ class AbqData(object):
         self.out = out
 
 
-    def write(self,jobname=None,group_by_eset=True,group_by_group=False,header=''):
+    def write(self,jobname=None,group_by_eset=True,group_by_group=False,header='',create_part=False):
         """Write an Abaqus input file.
 
-        jobname is the name of the inputfile, with or without '.inp' extension.
-        If None is specified, output is written to sys.stdout
-        An extra header text may be specified.
+        - `jobname` : the name of the inputfile, with or without '.inp'
+          extension. If None is specified, the output is written to sys.stdout
+          An extra header text may be specified.
+        - `create_part` : if True, the model will be created as an Abaqus Part,
+          followed by and assembly of that part.
         """
         global materialswritten
         materialswritten = []
@@ -1271,6 +1312,9 @@ class AbqData(object):
 Script: %s 
 %s
 """ % (jobname, datetime.now(), GD.scriptName, header)))
+
+        if create_part:
+            fil.write("*PART, name=Part-0\n")
         
         nnod = self.model.nnodes()
         GD.message("Writing %s nodes" % nnod)
@@ -1349,6 +1393,13 @@ Script: %s
         for p in self.prop.getProp('e',attr=['section','eltype']):
             writeSection(fil,p)
 
+        if create_part:
+            fil.write("*END PART\n")
+            fil.write("*ASSEMBLY, name=Assembly\n")
+            fil.write("*INSTANCE, name=Part-0-0, part=Part-0\n")
+            fil.write("*END INSTANCE\n")
+            fil.write("*END ASSEMBLY\n")
+
         GD.message("Writing global model properties")
             
         prop = self.prop.getProp('',attr=['amplitude'])
@@ -1389,6 +1440,33 @@ Script: %s
             fil.close()
         GD.message("Done")
 
+
+    
+##################################################
+## Some convenience functions
+##################################################
+
+def exportMesh(filename,mesh,eltype=None,header=''):
+    """Export a finite element mesh in Abaqus .inp format.
+
+    This is a convenience function to quickly export a mesh to Abaqus
+    without having to go through the whole setup of a complete
+    finite element model.
+    This just writes the nodes and elements specified in the mesh to
+    the file with the specified name. The resulting file  can then be
+    imported in Abaqus/CAE or manual be edited to create a full model.
+    If an eltype is specified, it will oerride the value stored in the mesh.
+    This should be used to set a correct Abaqus element type matchin the mesh.
+    """
+    fil = file(filename,'w')
+    fil.write(fmtHeading(header))
+    if eltype is None:
+        eltype = mesh.eltype
+    writeNodes(fil,mesh.coords)
+    writeElems(fil,mesh.elems,eltype,nofs=1)
+    fil.close()
+    GD.message("Abaqus file %s written." % filename)
+
     
 ##################################################
 ## Test
