@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: fe_menu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: fe_menu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -105,18 +105,19 @@ def create_menu():
         ("&Reload Menu",reload_menu),
         ("&Close Menu",close_menu),
         ]
-    return widgets.Menu(_menu,items=MenuData,parent=GD.GUI.menu,before='help')
+    return menu.Menu(_menu,items=MenuData,parent=GD.GUI.menu,before='help')
+
 
 def show_menu():
     """Show the menu."""
     if not GD.GUI.menu.item(_menu):
         create_menu()
 
+
 def close_menu():
     """Close the menu."""
-    m = GD.GUI.menu.item(_menu)
-    if m :
-        m.remove()
+    GD.GUI.menu.removeItem(_menu)
+
 
 def reload_menu():
     """Reload the menu."""
@@ -129,13 +130,6 @@ def reload_menu():
 
 if __name__ == "draw":
 
-    clear()
-    reset()
-    smooth()
-##     if GD.GUI:
-##         workHere()
-##         print(os.getcwd())
-        
     reload_menu()
 
 # End
