@@ -1,6 +1,6 @@
-# $Id: fe_post.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: fe_post.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -33,9 +33,8 @@ Then execute the created script from inside pyFormex.
 """
 
 import pyformex as GD
-from numpy import *
-from gui.draw import export
-import connectivity
+from arraytools import *
+from script import export
 from odict import ODict
 
 import re
@@ -125,7 +124,7 @@ class FeResult(object):
         self.eset[self.esetkey] = union1d(self.eset[self.esetkey],asarray(data))
 
     def Finalize(self):
-        self.nid = connectivity.reverseUniqueIndex(self.nodid)
+        self.nid = inverseUniqueIndex(self.nodid)
         for k in self.elems.iterkeys():
             v = asarray(self.elems[k])
             self.elems[k] = asarray(self.nid[v])
