@@ -1,6 +1,6 @@
-# $Id: isopar.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: isopar.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -25,16 +25,18 @@
 """Isoparametric transformations"""
 
 from formex import *
+from utils import deprecation
 
 
-def build_matrix(atoms,x,y=0,z=0):
+def evaluate(atoms,x,y=0,z=0):
     """Build a matrix of functions of coords.
 
-    Atoms is a list of text strings representing some function of
-    x(,y)(,z). x is a list of x-coordinats of the nodes, y and z can be set
-    to lists of y,z coordinates of the nodes.
-    Each line of the returned matrix contains the atoms evaluated at a
-    node.
+    - `atoms`: a list of text strings representing a mathematical function of
+      `x`, and possibly of `y` and `z`.
+    - `x`, `y`, `z`: a list of x- (and optionally y-, z-) values at which the
+      `atoms` will be evaluated. The lists should have the same length.
+
+    Returns a matrix with `nvalues` rows and `natoms` colums.
     """
     aa = zeros((len(x),len(atoms)),Float)
     for k,a in enumerate(atoms):
@@ -43,7 +45,19 @@ def build_matrix(atoms,x,y=0,z=0):
 
 
 class Isopar(object):
-    """A class representing an isoparametric transformation"""
+    """A class representing an isoparametric transformation
+
+    The following three formulations are equivalent ::
+
+       trf = Isopar(eltype,coords,oldcoords)
+       G = F.isopar(trf)
+
+       trf = Isopar(eltype,coords,oldcoords)
+       G = trf.transform(F)
+
+       G = isopar(F,eltype,coords,oldcoords)
+
+    """
 
     # REM: we should create a function to produce these data
     isodata = {
@@ -126,7 +140,7 @@ class Isopar(object):
             z = oldcoords[:,2]
         else:
             z = 0
-        aa = build_matrix(atoms,x,y,z)
+        aa = evaluate(atoms,x,y,z)
         ab = linalg.solve(aa,coords)
         self.eltype = eltype
         self.trf = ab
@@ -137,42 +151,44 @@ class Isopar(object):
 
         Returns a Coords array with same shape as X
         """
+        try:
+            X = Coords(X)
+        except:
+            raise ValueError,"Expected a Coords object as argument"
+        
         ndim,atoms = Isopar.isodata[self.eltype]
-        X = Coords(X)
-        aa = build_matrix(atoms,X.x().ravel(),X.y().ravel(),X.z().ravel())
+        aa = evaluate(atoms,X.x().ravel(),X.y().ravel(),X.z().ravel())
         xx = reshape(dot(aa,self.trf),X.shape)
         if ndim < 3:
             xx[...,ndim:] += X[...,ndim:]
         return xx
 
 
-    def transformFormex(self,F):
-        """Apply an isoparametric transform to a Formex.
+def isopar(X,eltype,coords,oldcoords):
+    """Perform an isoparametric transformation on a Coords.
 
-        The result is a topologically equivalent Formex.
-        """
-        return Formex(self.transform(F.f))
+    This is a convenience function that creates and uses an isoparametric
+    transformation in a single line. It is equivalent to::
+
+      Isopar(eltype,coords,oldcoords).transform(X)
+    """
+    return Isopar(eltype,coords,oldcoords).transform(X)
 
 
-def transformFormex(F,trf):
-    return trf.transformFormex(F)
-    
-Formex.isopar = transformFormex
+Coords.isopar = isopar
 
+def _isopar_F(self,eltype,coords,oldcoords):
+    X = isopar(self.coords,eltype,coords,oldcoords)
+    return Formex(X,self.prop,self.eltype)
 
-def isopar(F,eltype,coords,oldcoords):
-    return Isopar(eltype,coords,oldcoords).transformFormex(F)
+Formex.isopar = _isopar_F
 
+from plugins import mesh
+
+def _isopar_M(self,eltype,coords,oldcoords):
+    return mesh.Mesh(self.coords.isopar(eltype,coords,oldcoords),self.elems,self.prop,self.eltype)
+
+mesh.Mesh.isopar = _isopar_M
 
-# the following formulations are equivalent
-# 1.
-#   trf = Isopar(eltype,coorsd,oldcoords)
-#   G = F.isopar(trf)
-# 2.
-#   trf = Isopar(eltype,coorsd,oldcoords)
-#   G = trf.transformFormex(F)
-# 3.
-#   G = isopar(F,eltype,,coorsd,oldcoords)
-#
 
 # End
