@@ -1,7 +1,7 @@
 #!/usr/bin/env python pyformex.py
-# $Id: jobs_menu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: jobs_menu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -24,7 +24,8 @@
 ##
 
 import pyformex as GD
-from gui import widgets
+from gui import menu
+
 import utils
 import os
 
@@ -33,6 +34,8 @@ from formex import *
 from gui.draw import *
 from gui.colors import *
 
+from subprocess import call
+
 
 def about():
     showInfo("""Jobs.py
@@ -112,18 +115,32 @@ def submitToCluster(filename=None):
             ('ncpus',4,{'text':'Number of cpus','min':1,'max':1024}),
             ('postabq',False,{'text':'Run postabq on the results?'}),
             ])
-        reqtxt = 'cpus=%s\n' % res['ncpus']
-        if res['postabq']:
-            reqtxt += 'postproc=postabq\n'
+        if res:
+            reqtxt = 'cpus=%s\n' % res['ncpus']
+            if res['postabq']:
+                reqtxt += 'postproc=postabq\n'
+            host = GD.cfg.get('jobs/host','mecaflix')
+            reqdir = GD.cfg.get('jobs/requests','bumper/requests')
+            cmd = "scp %s %s:%s" % (filename,host,reqdir)
+            ret = call(['scp',filename,'%s:%s' % (host,reqdir)])
+            print ret
+            ret = call(['ssh',host,"echo '%s' > %s/%s.request" % (reqtxt,reqdir,jobname)])
+            print ret
+        
+
+def killClusterJob(jobname=None):
+    """Kill a job to the cluster."""
+    res = askItems([('jobname','')])
+    if res:
+        jobname = res['jobname']
         host = GD.cfg.get('jobs/host','mecaflix')
         reqdir = GD.cfg.get('jobs/requests','bumper/requests')
-        cmd = "scp %s %s:%s" % (filename,host,reqdir)
-        from subprocess import call
-        ret = call(['scp',filename,'%s:%s' % (host,reqdir)])
+        cmd = "touch %s/%s.kill" % (reqdir,jobname)
+        print host
+        print cmd
+        ret = call(['ssh',host,"%s" % cmd])
         print ret
-        ret = call(['ssh',host,"echo '%s' > %s/%s.request" % (reqtxt,reqdir,jobname)])
-        print ret
-
+       
 
 the_server = None
 the_userdir = None
@@ -204,37 +221,46 @@ def getResultsFromServer(jobname=None,targetdir=None,ext=['.fil']):
 ####################################################################
 ######### MENU #############
 
+_menu = 'Jobs'
+
 def create_menu():
     """Create the Jobs menu."""
     MenuData = [
         ("&About",about),
         ("&Submit Abaqus Job",submitToCluster),
+        ("&Kill Cluster Job",killClusterJob),
         ("&Check result cases on server",checkResultsOnServer),
         ("&Get results from server",getResultsFromServer),
         ("&Execute remote command",remoteCommand),
         ("---",None),
+        ("&Reload Menu",reload_menu),
         ("&Close Menu",close_menu),
         ]
-    return widgets.Menu('Jobs',items=MenuData,parent=GD.GUI.menu,before='help')
+    return menu.Menu('Jobs',items=MenuData,parent=GD.GUI.menu,before='help')
+
 
- 
 def show_menu():
     """Show the menu."""
-    if not GD.GUI.menu.item('Jobs'):
+    if not GD.GUI.menu.item(_menu):
         create_menu()
 
+
 def close_menu():
     """Close the menu."""
-    m = GD.GUI.menu.item('Jobs')
-    if m :
-        m.remove()
+    GD.GUI.menu.removeItem(_menu)
+
 
 def reload_menu():
     """Reload the menu."""
     close_menu()
     show_menu()
 
+
+####################################################################
+######### What to do when the script is executed ###################
+
 if __name__ == "draw":
+
     reload_menu()
 
 # End
