@@ -1,7 +1,7 @@
 #!/usr/bin/env python pyformex
-# $Id: objects.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: objects.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -31,24 +31,24 @@ This is a support module for other pyFormex plugins.
 import pyformex as GD
 
 from coords import bbox
-from formex import Formex
-from plugins.surface import TriSurface
+from script import named
+import geomfile
+import odict
+
 from copy import deepcopy
 
 
 class Objects(object):
-    """A selection of objects from the globals().
+    """A selection of objects from the pyFormex Globals().
 
     The class provides facilities to filter the global objects by their type
     and select one or more objects by their name(s). The values of these
     objects can be changed and the changes can be undone.
     """
 
-    def __init__(self,clas=None,filter=None,namelist=[]):
+    def __init__(self,clas=None,like=None,filter=None,namelist=[]): 
         """Create a new selection of objects.
 
-        If a dict is given, objects will be selected from this dict, else
-        from the global pyFormex dict GD.PF.
         If a filter is given, only objects passing it will be accepted.
         The filter will be applied dynamically on the dict.
 
@@ -56,6 +56,7 @@ class Objects(object):
         those names (provided they are in the dictionary).
         """
         self.clas = clas
+        self.like = like
         self.filter = filter
         self.names = []
         self.values = []
@@ -108,8 +109,17 @@ class Objects(object):
         return self.names[i]
     
 
+    def listAll(self):
+        """Return a list with all selectable objects.
+
+        This lists all the global names in pyformex.PF that match
+        the class and/or filter (if specified).
+        """
+        return listAll(clas=self.clas,like=self.like,filter=self.filter)
+
+
     def selectAll(self):
-        self.set(self.listAll(clas=self.clas))
+        self.set(self.listAll())
 
 
     def remember(self,copy=False):
@@ -163,6 +173,15 @@ class Objects(object):
             return map(named,self.names)
 
 
+    def odict(self):
+        """Return the currently selected items as a dictionary.
+
+        Returns an ODict with the currently selected objects in the order
+        of the selection.names.
+        """
+        return odict.ODict(zip(self.names,self.check(warn=False)))
+    
+
     def ask(self,mode='multi'):
         """Show the names of known objects and let the user select one or more.
 
@@ -170,7 +189,7 @@ class Objects(object):
         This sets the current selection to the selected names.
         Return a list with the selected names or None.
         """
-        res = widgets.Selection(listAll(clas=self.clas),
+        res = widgets.Selection(self.listAll(),
                                 'Known %sobjects' % self.object_type(),
                                 mode,sort=True,selected=self.names
                                 ).getResult()
@@ -213,9 +232,24 @@ class Objects(object):
                 GD.message("Overal bbox is %s" % bbox(objects))
 
 
+    def writeToFile(self,filename):
+        """Write objects to a geometry file."""
+        objects = self.odict()
+        if objects:
+            writeGeomFile(filename,objects) 
+
+
+    def readFromFile(self,filename):
+        """Read objects from a geometry file."""
+        res = readGeomFile(filename)
+        export(res)
+        self.set(res.keys())
+
 
 ###################### Drawable Objects #############################
 
+from gui.draw import *
+
 # Default Annotations
             
 def draw_object_name(n):
@@ -224,23 +258,29 @@ def draw_object_name(n):
 
 def draw_elem_numbers(n):
     """Draw the numbers of an object's elements."""
-    return drawNumbers(named(n))
+    return drawNumbers(named(n),color='blue')
+
+def draw_nodes(n):
+    """Draw the numbers of an object's nodes."""
+    return draw(named(n).coords,flat=True)
+
+def draw_node_numbers(n):
+    """Draw the numbers of an object's nodes."""
+    return drawNumbers(named(n).coords,color='red')
 
 def draw_bbox(n):
     """Draw the bbox of an object."""
     return drawBbox(named(n))
    
 
-from gui.draw import *
-
 class DrawableObjects(Objects):
     """A selection of drawable objects from the globals().
 
-    ``annotations``, if set, is a list of (func,active) tuples, where
+    `annotations`, if set, is a list of (func,active) tuples, where
 
-    - ``func`` is a function that is to be called with the object name as
+    - `func` is a function that is to be called with the object name as
       argument to draw some annotation for the object,
-    - ``active`` is a flag to signal if the annotation should be drawn or not.
+    - `active` is a flag to signal if the annotation should be drawn or not.
 
     The default is to draw object name and element numbers.
     """
@@ -250,6 +290,8 @@ class DrawableObjects(Objects):
         self.shrink = None
         self.annotations = [[draw_object_name,False],
                             [draw_elem_numbers,False],
+                            [draw_node_numbers,False],
+                            [draw_nodes,False],
                             [draw_bbox,False],
                             ]
         self._annotations = {}
@@ -264,6 +306,7 @@ class DrawableObjects(Objects):
             if a[1]:
                 self.drawAnnotation(i)
 
+    
 
     def ask(self,mode='multi'):
         """Interactively sets the current selection."""
@@ -278,7 +321,6 @@ class DrawableObjects(Objects):
         old and new can be a either Formex instances or names or lists thereof.
         old are drawn in yellow, new in the current color.
         """
-        #print("DRAWCHANGES %s" % type(self))
         self.draw(wait=False)
         draw(self.values,color='yellow',bbox=None,clear=False,shrink=self.shrink)
 
@@ -290,14 +332,12 @@ class DrawableObjects(Objects):
 
 
     def toggleAnnotation(self,i=0,onoff=None):
-        """Toggle the display of number On or Off.
+        """Toggle the display of an annotaion On or Off.
 
         If given, onoff is True or False. 
         If no onoff is given, this works as a toggle. 
         """
         active = self.annotations[i][1]
-        #print("WAS")
-        #print(self.annotations)
         if onoff is None:
             active = not active
         elif onoff:
@@ -305,35 +345,48 @@ class DrawableObjects(Objects):
         else:
             active = False
         self.annotations[i][1] = active
-        #print("BECOMES")
-        #print(self.annotations)
         if active:
             self.drawAnnotation(i)
         else:
             self.removeAnnotation(i)
-        #print(self._annotations)
 
 
     def drawAnnotation(self,i=0):
         """Draw some annotation for the current selection."""
-        #print("DRAW %s" % i)
         self._annotations[i] = [ self.annotations[i][0](n) for n in self.names ]
 
 
     def removeAnnotation(self,i=0):
         """Remove the annotation i."""
-        #print("REMOVE %s" % i)
         GD.canvas.removeAnnotations(self._annotations[i])
         GD.canvas.update()
         del self._annotations[i]
 
 
-    def toggleNames(self):
-        self.toggleAnnotation(0)
-    def toggleNumbers(self):
-        self.toggleAnnotation(1)
-    def toggleBbox(self):
-        self.toggleAnnotation(2)
+    def hasAnnotation(self,i=0):
+        """Return the status of annotation i"""
+        return self.annotations[i][1]
+    def hasNames(self):
+        return self.hasAnnotation(0)
+    def hasNumbers(self):
+        return self.hasAnnotation(1)
+    def hasNodeNumbers(self):
+        return self.hasAnnotation(2)
+    def hasNodeMarks(self):
+        return self.hasAnnotation(3)
+    def hasBbox(self):
+        return self.hasAnnotation(4)
+        
+    def toggleNames(self,onoff=None):
+        self.toggleAnnotation(0,onoff)
+    def toggleNumbers(self,onoff=None):
+        self.toggleAnnotation(1,onoff)
+    def toggleNodeNumbers(self,onoff=None):
+        self.toggleAnnotation(2,onoff)
+    def toggleNodes(self,onoff=None):
+        self.toggleAnnotation(3,onoff)
+    def toggleBbox(self,onoff=None):
+        self.toggleAnnotation(4,onoff)
 
 
     def setProperty(self,prop=None):
