@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: partition.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: partition.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -39,7 +39,7 @@ def prepare(V):
     P = V.center()
     print("Initial P = %s" % P)
     VA = draw(V,bbox=None,color='black')
-    area,norm = surface.areaNormals(V.f)
+    area,norm = surface.areaNormals(V.coords)
     N = norm[0]
     return V,P,N
 
@@ -60,7 +60,7 @@ def testview(F,V,P):
     print(F.center())
     undraw(VA)
     VA = draw(V)
-    area,norm = surface.areaNormals(V.f)
+    area,norm = surface.areaNormals(V.coords)
     N = norm[0]
     return P,N
 
@@ -75,7 +75,7 @@ def colorCut(F,P,N,prop):
     print(dist)
     right = any(dist>0.0,axis=1)
     print(right)
-    F.p[right] = prop
+    F.prop[right] = prop
     nright = right.sum()
     nleft = F.nelems() - nright
     print("Left part has %s elements, right part has %s elements" % (nleft,nright))
@@ -89,7 +89,7 @@ def splitProp(F,name):
     these named Formex instances.
     It the Formex has no props, the whole Formex is given the name.
     """
-    if F.p is None:
+    if F.prop is None:
         d = { name:F }
     else:
         d = dict([['%s-%s' % (name,p),F.withProp(p)] for p in F.propSet()])
@@ -132,7 +132,7 @@ def partition(Fin,prop=0):
     F = Fin.translate(initial_trl)
 
     # Store the inital properties and make all properties equal to start value
-    initial_prop = F.p
+    initial_prop = F.prop
     F.setProp(prop)
 
     # draw it
@@ -182,7 +182,7 @@ def partition(Fin,prop=0):
     QtCore.QObject.disconnect(GD.canvas,QtCore.SIGNAL("Wakeup"),wakeup)
     clear()
     draw(F)
-    Fin.setProp(F.p)
+    Fin.setProp(F.prop)
     return array(cut_planes)
 
    
