@@ -1,6 +1,6 @@
-# $Id: plot2d.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: plot2d.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -53,11 +53,11 @@ def showHistogram(x,y,txt,**options):
         #from PyQt4.Qwt5.qplt import *
 
 
-def createHistogram(data,cumulative=False):
+def createHistogram(data,cumulative=False,**kargs):
     """Create a histogram from data
 
     """
-    y,x = histogram(data)
+    y,x = histogram(data,**kargs)
     if cumulative:
         y = y.cumsum()
     return y,x
