@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex --gui
-# $Id: postproc_menu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: postproc_menu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -24,11 +24,12 @@
 ##
 
 import pyformex as GD
+from gui import menu
+
 from plugins.postproc import *
 from plugins.fe_post import FeResult
 from plugins.objects import Objects
 from gui.colorscale import ColorScale,ColorLegend
-from gui import decors,canvas,widgets
 import utils
 from odict import ODict
 
@@ -154,7 +155,8 @@ class Table(QtGui.QDialog):
 
 
 if globals().has_key('tbl'):
-    tbl.close()
+    if tbl is not None:
+        tbl.close()
 tbl = None
 
 def showfields():
@@ -211,12 +213,12 @@ def named_item(items,name):
 
 
 def showModel(nodes=True,elems=True):
+    print DB.elems
+    M = [ Mesh(DB.nodes,el,eltype='quad%d'%el.shape[1],prop=i) for i,el in enumerate(DB.elems.itervalues()) ]
     if nodes:
-        Fn = Formex(DB.nodes)
-        draw(Fn)
+        draw([m.coords for m in M])
     if elems:
-        Fe = [ Formex(DB.nodes[e],i+1) for i,e in enumerate(DB.elems.itervalues()) ]
-        draw(Fe)
+        draw(M)
     smoothwire()
     lights(False)
     transparent(True)
@@ -243,7 +245,7 @@ def showResults(nodes,elems,displ,text,val,showref=False,dscale=100.,
 
     # draw undeformed structure
     if showref:
-        ref = [ Formex(nodes[el]) for el in elems ]
+        ref = [ Mesh(nodes,el,eltype='quad%d'%el.shape[1]) for el in elems ]
         draw(ref,bbox=None,color='green',linewidth=1,mode='wireframe')
 
     # compute the colors according to the values
@@ -276,7 +278,7 @@ def showResults(nodes,elems,displ,text,val,showref=False,dscale=100.,
     if text:
         if multiplier != 0:
             text += ' (* 10**%s)' % -multiplier
-        drawtext(text,200,30)
+        drawText(text,200,30)
 
     smooth()
     lights(False)
@@ -285,24 +287,23 @@ def showResults(nodes,elems,displ,text,val,showref=False,dscale=100.,
     # create the frames while displaying them
     dscale = array(dscale)
     frames = []   # a place to store the drawn frames
-    bbox = []
+    bboxes = []
     for dsc in dscale.flat:
 
         #print(dsc)
         #print(nodes.shape)
         #print(displ.shape)
         dnodes = nodes + dsc * displ
-        deformed = [ Formex(dnodes[el]) for el in elems ]
-        bboxes = [ df.bbox() for df in deformed ]
-        bbox.append(Formex(bboxes).bbox())
+        deformed = [ Mesh(dnodes,el,eltype='quad%d'%el.shape[1]) for el in elems ]
+        bboxes.append(bbox(deformed))
         # We store the changing parts of the display, so that we can
         # easily remove/redisplay them
         #print(val)
         if val is None:
-            F = [ draw(df,color='blue',view='__last__',bbox=None,wait=None) for df in deformed ]
+            F = [ draw(df,color='blue',view=None,bbox='last',wait=None) for df in deformed ]
         else:
-            F = [ draw(df,color=cval[el],view='__last__',bbox=None,wait=None) for df,el in zip(deformed,elems) ]
-        T = drawtext('Deformation scale = %s' % dsc,200,10)
+            F = [ draw(df,color=cval[el],view=None,bbox='last',wait=None) for df,el in zip(deformed,elems) ]
+        T = drawText('Deformation scale = %s' % dsc,200,10)
 
         # remove the last frame
         # This is a clever trick: we remove the old drawings only after
@@ -318,7 +319,7 @@ def showResults(nodes,elems,displ,text,val,showref=False,dscale=100.,
             GD.app.processEvents()
             sleep(sleeptime)
 
-    zoomBbox(Formex(bbox).bbox())
+    zoomBbox(bbox(bboxes))
     
     # display the remaining cycles
     count -= 1
@@ -667,7 +668,7 @@ def selectDB(db=None):
     if db:
         setDB(db)
         clear()
-        print(DB.about['heading'])
+        print(DB.about.get('heading','No Heading'))
         print('Stress tensor has %s components' % DB.data_size['S'])
         showModel()
     return db
@@ -723,6 +724,7 @@ def open_results_dialog():
     if not checkDB():
         warning("No results database was selected!")
         return
+    close_dialog()
     data = GD.PF.get('__PostProcMenu_data__',dia_defaults)
     print("SAVED DATA",data)
     for k,v in data.items():
@@ -763,6 +765,10 @@ def askPoint():
         return None
 
 
+################################## Menu #############################
+
+_menu = 'Postproc'
+
 def create_menu():
     """Create the Postproc menu."""
     MenuData = [
@@ -779,19 +785,19 @@ def create_menu():
         ("&Reload menu",reload_menu),
         ("&Close menu",close_menu),
         ]
-    return widgets.Menu('Postproc',items=MenuData,parent=GD.GUI.menu,before='help')
-  
+    return menu.Menu(_menu,items=MenuData,parent=GD.GUI.menu,before='help')
+
+
 def show_menu():
-    """Show the Postproc menu."""
-    if not GD.GUI.menu.item('Postproc'):
+    """Show the menu."""
+    if not GD.GUI.menu.item(_menu):
         create_menu()
-        
+
+
 def close_menu():
-    """Close the Postproc menu."""
-    m = GD.GUI.menu.item('Postproc')
-    if m :
-        m.remove()
-        #GD.GUI.statusbar.removeWidget(GD.GUI.postbutton)
+    """Close the menu."""
+    GD.GUI.menu.removeItem(_menu)
+
 
 def reload_menu():
     """Reload the Postproc menu."""
@@ -799,14 +805,18 @@ def reload_menu():
     close_menu()
     DB =  GD.PF.get('__PostProcMenu_result__',None)
     print("Current database %s" % DB)
+    import plugins
+    plugins.refresh('postproc_menu')
     show_menu()
 
 
+####################################################################
+######### What to do when the script is executed ###################
+
 if __name__ == "draw":
+
     reload_menu()
-    
-elif __name__ == "__main__":
-    print(__doc__)
+
 
 # End
 
