@@ -1,7 +1,7 @@
 #!/usr/bin/env pyformex
-# $Id: properties.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: properties.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -322,14 +322,14 @@ def checkArrayOrIdValue(values):
     
     If ok, return the values as a list of (int,float) tuples.
     """
-    print("VALUES IN: %s" % values)
+    ##print("VALUES IN: %s" % values)
     try:
-        v = checkArray1D(values,6,'f','i')
+        v = checkArray(values,(6,),'f','i')
         w = where(v != 0.0)[0]
         values = [ (i,v[i]) for i in w ]
     except:
         values = checkIdValue(values)
-    print("VALUES OUT: %s" % values)
+    ##print("VALUES OUT: %s" % values)
     return values
 
 
@@ -366,7 +366,6 @@ def Eset(*args):
 ##################### Properties Database ###################
 
 
-
 def FindListItem(l,p):
     """Find the item p in the list l.
 
