@@ -1,6 +1,6 @@
 #!/usr/bin/env pyformex
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -49,13 +49,13 @@ class planeSection(object):
         Initialization can be done either by a list of points or a set of line
         segments.
 
-        By Points
+        By Points:
           Each point is connected to the following one, and (unless they are
           very close) the last one back to the first. Traversing the resulting
           path should rotate positively around the z axis to yield a positive
           surface.
 
-        By Segments
+        By Segments:
           It is the responsibilty of the user to ensure that the segments
           form a closed curve. If not, the calculated section data will be
           rather meaningless.
@@ -84,13 +84,14 @@ def sectionChar(F):
     z axis to yield a positive surface.
 
     The return value is a dict with the following characteristics:
-    'L'   : circumference,
-    'A'   : enclosed surface,
-    'Sx'  : first area moment around global x-axis
-    'Sy'  : first area moment around global y-axis
-    'Ixx' : second area moment around global x-axis
-    'Iyy' : second area moment around global y-axis
-    'Ixy' : product moment of area around global x,y-axes
+    
+    - `L`   : circumference,
+    - `A`   : enclosed surface,
+    - `Sx`  : first area moment around global x-axis
+    - `Sy`  : first area moment around global y-axis
+    - `Ixx` : second area moment around global x-axis
+    - `Iyy` : second area moment around global y-axis
+    - `Ixy` : product moment of area around global x,y-axes
     """
     if F.nplex() != 2:
         raise ValueError, "Expected a plex-2 Formex!"
@@ -120,12 +121,12 @@ def extendedSectionChar(S):
     sectionChar().
     This function computes and returns a dict with the following:
 
-    - 'xG', 'yG' : coordinates of the center of gravity G of the plane section
-    - 'IGxx', 'IGyy', 'IGxy' : second area moments and product around axes
+    - `xG`, `yG` : coordinates of the center of gravity G of the plane section
+    - `IGxx`, `IGyy`, `IGxy` : second area moments and product around axes
       through G and  parallel with the global x,y-axes
-    - 'alpha' : angle(in radians) between the glabla x,y axes and the principal
+    - `alpha` : angle(in radians) between the global x,y axes and the principal
       axes (X,Y) of the section (X and Y always pass through G)
-    - 'IXX','IYY': principal second area moments around X,Y respectively. (The
+    - `IXX`, `IYY` : principal second area moments around X,Y respectively. (The
       second area product is always zero.)
     """
     xG =  S['Sy']/S['A']
@@ -151,8 +152,8 @@ def princTensor2D(Ixx,Iyy,Ixy):
 
     Returns a tuple with three values:
     
-    - alpha: angle (in radians) from x-axis to principal X-axis
-    - IXX,IYY: principal values of the tensor
+    - `alpha` : angle (in radians) from x-axis to principal X-axis
+    - `IXX,IYY` : principal values of the tensor
     """
     from math import sqrt,atan2
     C = (Ixx+Iyy) * 0.5
