@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: sectionize.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: sectionize.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -70,10 +70,10 @@ def centerline(F,dir,nx=2,mode=2,th=0.2):
         for P in grid:
             test = abs(F.distanceFromPlane(P,n)) < th
             if mode == 1:
-                C = F.f[test].mean(axis=0)
+                C = F.coords[test].mean(axis=0)
             elif mode == 2:
                 test = test.sum(axis=-1)
-                G = F.select(test==F.f.shape[1])
+                G = F.select(test==F.coords.shape[1])
                 print(G)
                 C = G.center()
             center.append(C)
