@@ -1,6 +1,6 @@
-# $Id: surface_menu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: surface_menu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,16 +28,19 @@ Surface operations plugin menu for pyFormex.
 """
 
 import pyformex as GD
-from gui import actors,colors,decors,widgets
+from gui import actors,colors,decors,widgets,menu
 from gui.colorscale import ColorScale,ColorLegend
 from gui.draw import *
 from plugins.surface import *
 from plugins.objects import *
-from plugins import plot2d,formex_menu,surface_abq
+from plugins import plot2d,formex_menu,mesh_menu,fe_abq
 import simple
 from plugins.tools import Plane
 from pyformex.arraytools import niceLogSize
 
+from gui.widgets import simpleInputItem as I
+from gui.widgets import groupInputItem as G
+
 import os, timer
 
 ##################### selection and annotations ##########################
@@ -46,14 +49,14 @@ import os, timer
 def draw_edge_numbers(n):
     """Draw the edge numbers of the named surface."""
     S = named(n)
-    F = Formex(S.coords[S.edges]) 
-    return drawNumbers(F,color=colors.red)
+    F = Formex(S.coords[S.getEdges()]) 
+    return drawNumbers(F,color='green')
 
 def draw_node_numbers(n):
     """Draw the node numbers of the named surface."""
     S = named(n)
     F = Formex(S.coords) 
-    return drawNumbers(F,color=colors.blue)
+    return drawNumbers(F,color='red')
 
 def draw_normals(n):
     S = named(n)
@@ -76,6 +79,7 @@ def draw_avg_normals(n):
     return draw(F,color='orange')
     
 selection = DrawableObjects(clas=TriSurface)
+
 ntoggles = len(selection.annotations)
 def toggleEdgeNumbers():
     selection.toggleAnnotation(0+ntoggles)
@@ -228,6 +232,62 @@ def fromFormex(suffix=''):
     selection.set(surfaces.keys())
 
 
+def toMesh(suffix=''):
+    """Transform the selection to Meshes.
+
+    If a suffix is given, the Meshes are stored with names equal to the
+    surface names plus the suffix, else, the surface names will be used
+    (and the surfaces will thus be cleared from memory).
+    """
+    if not selection.check():
+        selection.ask()
+
+    if not selection.names:
+        return
+
+    newnames = selection.names
+    if suffix:
+        newnames = [ n + suffix for n in newnames ]
+
+    newvalues = [ named(n).toMesh() for n in newnames ]
+    export2(newnames,newvalues)
+
+    if not suffix:
+        selection.clear()
+    mesh_menu.selection.set(newnames)
+    clear()
+    mesh_menu.selection.draw()
+    
+
+def fromMesh(suffix=''):
+    """Transform the Mesh selection to TriSurfaces.
+
+    If a suffix is given, the TriSurfaces are stored with names equal to the
+    Mesh names plus the suffix, else, the Mesh names will be used
+    (and the Meshes will thus be cleared from memory).
+    """
+    if not mesh_menu.selection.check():
+        mesh_menu.selection.ask()
+
+    if not mesh_menu.selection.names:
+        return
+
+    names = mesh_menu.selection.names
+    meshes = [ named(n) for n in names ]
+    if suffix:
+        names = [ n + suffix for n in names ]
+
+    t = timer.Timer()
+    surfaces =  dict([ (n,TriSurface(M)) for n,M in zip(names,meshes) if M.eltype == 'tri3'])
+    print("Converted in %s seconds" % t.seconds())
+    print(surfaces.keys())
+    export(surfaces)
+
+    if not suffix:
+        mesh_menu.selection.clear()
+    selection.set(surfaces.keys())
+
+
 def toggle_shrink():
     """Toggle the shrink mode"""
     if selection.shrink is None:
@@ -314,7 +374,7 @@ def fillHoles():
     """Fill the holes in the selected surface."""
     S = selection.check(single=True)
     if S:
-        border_elems = S.edges[S.borderEdges()]
+        border_elems = S.getEdges()[S.borderEdges()]
         if border_elems.size != 0:
             # partition borders
             print(border_elems)
@@ -329,7 +389,7 @@ def fillHoles():
             GD.canvas.camera.rot = R
             toolbar.setPerspective(P)
             for i,elems in enumerate(border_elems):
-                draw(Formex(S.coords[elems]))
+                draw(Formex(S.coords[elems],i))
             zoomAll()
             # pick borders for which the hole must be filled
             info("PICK HOLES WHICH HAVE TO BE FILLED.")
@@ -355,43 +415,145 @@ def fillHoles():
 #  - function to calculate the values
 #  - domain to display: True to display on edges, False to display on elements
 
-SelectableStatsValues = {
-    'Facet Area': (TriSurface.facetArea,False),
-    'Aspect ratio': (TriSurface.aspectRatio,False),
-    'Smallest altitude': (TriSurface.smallestAltitude,False),
-    'Longest edge': (TriSurface.longestEdge,False),
-    'Shortest edge': (TriSurface.shortestEdge,False),
-    'Number of node adjacent elements': (TriSurface.nNodeAdjacent,False),
-    'Number of edge adjacent elements': (TriSurface.nEdgeAdjacent,False),
-    'Edge angle': (TriSurface.edgeAngles,True),
-    'Number of connected elements': (TriSurface.nEdgeConnected,True),
-    }
+SelectableStatsValues = odict.ODict([
+    ('Aspect ratio', (TriSurface.aspectRatio,False)),
+    ('Facet Area', (TriSurface.facetArea,False)),
+    ('Smallest altitude', (TriSurface.smallestAltitude,False)),
+    ('Longest edge', (TriSurface.longestEdge,False)),
+    ('Shortest edge', (TriSurface.shortestEdge,False)),
+    ('Number of node adjacent elements', (TriSurface.nNodeAdjacent,False)),
+    ('Number of edge adjacent elements', (TriSurface.nEdgeAdjacent,False)),
+    ('Edge angle', (TriSurface.edgeAngles,True)),
+    ('Number of connected elements', (TriSurface.nEdgeConnected,True)),
+    ('Curvature', (TriSurface.curvature,False)),
+    ])
+
+CurvatureValues = ['Gaussian curvature','Mean curvature','Shape index','Curvedness','First principal curvature','Second principal curvature']
+
 
 def showHistogram(key,val,cumulative):
     y,x = plot2d.createHistogram(val,cumulative=cumulative)
     return plot2d.showHistogram(x,y,key)
 
-    
-def showStatistics():
+
+_stat_dia = None
+
+def showStatistics(key=None,domain=True,dist=False,cumdist=False,clip=None,vmin=None,vmax=None,percentile=False):
+    """Show the values corresponding with key in the specified mode.
+
+    key is one of the keys of SelectableStatsValues
+    mode is one of ['On Domain','Histogram','Cumulative Histogram']
+    """
     S = selection.check(single=True)
     if S:
-        dispmodes = ['On Domain','Histogram','Cumulative Histogram']
-        keys = SelectableStatsValues.keys()
-        keys.sort()
-        res  = askItems([('Select Value',None,'select',keys),
-                         ('Display Mode',None,'select',dispmodes)
-                         ])
-        GD.app.processEvents()
-        if res:
-            key = res['Select Value']
-            func,onEdges = SelectableStatsValues[key]
-            val = func(S)
-            mode = res['Display Mode']
-            if mode == 'On Domain':
-                showSurfaceValue(S,key,val,onEdges)
-            else: 
-                showHistogram(key,val,cumulative= mode.startswith('Cumul'))
+        func,onEdges = SelectableStatsValues[key]
+        kargs = {}
+        if key == 'Curvature':
+            kargs['neighbours'] = _stat_dia.results['neighbours']
+        val = func(S,**kargs)
+        if key == 'Curvature':
+            ind = CurvatureValues.index(_stat_dia.results['curval'])
+            val = val[ind]
+            val = val[S.elems]
 
+        # !! THIS SHOULD BE IMPLEMENTED AS A GENERAL VALUE CLIPPER
+        # !! e.g popping up when clicking the legend
+        # !! and the values should be changeable 
+
+        if clip:
+            clip = clip.lower()
+            if percentile:
+                try:
+                    from scipy.stats.stats import scoreatpercentile
+                except:
+                    warning("""..
+                
+**The **percentile** clipping option is not available.
+Most likely because 'python-scipy' is not installed on your system.""")
+                    return
+
+                Q1 = scoreatpercentile(val,vmin)
+                Q3 = scoreatpercentile(val,vmax)
+                factor = 3
+                if vmin:
+                    vmin = Q1-factor*(Q3-Q1)
+                if vmax:
+                    vmax = Q3+factor*(Q3-Q1)
+                
+            if clip == 'top':
+                val = val.clip(max=vmax)
+            elif clip == 'bottom':
+                val = val.clip(min=vmin)
+            else:
+                val = val.clip(vmin,vmax)
+
+        if domain:
+            showSurfaceValue(S,key,val,onEdges)
+        if dist:
+            showHistogram(key,val,cumulative=False)
+        if cumdist:
+            showHistogram(key,val,cumulative=True)
+
+
+def _show_stats(domain,dist):
+    _stat_dia.acceptData()
+    res = _stat_dia.results
+    key = res['Value']
+    if dist and res['Cumulative Distribution']:
+        cumdist = True
+        dist = Fals/e
+    else:
+        cumdist = False
+    clip = res['clip']
+    if clip == 'None':
+        clip = None
+    percentile = res['Clip Mode'] != 'Range'
+    minval = res['Bottom']
+    maxval = res['Top']
+    showStatistics(key,domain,dist,cumdist,clip=clip,vmin=minval,vmax=maxval,percentile=percentile)
+
+def _show_domain():
+    _show_stats(True,False)
+def _show_dist():
+    _show_stats(False,True)
+
+def _close_stats_dia():
+    global _stat_dia
+    if _stat_dia:
+        _stat_dia.close()
+        _stat_dia = None
+
+    
+def showStatisticsDialog():
+    global _stat_dia
+    if _stat_dia:
+        _close_stats_dia()
+        
+    dispmodes = ['On Domain','Histogram','Cumulative Histogram']
+    keys = SelectableStatsValues.keys()
+    _stat_dia = widgets.NewInputDialog(
+        caption='Surface Statistics',items=[
+            I('Value',itemtype='vradio',choices=keys),
+            I('neighbours',text='Curvature Neighbourhood',value=1),
+            I('curval',text='Curvature Value',itemtype='vradio',choices=CurvatureValues),
+            I('clip',itemtype='hradio',choices=['None','Top','Bottom','Both']),
+            I('Clip Mode',itemtype='hradio',choices=['Range','Percentile']),
+            G('Clip Values',checkable=True,items=[
+                I('Top',1.0),
+                I('Bottom',0.0),
+                ],
+              ),
+            I('Cumulative Distribution',False),
+            ],
+        actions=[
+            ('Close',_close_stats_dia),
+            ('Distribution',_show_dist),
+            ('Show on domain',_show_domain)],
+        default='Show on domain'
+        )
+    _stat_dia.show()
+    
+            
 
 def showSurfaceValue(S,txt,val,onEdges):
     val = nan_to_num(val)
@@ -405,44 +567,22 @@ def showSurfaceValue(S,txt,val,onEdges):
     else:
         dec = 2
     # create a colorscale and draw the colorlegend
-    CS = ColorScale([colors.blue,colors.yellow,colors.red],mi,ma,0.5*(mi+ma),1.)
+    CS = ColorScale('RAINBOW',mi,ma,0.5*(mi+ma),1.)
     cval = array(map(CS.color,ravel(val)))
     cval = cval.reshape(append(val.shape,cval.shape[-1]))
     clear()
     if onEdges:
-        F = Formex(S.coords[S.edges])
+        F = Formex(S.coords[S.getEdges()])
         draw(F,color=cval)#,linewidth=2)
     else:
         draw(S,color=cval)
+    lights(False)
     CL = ColorLegend(CS,100)
     CLA = decors.ColorLegend(CL,10,10,30,200,dec=dec) 
     GD.canvas.addDecoration(CLA)
     drawtext(txt,10,230,'hv18')
 
 
-def showCurvature():
-    S = selection.check(single=True)
-    if S:
-        dispmodes = ['On Domain','Histogram','Cumulative Histogram']
-        res  = askItems([('Neighbourhood',1),
-                         ('Select Value',None,'select',['Gaussian curvature','Mean curvature','Shape index','Curvedness']),
-                         ('Display Mode',None,'select',dispmodes)
-                         ])        
-        if res:
-            key = res['Select Value']
-            curv = S.curvature(neighbours=res['Neighbourhood'])
-            val = {'Gaussian curvature':0,'Mean curvature':1,'Shape index':2,'Curvedness':3}
-            val = curv[val[key]]
-            mode = res['Display Mode']
-            if mode == 'On Domain':
-                val = val[S.elems]
-                showSurfaceValue(S,key,val,False)
-                smoothwire()
-                lights(False)
-            else: 
-                showHistogram(key,val,cumulative= mode.startswith('Cumul'))
-
-
 def colorByFront():
     S = selection.check(single=True)
     if S:
@@ -466,7 +606,7 @@ def colorByFront():
             else:
                 p = S.walkEdgeFront(nsteps=nsteps,startat=startat)
             S.setProp(p/nwidth + firstprop)
-            print("Colored in %s parts (%s seconds)" % (S.p.max()+1,t.seconds()))
+            print("Colored in %s parts (%s seconds)" % (S.prop.max()+1,t.seconds()))
             selection.draw()
 
 
@@ -475,8 +615,8 @@ def partitionByConnection():
     if S:
         selection.remember()
         t = timer.Timer()
-        S.p = S.partitionByConnection()
-        print("Partitioned in %s parts (%s seconds)" % (S.p.max()+1,t.seconds()))
+        S.prop = S.partitionByConnection()
+        print("Partitioned in %s parts (%s seconds)" % (S.prop.max()+1,t.seconds()))
         selection.draw()
 
 
@@ -488,8 +628,8 @@ def partitionByAngle():
         if res:
             selection.remember()
             t = timer.Timer()
-            S.p = S.partitionByAngle(**res)
-            print("Partitioned in %s parts (%s seconds)" % (S.p.max()+1,t.seconds()))
+            S.prop = S.partitionByAngle(**res)
+            print("Partitioned in %s parts (%s seconds)" % (S.prop.max()+1,t.seconds()))
             selection.draw()
          
  
@@ -623,7 +763,7 @@ def clip_surface():
     if not check_surface():
         return
     itemlist = [['axis',0],['begin',0.0],['end',1.0],['nodes','any']]
-    res = widgets.InputDialog(itemlist,'Clipping Parameters').getResult()
+    res = askItems(itemlist,caption='Clipping Parameters')
     if res:
         updateGUI()
         nodes,elems = PF['old_surface'] = PF['surface']
@@ -703,6 +843,41 @@ def cutAtPlane():
             selection.drawChanges()
 
 
+def clipAtPlane():
+    """Clip the selection with a plane."""
+    FL = selection.check()
+    if not FL:
+        return
+    
+    dsize = bbox(FL).dsize()
+    esize = 10 ** (niceLogSize(dsize)-5)
+
+    res = askItems([['Point',(0.0,0.0,0.0)],
+                    ['Normal',(1.0,0.0,0.0)],
+                    ['Keep side','positive', 'radio', ['positive','negative']],
+                    ['Nodes','all','radio',['all','any','none']],
+                    ['Tolerance',esize],
+                    ['Property',1],
+                    ],caption = 'Define the clipping plane')
+    if res:
+        P = res['Point']
+        N = res['Normal']
+        side = res['Keep side']
+        nodes = res['Nodes']
+        atol = res['Tolerance']
+        prop = res['Property']
+        selection.remember(True)
+        if side == 'positive':
+            func = TriSurface.clip
+        else:
+            func = TriSurface.cclip
+        FL = [ func(F,F.test(nodes=nodes,dir=N,min=P,atol=atol)) for F in FL]
+        FL = [ F.setProp(prop) for F in FL ]
+        export(dict([('%s/clip' % n,F) for n,F in zip(selection,FL)]))
+        selection.set(['%s/clip' % n for n in selection])
+        selection.draw()
+
+
 def cutSelectionByPlanes():
     """Cut the selection with one or more planes, which are already created."""
     S = selection.check(single=True)
@@ -756,40 +931,64 @@ def cutSelectionByPlanes():
 def intersectWithPlane():
     """Intersect the selection with a plane."""
     FL = selection.check()
-    res = askItems([['Name','__auto__'],
+    if not FL:
+        return
+    res = askItems([['Suffix','intersect'],
                     ['Point',(0.0,0.0,0.0)],
                     ['Normal',(1.0,0.0,0.0)],
                     ],caption = 'Define the cutting plane')
     if res:
-        name = res['Name']
+        suffix = res['Suffix']
         P = res['Point']
         N = res['Normal']
-        G = [F.toFormex().intersectionLinesWithPlane(P,N) for F in FL]
-        draw(G,color='red',linewidth=3)
-        export(dict([('%s/%s' % (n,name),g) for n,g in zip(selection,G)])) 
+        M = [ S.intersectionWithPlane(P,N,ignoreErrors=True) for S in FL ]
+        draw(M,color='red')
+        export(dict([('%s/%s' % (n,suffix), m) for (n,m) in zip(selection,M)]))
+            
 
-
-def sliceIt():
+def slicer():
     """Slice the surface to a sequence of cross sections."""
     S = selection.check(single=True)
-    res = askItems([['Direction',0],
-                    ['# slices',10],
+    if not S:
+        return
+    res = askItems([['Direction',[1.,0.,0.]],
+                    ['# slices',20],
                    ],caption = 'Define the slicing planes')
     if res:
         axis = res['Direction']
         nslices = res['# slices']
-        bb = S.bbox()
-        xmin,xmax = bb[:,axis]
-        dx =  (xmax-xmin) / nslices
-        x = arange(nslices+1) * dx
-        N = unitVector(axis)
-        #print(N)
-        P = [ bb[0]+N*s for s in x ]
-        G = [S.toFormex().intersectionLinesWithPlane(Pi,N) for Pi in P]
-        #[ G.setProp(i) for i,G in enumerate(G) ]
-        G = Formex.concatenate(G)
-        draw(G,color='red',linewidth=3)
-        export({'%s/slices%s' % (selection[0],axis):G}) 
+        GD.GUI.setBusy(True)
+        slices = S.slice(dir=axis,nplanes=nslices,ignoreErrors=True)
+        GD.GUI.setBusy(False)
+        print [ s.nelems() for s in slices ]
+        draw([ s for s in slices if s.nelems() > 0],color='red',bbox='last',view=None)
+        export({'%s/slices' % selection[0]:slices}) 
+
+
+def spliner():
+    """Slice the surface to a sequence of cross sections."""
+    import olist
+    from plugins.curve import BezierSpline
+    S = selection.check(single=True)
+    if not S:
+        return
+    res = askItems([['Direction',[1.,0.,0.]],
+                    ['# slices',20],
+                   ],caption = 'Define the slicing planes')
+    if res:
+        axis = res['Direction']
+        nslices = res['# slices']
+        GD.GUI.setBusy(True)
+        slices = S.slice(dir=axis,nplanes=nslices,ignoreErrors=True)
+        GD.GUI.setBusy(False)
+        print [ s.nelems() for s in slices ]
+        split = [ s.splitProp().values() for s in slices if s.nelems() > 0 ]
+        split = olist.flatten(split)
+        export({'%s/split' % selection[0]:split}) 
+        draw(split,color='blue',bbox='last',view=None)
+        splines = [ BezierSpline(s.coords[s.elems[:,0]],closed=True) for s in split ]
+        draw(splines,color='red',bbox='last',view=None)
+        export({'%s/splines' % selection[0]:splines}) 
 
 
 ##################  Smooth the selected surface #############################
@@ -799,7 +998,7 @@ def smoothLowPass():
     S = selection.check(single=True)
     if S:
         res = askItems([('lambda_value',0.5),
-                ('n_iterations',2)],'Low-pass filter')
+                ('n_iterations',2),('neighbourhood',1)],'Low-pass filter')
         if res:
             if not 0.0 <= res['lambda_value'] <= 1.0:
                 warning("Lambda should be between 0 and 1.")
@@ -808,7 +1007,7 @@ def smoothLowPass():
                 warning("An even number of iterations is required.")
                 return
             selection.remember(True)
-            S.smoothLowPass(res['n_iterations'],res['lambda_value'])
+            S.smoothLowPass(res['n_iterations'],res['lambda_value'],res['neighbourhood'])
             selection.drawChanges()
 
 
@@ -817,7 +1016,7 @@ def smoothLaplaceHC():
     S = selection.check(single=True)
     if S:
         res = askItems([('lambda_value',0.5),
-                ('n_iterations',2),('alpha',0.),('beta',0.2)],'Laplace filter and HC algorithm')
+                ('n_iterations',2),('alpha',0.),('beta',0.2),('neighbourhood',1)],'Laplace filter and HC algorithm')
         if res:
             if not 0.0 <= res['lambda_value'] <= 1.0:
                 warning("Lambda should be between 0 and 1.")
@@ -829,7 +1028,7 @@ def smoothLaplaceHC():
                 warning("Beta should be between 0 and 1.")
                 return            
             selection.remember(True)
-            S.smoothLaplaceHC(res['n_iterations'],res['lambda_value'],res['alpha'],res['beta'])
+            S.smoothLaplaceHC(res['n_iterations'],res['lambda_value'],res['alpha'],res['beta'],res['neighbourhood'])
             selection.drawChanges()
 
 
@@ -837,14 +1036,6 @@ def smoothLaplaceHC():
 ########### The following functions are in need of a make-over
 
 
-def create_tetgen_volume():
-    """Generate a volume tetraeder mesh inside an stl surface."""
-    types = [ 'STL/OFF Files (*.stl *.off)', 'All Files (*)' ]
-    fn = askFilename(GD.cfg['workdir'],types)
-    if os.path.exists(fn):
-        sta,out = utils.runCommand('tetgen -z %s' % fn)
-        GD.message(out)
-
 
 def flytru_stl():
     """Fly through the stl model."""
@@ -852,21 +1043,17 @@ def flytru_stl():
     Fc = Formex(array(ctr).reshape((-1,1,3)))
     path = connect([Fc,Fc],bias=[0,1])
     flyAlong(path)
+
+
+def create_volume():
+    """Generate a volume tetraeder mesh inside an stl surface."""
+    types = [ 'STL/OFF Files (*.stl *.off)', 'All Files (*)' ]
+    fn = askFilename(GD.cfg['workdir'],types)
+    if fn:
+        tetgen.runTetgen(fn)        
     
 
-def export_stl():
-    """Export an stl model stored in Formex F in Abaqus .inp format."""
-    global project,F
-    if ack("Creating nodes and elements.\nFor a large model, this could take quite some time!"):
-        GD.app.processEvents()
-        GD.message("Creating nodes and elements.")
-        nodes,elems = F.feModel()
-        nnodes = nodes.shape[0]
-        nelems = elems.shape[0]
-        GD.message("There are %d unique nodes and %d triangle elements in the model." % (nnodes,nelems))
-        stl_abq.abq_export(project+'.inp',nodes,elems,'S3',"Created by stl_examples.py")
-
-def export_surface():       
+def export_surface():
     S = selection.check(single=True)
     if S:
         types = [ "Abaqus INP files (*.inp)" ]
@@ -874,10 +1061,7 @@ def export_surface():
         if fn:
             print("Exporting surface model to %s" % fn)
             updateGUI()
-            S.refresh()
-            nodes,elems = S.coords,S.elems
-            surface_abq.abq_export(fn,nodes,elems,'S3',"Abaqus model generated by pyFormex from input file %s" % os.path.basename(fn))
-
+            fe_abq.exportMesh(fn,S,eltype='S3',header="Abaqus model generated by pyFormex from input file %s" % os.path.basename(fn))
 
 
 def export_volume():
@@ -888,8 +1072,8 @@ def export_volume():
     if fn:
         print("Exporting volume model to %s" % fn)
         updateGUI()
-        nodes,elems = PF['volume']
-        stl_abq.abq_export(fn,nodes,elems,'C3D%d' % elems.shape[1],"Abaqus model generated by tetgen from surface in STL file %s.stl" % PF['project'])
+        mesh = Mesh(PF['volume'])
+        fe_abq.exportMesh(fn,mesh,eltype='C3D%d' % elems.shape[1],header="Abaqus model generated by tetgen from surface in STL file %s.stl" % PF['project'])
 
 
 def show_nodes():
@@ -1159,6 +1343,8 @@ def boolean():
     
 ################### menu #################
 
+_menu = 'Surface'
+
 def create_menu():
     """Create the Surface menu."""
     MenuData = [
@@ -1196,8 +1382,7 @@ def create_menu():
           ("&AvgNormals",toggleAvgNormals,dict(checkable=True)),
           ('&Toggle Bbox',selection.toggleBbox,dict(checkable=True)),
           ]),
-        ("&Statistics",showStatistics),
-        ("&Show Curvature",showCurvature),
+        ("&Statistics",showStatisticsDialog),
         ("---",None),
         ("&Frontal Methods",
          [("&Color By Front",colorByFront),
@@ -1224,10 +1409,12 @@ def create_menu():
           ]),
         ("&Clip/Cut",
          [("&Clip",clipSelection),
+          ("&Clip At Plane",clipAtPlane),
           ("&Cut At Plane",cutAtPlane),
           ("&Multiple Cut",cutSelectionByPlanes),
-          ("&Slicer",sliceIt),
           ("&Intersection With Plane",intersectWithPlane),
+          ("&Slicer",slicer),
+          ("&Spliner",spliner),
            ]),
         ("&Smoothing",
          [("&Low-pass filter",smoothLowPass),
@@ -1248,32 +1435,39 @@ def create_menu():
         # ("&Convert STL file to OFF file",convert_stl_to_off),
         # ("&Sanitize STL file to OFF file",sanitize_stl_to_off),
 #        ("&Trim border",trim_surface),
-        ("&Create volume mesh",create_tetgen_volume),
+        ("&Create volume mesh",create_volume),
 #        ("&Read Tetgen Volume",read_tetgen_volume),
         ("&Export surface to Abaqus",export_surface),
-#        ("&Export volume to Abaqus",export_volume),
+        ("&Export volume to Abaqus",export_volume),
+        ("---",None),
+        ("&Reload Menu",reload_menu),
         ("&Close Menu",close_menu),
         ]
-    return widgets.Menu('Surface',items=MenuData,parent=GD.GUI.menu,before='Help')
+    return menu.Menu('Surface',items=MenuData,parent=GD.GUI.menu,before='Help')
+
 
-    
 def show_menu():
-    """Show the Tools menu."""
-    global _the_menu
-    if not GD.GUI.menu.item('Surface'):
+    """Show the menu."""
+    if not GD.GUI.menu.item(_menu):
         create_menu()
 
 
 def close_menu():
-    """Close the Tools menu."""
-    m = GD.GUI.menu.item('Surface')
-    if m :
-        m.remove()
-    
+    """Close the menu."""
+    GD.GUI.menu.removeItem(_menu)
 
-if __name__ == "draw":
-    # If executed as a pyformex script
+
+def reload_menu():
+    """Reload the menu."""
     close_menu()
     show_menu()
 
+
+####################################################################
+######### What to do when the script is executed ###################
+
+if __name__ == "draw":
+
+    reload_menu()
+
 # End
