@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: tetgen.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: tetgen.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -180,11 +180,14 @@ def nextFilename(fn):
 
 
 def runTetgen(fn):
-    """Run tetgen mesher on the specified file."""
+    """Run tetgen mesher on the specified file.
+
+    The input file is a closed triangulated surface.
+    tetgen will generate a volume tetraeder mesh inside the surface,
+    and create a new approximation of the surface as a by-product.
+    """
     if os.path.exists(fn):
         sta,out = utils.runCommand('tetgen -z %s' % fn)
-        #GD.message(out)
-
 
 
 if __name__ == "__main__":
