@@ -1,6 +1,6 @@
-# $Id: tools.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: tools.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -94,7 +94,7 @@ def reportActors(K):
     s += "Actors %s\n" % v
     for k in v:
         A = GD.canvas.actors[k]
-        t = A.atype
+        t = A.getType()
         s += "  Actor %s (type %s)\n" % (k,t)
     return s
 
@@ -104,7 +104,7 @@ def reportElements(K):
     for k in K.keys():
         v = K[k]
         A = GD.canvas.actors[k]
-        t = A.atype
+        t = A.getType()
         s += "Actor %s (type %s); Elements %s\n" % (k,t,v)
         if t == 'Formex':
             e = A
@@ -120,7 +120,7 @@ def reportPoints(K):
     for k in K.keys():
         v = K[k]
         A = GD.canvas.actors[k]
-        s += "Actor %s (type %s); Points %s\n" % (k,A.atype,v)
+        s += "Actor %s (type %s); Points %s\n" % (k,A.getType(),v)
         x = A.vertices()
         for p in v:
             s += "  Point %s: %s\n" % (p,x[p]) 
@@ -132,7 +132,7 @@ def reportEdges(K):
     for k in K.keys():
         v = K[k]
         A = GD.canvas.actors[k]
-        s += "Actor %s (type %s); Edges %s\n" % (k,A.atype,v)
+        s += "Actor %s (type %s); Edges %s\n" % (k,A.getType(),v)
         e = A.edges()
         for p in v:
             s += "  Edge %s: %s\n" % (p,e[p]) 
@@ -143,7 +143,7 @@ def reportPartitions(K):
     for k in K.keys():
         P = K[k][0]
         A = GD.canvas.actors[k]
-        t = A.atype
+        t = A.getType()
         for l in P.keys():
             v = P[l]
             s += "Actor %s (type %s); Partition %s; Elements %s\n" % (k,t,l,v)
@@ -190,10 +190,10 @@ def reportAngles(K):
 def getObjectItems(obj,items,mode):
     """Get the specified items from object."""
     if mode == 'actor':
-        return [ obj[i] for i in items ]
+        return [ obj[i].object for i in items if hasattr(obj[i],'object') ]
     elif mode in ['element','partition']:
-        if hasattr(obj,'select'):
-            return obj.select(items)
+        if hasattr(obj,'object') and hasattr(obj.object,'select'):
+            return obj.object.select(items)        
     elif mode == 'point':
         if hasattr(obj,'vertices'):
             return obj.vertices()[items]
@@ -203,7 +203,7 @@ def getObjectItems(obj,items,mode):
 def getCollection(K):
     """Returns a collection."""
     if K.obj_type == 'actor':
-        return [ GD.canvas.actors[int(i)] for i in K.get(-1,[]) ]
+        return [ GD.canvas.actors[int(i)].object for i in K.get(-1,[]) if hasattr(GD.canvas.actors[int(i)],'object') ]
     elif K.obj_type in ['element','point']:
         return [ getObjectItems(GD.canvas.actors[k],K[k],K.obj_type) for k in K.keys() ]
     elif K.obj_type == 'partition':
@@ -212,34 +212,6 @@ def getCollection(K):
         return None
 
    
-def setpropCollection(K,prop):
-    """Set the property of a collection.
-
-    prop should be a single non-negative integer value or None.
-    If None is given, the prop attribute will be removed from the objects
-    in collection even the non-selected items.
-    If a selected object does not have a setProp method, it is ignored.
-    """
-    if K.obj_type == 'actor':
-        obj = [ GD.canvas.actors[i] for i in K.get(-1,[]) ]
-        for o in obj:
-            if hasattr(o,'setProp'):
-                o.setProp(prop)
-            
-    elif K.obj_type in ['element','point']:
-        for k in K.keys():
-            o = GD.canvas.actors[k]
-            if prop is None:
-                o.setProp(prop)
-            elif hasattr(o,'setProp'):
-                if not hasattr(o,'p') or o.p is None:
-                    o.setProp(0)
-                o.p[K[k]] = prop
-                print(o.p)
-                o.setColor(o.p)
-                o.redraw()
-
-   
 def growCollection(K,**kargs):
     """Grow the collection with n frontal rings.
 
@@ -302,5 +274,6 @@ def exportObjects(obj,name,single=False):
         export(dict([ ("name-%s"%i,v) for i,v in enumerate(obj)]))
     else:
         export({name:obj})
+
                 
 # End
