@@ -1,6 +1,6 @@
-# $Id: tools_menu.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: tools_menu.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -28,20 +28,22 @@ Graphic Tools plugin menu for pyFormex.
 """
 
 import pyformex as GD
+from gui import menu
 
 import utils
-from gui import actors,colors
 from formex import *
 from gui.draw import *
+from gui.widgets import simpleInputItem as I, compatInputItem as C
 from plugins import objects
 from plugins.tools import *
+from plugins.surface import TriSurface
 
 
 def editFormex(F,name):
     """Edit a Formex"""
-    items = [('coords',repr(array(F.f)),'text')]
-    if F.p is not None:
-        items.append(('prop',repr(array(F.p)),'text'))
+    items = [('coords',repr(array(F.coords)),'text')]
+    if F.prop is not None:
+        items.append(('prop',repr(array(F.prop)),'text'))
     items.append(('eltype',F.eltype))
     res = askItems(items)
     if res:
@@ -54,7 +56,7 @@ def editFormex(F,name):
         print(x)
         print(p)
         print(e)
-        F.__init__(x,F.p,e)
+        F.__init__(x,F.prop,e)
 
 Formex.edit = editFormex
 
@@ -69,8 +71,8 @@ def command():
 
 ##################### database tools ##########################
 
-database = objects.Objects()
-drawable = objects.DrawableObjects()
+database = GD.GUI.database
+drawable = GD.GUI.drawable
 
 
 def printall():
@@ -90,7 +92,7 @@ def printbbox():
     database.printbbox()
     
 
-def forget():
+def forget_():
     """Forget global variables."""
     database.ask()
     database.forget()
@@ -139,7 +141,7 @@ def edit():
                 GD.PF.update(res)
 
 
-def rename():
+def rename_():
     """Rename a global variable."""
     database.ask(mode='single')
     F = database.check(single=True)
@@ -157,140 +159,46 @@ def deleteAll():
     printall()
     if ack("Are you sure you want to unrecoverably delete all global variables?"):
         GD.PF = {}
+
+
+def writeGeometry():
+    """Write geometry to file."""
+    drawable.ask()
+    if drawable.check():
+        filter = utils.fileDescription(['pgf','all'])
+        cur = GD.cfg['workdir']
+        fn = askNewFilename(cur=cur,filter=filter)
+        if fn:
+            drawable.writeToFile(fn)
+
+
+def readGeometry():
+    """Read geometry from file."""
+    filter = utils.fileDescription(['pgf','all'])
+    cur = GD.cfg['workdir']
+    fn = askFilename(cur=cur,filter=filter)
+    if fn:
+        drawable.readFromFile(fn)
+        drawable.draw()
+        zoomAll()
+        print drawable.names
+
+def dos2unix():
+    fn = askFilename(multi=True)
+    print("selected %s" % fn)
+    if fn:
+        for f in fn:
+            utils.dos2unix(f)
+
+
+def unix2dos():
+    fn = askFilename(multi=True)
+    if fn:
+        for f in fn:
+            utils.unix2dos(f)
+
         
-
-###################### Drawn Objects #############################
-
-            
-def draw_object_name(n):
-    """Draw the name of an object at its center."""
-    return drawText3D(named(n).center(),n)
-
-def draw_elem_numbers(n):
-    """Draw the numbers of an object's elements."""
-    return drawNumbers(named(n))
-
-def draw_bbox(n):
-    """Draw the bbox of an object."""
-    return drawBbox(named(n))
-   
-
-class DrawnObjects(dict):
-    """A dictionary of drawn objects.
-    """
-    def __init__(self,*args,**kargs):
-        dict.__init__(self,*args,**kargs)
-        self.autodraw = False
-        self.shrink = None
-        self.annotations = [[draw_object_name,False],
-                            [draw_elem_numbers,False],
-                            [draw_bbox,False],
-                            ]
-        self._annotations = {}
-        self._actors = []
-
-
-    def draw(self,*args,**kargs):
-        clear()
-        print("SELECTION: %s" % self.names)
-        self._actors = draw(self.names,clear=False,shrink=self.shrink,*args,**kargs)
-        #print(self.annotations)
-        for i,a in enumerate(self.annotations):
-            if a[1]:
-                self.drawAnnotation(i)
-
-
-    def ask(self,mode='multi'):
-        """Interactively sets the current selection."""
-        new = Objects.ask(self,mode)
-        if new is not None:
-            self.draw()
-
-
-    def drawChanges(self):
-        """Draws old and new version of a Formex with differrent colors.
-
-        old and new can be a either Formex instances or names or lists thereof.
-        old are drawn in yellow, new in the current color.
-        """
-        self.draw(wait=False)
-        draw(self.values,color='yellow',bbox=None,clear=False,shrink=self.shrink)
- 
-
-    def undoChanges(self):
-        """Undo the last changes of the values."""
-        Objects.undoChanges(self)
-        self.draw()
-
-
-    def toggleAnnotation(self,i=0,onoff=None):
-        """Toggle the display of number On or Off.
-
-        If given, onoff is True or False. 
-        If no onoff is given, this works as a toggle. 
-        """
-        active = self.annotations[i][1]
-        #print("WAS")
-        #print(self.annotations)
-        if onoff is None:
-            active = not active
-        elif onoff:
-            active = True
-        else:
-            active = False
-        self.annotations[i][1] = active
-        #print("BECOMES")
-        #print(self.annotations)
-        if active:
-            self.drawAnnotation(i)
-        else:
-            self.removeAnnotation(i)
-        #print(self._annotations)
-
-
-    def drawAnnotation(self,i=0):
-        """Draw some annotation for the current selection."""
-        #print("DRAW %s" % i)
-        self._annotations[i] = [ self.annotations[i][0](n) for n in self.names ]
-
-
-    def removeAnnotation(self,i=0):
-        """Remove the annotation i."""
-        #print("REMOVE %s" % i)
-        map(undraw,self._annotations[i])
-        del self._annotations[i]
-
-
-    def toggleNames(self):
-        self.toggleAnnotation(0)
-    def toggleNumbers(self):
-        self.toggleAnnotation(1)
-    def toggleBbox(self):
-        self.toggleAnnotation(2)
-
-
-    def setProperty(self,prop=None):
-        """Set the property of the current selection.
-
-        prop should be a single integer value or None.
-        If None is given, a value will be asked from the user.
-        If a negative value is given, the property is removed.
-        If a selected object does not have a setProp method, it is ignored.
-        """
-        objects = self.check()
-        if objects:
-            if prop is None:
-                res = askItems([['property',0]],
-                               caption = 'Set Property Number for Selection (negative value to remove)')
-                if res:
-                    prop = int(res['property'])
-                    if prop < 0:
-                        prop = None
-            for o in objects:
-                if hasattr(o,'setProp'):
-                    o.setProp(prop)
-            self.draw()
-
+    
 ##################### planes ##########################
 
 planes = objects.DrawableObjects(clas=Plane)
@@ -368,6 +276,7 @@ selection = None
 
 def set_selection(obj_type):
     global selection
+    selection = None
     selection = pick(obj_type)
 
 def query(mode):
@@ -400,7 +309,7 @@ def query_angle():
     showInfo("Select two line elements.")
     set_selection('element')
     print(reportAngles(selection))
-                   
+
 
 def report_selection():
     if selection is None:
@@ -410,6 +319,72 @@ def report_selection():
     print(report(selection))
 
 
+def edit_point(pt):
+    x,y,z = pt
+    dia = None
+    def close():
+        dia.close()
+    def accept():
+        dia.acceptData()
+        res = dia.results
+        return [ res[i] for i in 'xyz' ]
+        
+    dia = widgets.NewInputDialog(
+        items = [I(x=x,y=y,z=z),]
+        )
+    dia.show()
+    
+
+def edit_points(K):
+    if K.obj_type == 'point':
+        for k in K.keys():
+            o = GD.canvas.actors[k].object
+            n =  drawable.names[k]
+            ind = K[k]
+            print "CHANGING points %s of object %s" % (ind,n)
+            print o[ind]
+        
+   
+def setpropCollection(K,prop):
+    """Set the property of a collection.
+
+    prop should be a single non-negative integer value or None.
+    If None is given, the prop attribute will be removed from the objects
+    in collection even the non-selected items.
+    If a selected object does not have a setProp method, it is ignored.
+    """
+    if K.obj_type == 'actor':
+        obj = [ GD.canvas.actors[i] for i in K.get(-1,[]) ]
+        for o in obj:
+            if hasattr(o,'setProp'):
+                o.setProp(prop)
+            
+    elif K.obj_type in ['element','point']:
+        for k in K.keys():
+            o = GD.canvas.actors[k].object
+            print "SETPROP ACTOR %s" % type(o)
+            n =  drawable.names[k]
+            print "SETPROP DRAWABLE %s" % n
+            O = named(n)
+            if O is not None:
+                print "ALSO SETTING OBJECT PROPERTIES to %s" % prop
+            if prop is None:
+                o.setProp(prop)
+                if O:
+                    print id(o),id(O)
+                    O.setProp(prop)
+            elif hasattr(o,'setProp'):
+                if not hasattr(o,'prop') or o.prop is None:
+                    o.setProp(0)
+                print("PROP %s ?= %s" % (o.prop,O.prop))
+                o.prop[K[k]] = prop
+                o.setColor(o.prop)
+                o.redraw(mode=GD.canvas.rendermode)
+                if O and id(O.prop) != id(o.prop):
+                    O.setProp(o.prop)
+                print("PROP %s ?= %s" % (o.prop,O.prop))
+
+
 def setprop_selection():
     """Set the property of the current selection.
 
@@ -427,6 +402,7 @@ def setprop_selection():
         if prop < 0:
             prop = None
         setpropCollection(selection,prop)
+        removeHighlights()
 
 
 def grow_selection():
@@ -454,7 +430,7 @@ def partition_selection():
         warning("You need to pick actors or elements.")
         return
     for A in GD.canvas.actors:
-        if not A.atype == 'TriSurface':
+        if not A.getType() == TriSurface:
             warning("Currently I can only partition TriSurfaces." )
             return
     partitionCollection(selection)
@@ -498,20 +474,6 @@ def export_selection():
             export({name:sel})
         elif res['Export as'] == options[1]:
             export(dict([ (name+"-%s"%i,v) for i,v in enumerate(sel)]))
-
-
-def dos2unix():
-    fn = askFilename(multi=True)
-    print("selected %s" % fn)
-    if fn:
-        for f in fn:
-            utils.dos2unix(f)
-
-def unix2dos():
-    fn = askFilename(multi=True)
-    if fn:
-        for f in fn:
-            utils.unix2dos(f)
      
         
 ################### menu #################
@@ -521,7 +483,8 @@ _menu = 'Tools'
 def create_menu():
     """Create the Tools menu."""
     MenuData = [
-        ('&Execute pyFormex command',command),
+        ('&Read Geometry File',readGeometry),
+        ('&Write Geometry File',writeGeometry),
         ('-- Global Variables --',printall,dict(disabled=True)),
         ('  &List All',printall),
         ('  &Select',database.ask),
@@ -530,10 +493,14 @@ def create_menu():
         ('  &Draw',drawable.ask),
         ('  &Create',create),
         ('  &Change Value',edit),
-        ('  &Rename',rename),
-        ('  &Delete',forget),
+        ('  &Rename',rename_),
+        ('  &Delete',forget_),
         ('  &Delete All',deleteAll),
         ("---",None),
+        ('&Execute pyFormex command',command),
+        ("&DOS to Unix",dos2unix),
+        ("&Unix to DOS",unix2dos),
+        ("---",None),
         ("&Create Plane",
             [("Coordinates", 
                 [("Point and normal", createPlaneCoordsPointNormal),
@@ -553,6 +520,7 @@ def create_menu():
           ("&Points",pick_points),
           ("&Edges",pick_edges),
           ]),
+        ('&Edit Points',edit_points),
         ("&Remove Highlights",removeHighlights),
         ("---",None),
         ('&Selection',
@@ -573,13 +541,10 @@ def create_menu():
           ('&Angle',query_angle),
           ]),
         ("---",None),
-        ("&DOS to Unix",dos2unix),
-        ("&Unix to DOS",unix2dos),
-        ("---",None),
         ('&Reload',reload_menu),
         ("&Close",close_menu),
         ]
-    return widgets.Menu(_menu,items=MenuData,parent=GD.GUI.menu,before='help')
+    return menu.Menu(_menu,items=MenuData,parent=GD.GUI.menu,before='help')
 
 
 def show_menu():
@@ -587,23 +552,23 @@ def show_menu():
     if not GD.GUI.menu.item(_menu):
         create_menu()
 
+
 def close_menu():
     """Close the menu."""
-    m = GD.GUI.menu.item(_menu)
-    if m :
-        m.remove()
+    GD.GUI.menu.removeItem(_menu)
+
 
 def reload_menu():
     """Reload the menu."""
     close_menu()
-    reload(tools_menu)
     show_menu()
 
 
-def reload_menu():
-    close_menu()
-    reload(tools_menu)
-    show_menu()
-    
+####################################################################
+######### What to do when the script is executed ###################
+
+if __name__ == "draw":
+
+    reload_menu()
 
 # End
