@@ -1,6 +1,6 @@
-# $Id: project.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: project.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -65,7 +65,7 @@ class Project(dict):
 
     def header_data(self):
         """Construct the data to be saved in the header."""
-        store_attr = ['gzip','mode','autofile']
+        store_attr = ['gzip','mode','autofile','_autoscript_']
         store_vals = [getattr(self,k,None) for k in store_attr]
         return dict([(k,v) for k,v in zip(store_attr,store_vals) if v is not None])
 
