@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: script.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: script.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -30,30 +30,34 @@ applications, without the need to explicitely importing the :mod:`script`
 module.
 """
 
-
-import pyformex as GD
-import threading,os,commands,copy,re,time
-
+import pyformex
 import formex
+import geomfile
 import utils
-from utils import deprecation
 
+########################
+# Imported here only to make it available in scripts
+from plugins.mesh import Mesh
+
+########################
+
+import threading,os,commands,copy,re,time
 
 ######################### Exceptions #########################################
 
-class Exit(Exception):
+class _Exit(Exception):
     """Exception raised to exit from a running script."""
     pass    
 
-class ExitAll(Exception):
+class _ExitAll(Exception):
     """Exception raised to exit pyFormex from a script."""
     pass    
 
-class ExitSeq(Exception):
+class _ExitSeq(Exception):
     """Exception raised to exit from a sequence of scripts."""
     pass    
 
-class TimeOut(Exception):
+class _TimeOut(Exception):
     """Exception raised to timeout from a dialog widget."""
     pass    
 
@@ -73,9 +77,9 @@ def Globals():
     was executed in the 'draw' module (--gui option) or the 'script'
     module (--nogui option).
     """
-    g = copy.copy(GD.PF)
+    g = copy.copy(pyformex.PF)
     g.update(globals())
-    if GD.GUI:
+    if pyformex.GUI:
         from gui import colors,draw
         g.update(colors.__dict__)
         g.update(draw.__dict__)
@@ -85,7 +89,7 @@ def Globals():
 
 def export(dic):
     """Export the variables in the given dictionary."""
-    GD.PF.update(dic)
+    pyformex.PF.update(dic)
 
 
 def export2(names,values):
@@ -95,7 +99,7 @@ def export2(names,values):
 
 def forget(names):
     """Remove the global variables specified in list."""
-    g = GD.PF
+    g = pyformex.PF
     for name in names:
         if g.has_key(name):
             del g[name]
@@ -103,46 +107,54 @@ def forget(names):
 
 def rename(oldnames,newnames):
     """Rename the global variables in oldnames to newnames."""
-    g = GD.PF
+    g = pyformex.PF
     for oldname,newname in zip(oldnames,newnames):
         if g.has_key(oldname):
             g[newname] = g[oldname]
             del g[oldname]
 
 
-def listAll(clas=None,dic=None):
-    """Return a list of all objects in dic that are of given clas.
+def listAll(clas=None,like=None,filter=None,dic=None):
+    """Return a list of all objects in dictionay that match criteria.
 
-    If no class is given, Formex objects are sought.
-    If no dict is given, the objects from both GD.PF and locals()
-    are returned.
+    - dic: a dictionary object, defaults to pyformex.PF
+    - clas: a class name: if specified, only instances of this class will be
+      returned
+    - like: a string: if given, only object names starting with this string
+      will be returned
+    - filter: a function taking an object name as parameter and returning True
+      or False. If specified, onnly objects passing the test will be returned.
+
+    The return value is a list of keys from dic.
     """
     if dic is None:
-        dic = GD.PF
+        dic = pyformex.PF
 
-    if clas is None:
-        return dic.keys()
-    else:
-        return [ k for k in dic.keys() if isinstance(dic[k],clas) ]
+    names = dic.keys()
+    if clas is not None:
+        names = [ n for n in names if isinstance(dic[n],clas) ]
+    if like is not None:
+        names = [ n for n in names if n.startswith(like) ]
+    if filter is not None:
+        names = [ n for n in names if filter(n) ]
+    return names
 
 
 def named(name):
     """Returns the global object named name."""
-    #GD.debug("name %s" % name)
-    if GD.PF.has_key(name):
-        #GD.debug("Found %s in GD.PF" % name)
-        dic = GD.PF
-    elif globals().has_key(name):
-        GD.debug("Found %s in globals()" % name)
-        dic = globals()
+    if pyformex.PF.has_key(name):
+        dic = pyformex.PF
+    elif pyformex._PF_.has_key(name):
+        pyformex.debug("Found %s in pyformex._PF_" % name)
+        dic = pyformex._PF_
     else:
-        raise NameError,"Name %s is in neither GD.PF nor globals()" % name
+        raise NameError,"Name %s is in neither pyformex.PF nor pyformex._PF_" % name
     return dic[name]
 
 
 def getcfg(name):
     """Return a value from the configuration."""
-    return GD.cfg.get(name,None)
+    return pyformex.cfg.get(name,None)
 
 
 #################### Interacting with the user ###############################
@@ -200,7 +212,7 @@ def showInfo(message):
 # message is the preferred function to send text info to the user.
 # The default message handler is set here.
 # Best candidates are log/info
-message = GD.message
+message = pyformex.message
 
 def system(cmdline,result='output'):
     """Run a command and return its output.
@@ -235,7 +247,7 @@ def playScript(scr,name=None,filename=None,argv=[],pye=False):
     There is a lock to prevent multiple scripts from being executed at the
     same time. This implies that pyFormex scripts can currently not be
     recurrent.
-    If a name is specified, set the global variable GD.scriptName to it
+    If a name is specified, set the global variable pyformex.scriptName to it
     when the script is started.
     If a filename is specified, set the global variable __file__ to it.
     
@@ -244,29 +256,29 @@ def playScript(scr,name=None,filename=None,argv=[],pye=False):
     (including comments) is echoed to the message board.
     """
     global scriptDisabled,scriptRunning,exitrcrequested
-    #GD.debug('SCRIPT MODE %s,%s,%s'% (scriptDisabled,scriptRunning,exitrequested))
+    #pyformex.debug('SCRIPT MODE %s,%s,%s'% (scriptDisabled,scriptRunning,exitrequested))
     # (We only allow one script executing at a time!)
     # and scripts are non-reentrant
     if scriptRunning or scriptDisabled :
-        GD.message("Not executing this script because another one is already running")     
+        pyformex.message("Not executing this script because another one is already running")     
         return
     
     scriptRunning = True
     exitrequested = False
 
-    if GD.GUI:
+    if pyformex.GUI:
         global stepmode,exportNames,starttime
-        #GD.debug('GUI SCRIPT MODE %s'% (stepmode))
-        GD.GUI.drawlock.allow()
-        GD.canvas.update()
-        GD.GUI.actions['Play'].setEnabled(False)
-        GD.GUI.actions['Continue'].setEnabled(True)
-        GD.GUI.actions['Stop'].setEnabled(True)
-        GD.app.processEvents()
+        #pyformex.debug('GUI SCRIPT MODE %s'% (stepmode))
+        pyformex.GUI.drawlock.allow()
+        pyformex.canvas.update()
+        pyformex.GUI.actions['Play'].setEnabled(False)
+        pyformex.GUI.actions['Continue'].setEnabled(True)
+        pyformex.GUI.actions['Stop'].setEnabled(True)
+        pyformex.app.processEvents()
     
     # Get the globals
     g = Globals()
-    if GD.GUI:
+    if pyformex.GUI:
         modname = 'draw'
     else:
         modname = 'script'
@@ -275,22 +287,25 @@ def playScript(scr,name=None,filename=None,argv=[],pye=False):
         g.update({'__file__':filename})
     g.update({'argv':argv})
 
+    # Make this directory available
+    pyformex._PF_ = g
+
     # Now we can execute the script using these collected globals
     exportNames = []
-    GD.scriptName = name
+    pyformex.scriptName = name
     exitall = False
 
     starttime = time.clock()
-    GD.debug('STARTING SCRIPT (%s)' % starttime)
-    #GD.debug(scr)
-    #GD.debug(pye)
+    pyformex.debug('STARTING SCRIPT (%s)' % starttime)
+    #pyformex.debug(scr)
+    #pyformex.debug(pye)
     try:
         try:
-            if GD.GUI and stepmode:
-                #GD.debug("STEPPING THROUGH SCRIPT")
+            if pyformex.GUI and stepmode:
+                #pyformex.debug("STEPPING THROUGH SCRIPT")
                 step_script(scr,g,True)
             else:
-                if GD.options.executor:
+                if pyformex.options.executor:
                     import sys
                     print(name,filename)
                     n = os.path.split(name)
@@ -307,36 +322,45 @@ def playScript(scr,name=None,filename=None,argv=[],pye=False):
                     __import__(s,g)
                 else:
                     if pye:
-                        n = (len(scr)+1) // 2
-                        # print scr
+                        if type(scr) is file:
+                             scr = scr.read() + '\n'
+                        n = len(scr) // 2
                         scr = utils.mergeme(scr[:n],scr[n:])
-                        # print scr
                     exec scr in g
 
-            if GD.cfg['autoglobals']:
-                exportNames.extend(listAll(clas=formex.Formex,dic=g))
-            GD.PF.update([(k,g[k]) for k in exportNames])
-        except Exit:
+        except _Exit:
             pass
-        except ExitAll:
+        except _ExitAll:
             exitall = True
         except:
             raise
             
     finally:
+        # honour the exit function
+        if g.has_key('atExit'):
+            atExit = g['atExit']
+            try:
+                atExit()
+            except:
+                GD.debug('Error while calling script exit function')
+                
+        if pyformex.cfg['autoglobals']:
+            exportNames.extend(listAll(clas=formex.Formex,dic=g))
+        pyformex.PF.update([(k,g[k]) for k in exportNames])
+
         scriptRunning = False # release the lock in case of an error
         elapsed = time.clock() - starttime
-        GD.debug('SCRIPT RUNTIME : %s seconds' % elapsed)
-        if GD.GUI:
+        pyformex.debug('SCRIPT RUNTIME : %s seconds' % elapsed)
+        if pyformex.GUI:
             stepmode = False
-            GD.GUI.drawlock.release() # release the lock
-            GD.GUI.actions['Play'].setEnabled(True)
-            #GD.GUI.actions['Step'].setEnabled(False)
-            GD.GUI.actions['Continue'].setEnabled(False)
-            GD.GUI.actions['Stop'].setEnabled(False)
+            pyformex.GUI.drawlock.release() # release the lock
+            pyformex.GUI.actions['Play'].setEnabled(True)
+            #pyformex.GUI.actions['Step'].setEnabled(False)
+            pyformex.GUI.actions['Continue'].setEnabled(False)
+            pyformex.GUI.actions['Stop'].setEnabled(False)
 
     if exitall:
-        GD.debug("Calling exit() from playscript")
+        pyformex.debug("Calling exit() from playscript")
         exit()
 
 
@@ -356,7 +380,7 @@ def step_script(s,glob,paus=True):
             buf += line
         if paus and (line.strip().startswith('draw') or
                      line.find('draw(') >= 0 ):
-            GD.GUI.drawlock.block()
+            pyformex.GUI.drawlock.block()
             message(buf)
             exec(buf) in glob
             buf = ''
@@ -375,13 +399,13 @@ def breakpt(msg=None):
     global exitrequested
     if exitrequested:
         if msg is not None:
-            GD.message(msg)
+            pyformex.message(msg)
         exitrequested = False # reset for next time
-        raise Exit
+        raise _Exit
 
 
 def enableBreak(mode=True):
-    GD.GUI.actions['Stop'].setEnabled(mode)
+    pyformex.GUI.actions['Stop'].setEnabled(mode)
 
 
 def stopatbreakpt():
@@ -398,28 +422,32 @@ def playFile(fn,argv=[]):
     argv. This variable can be changed by the script and the resulting argv
     is returned to the caller.
     """
+    from timer import Timer
+    t = Timer()
     message("Running script (%s)" % fn)
-    GD.debug("  Executing with arguments: %s" % argv)
+    pyformex.debug("  Executing with arguments: %s" % argv)
     res = playScript(file(fn,'r'),fn,fn,argv,fn.endswith('.pye'))
-    GD.debug("  Arguments left after execution: %s" % argv)
-    message("Finished script %s" % fn)
+    pyformex.debug("  Arguments left after execution: %s" % argv)
+    message("Finished script %s in %s seconds" % (fn,t.seconds()))
     return res
 
 
 def play(fn=None,argv=[],step=False):
-    """Play a formex script from file fn or from the current file.
+    """Play the pyFormex script with filename `fn` or the current script file.
 
-    This function does nothing if no file is passed or no current
-    file was set.
+    This function does nothing if no filename is passed or no current
+    scriptfile was set.
+    If arguments are given, they are passed to the script. If `step` is True,
+    the script is executed in step mode.
     """
     global stepmode
     if not fn:
-        if GD.GUI.canPlay:
-            fn = GD.cfg['curfile']
+        if pyformex.GUI.canPlay:
+            fn = pyformex.cfg['curfile']
         else:
             return
     stepmode = step
-    GD.GUI.history.add(fn)
+    pyformex.GUI.history.add(fn)
     stepmode = step
     return playFile(fn,argv)
 
@@ -428,12 +456,12 @@ def exit(all=False):
     """Exit from the current script or from pyformex if no script running."""
     if scriptRunning:
         if all:
-            raise ExitAll # exit from pyformex
+            raise _ExitAll # exit from pyformex
         else:
-            raise Exit # exit from script only
-    if GD.app and GD.app_started: # exit from GUI
-        GD.debug("draw.exit called while no script running")
-        GD.app.quit() # close GUI and exit pyformex
+            raise _Exit # exit from script only
+    if pyformex.app and pyformex.app_started: # exit from GUI
+        pyformex.debug("draw.exit called while no script running")
+        pyformex.app.quit() # close GUI and exit pyformex
     else: # the gui didn't even start
         sys.exit(0) # exit from pyformex
         
@@ -445,32 +473,43 @@ def processArgs(args):
     with arguments for the scripts.
     Each running script should pop the required arguments from the list.
     """
+    res = 0
     while len(args) > 0:
         fn = args.pop(0)
         if fn.endswith('.pye'):
             pass
         elif not os.path.exists(fn) or not utils.isPyFormex(fn):
-            GD.message("Skipping %s: does not exist or is not a pyFormex script" % fn)
+            pyformex.message("Skipping %s: does not exist or is not a pyFormex script" % fn)
             continue
         res = playFile(fn,args)
-        if res and GD.GUI:
-            GD.message("Error during execution of script %s" % fn)
+        if res and pyformex.GUI:
+            pyformex.message("Error during execution of script %s" % fn)
         
     return res
+        
+
+def setPrefs(res,save=False):
+    """Update the current settings (store) with the values in res.
+
+    res is a dictionary with configuration values.
+    The current settings will be update with the values in res.
+
+    If save is True, the changes will be stored to the user's
+    configuration file.
+    """
+    pyformex.debug("Accepted settings:",res)
+    for k in res:
+        pyformex.cfg[k] = res[k]
+        if save and pyformex.prefcfg[k] != pyformex.cfg[k]:
+            pyformex.prefcfg[k] = pyformex.cfg[k]
+
+    pyformex.debug("New settings:",pyformex.cfg)
+    if save:
+        pyformex.debug("New preferences:",pyformex.prefcfg)
 
 
 ########################## print information ################################
     
-def formatInfo(F):
-    """Return formatted information about a Formex."""
-    bb = F.bbox()
-    return """shape    = %s
-bbox[lo] = %s
-bbox[hi] = %s
-center   = %s
-maxprop  = %s
-""" % (F.shape(),bb[0],bb[1],F.center(),F.maxprop())
-    
 
 def printall():
     """Print all Formices in globals()"""
@@ -480,6 +519,7 @@ def printall():
 def printglobals():
     print(globals())
 
+
 def printglobalnames():
     a = globals().keys()
     a.sort()
@@ -487,9 +527,10 @@ def printglobalnames():
 
     
 def printconfig():
-    print("Reference Configuration: " + str(GD.refcfg))
-    print("User Configuration: " + str(GD.cfg))
-        
+    print("Reference Configuration: " + str(pyformex.refcfg))
+    print("Preference Configuration: " + str(pyformex.prefcfg))
+    print("User Configuration: " + str(pyformex.cfg))
+
 
 def printdetected():
     print(utils.reportDetected())
@@ -506,7 +547,7 @@ def chdir(fn):
     If fn is a directory name, the current directory is set to fn.
     If fn is a file name, the current directory is set to the directory
     holding fn.
-    In either case, the current dirctory is stored in GD.cfg['workdir']
+    In either case, the current directory is stored in the user's preferences
     for persistence between pyFormex invocations.
     
     If fn does not exist, nothing is done.
@@ -515,8 +556,8 @@ def chdir(fn):
         if not os.path.isdir(fn):
             fn = os.path.dirname(os.path.abspath(fn))
         os.chdir(fn)
-        GD.cfg['workdir'] = fn
-        GD.message("Your current workdir is %s" % os.getcwd())
+        setPrefs({'workdir':fn},save=True)
+        pyformex.message("Your current workdir is %s" % os.getcwd())
 
 
 def runtime():
@@ -526,17 +567,21 @@ def runtime():
 
 def startGui(args=[]):
     """Start the gui"""
-    if GD.GUI is None:
-        GD.debug("Starting the pyFormex GUI")
-        from gui import gui
-        gui.startGUI(args)
-        gui.runGUI()
+    if pyformex.GUI is None:
+        pyformex.debug("Starting the pyFormex GUI")
+        from gui import guimain
+        if guimain.startGUI(args) == 0:
+            guimain.runGUI()
 
 
 def isWritable(path):
-    """Check that the specified path is writeable."""
+    """Check that the specified path is writeable.
+
+    BEWARE: this only works if the path exists!
+    """
     return os.access(path,os.W_OK)
 
+
 def checkRevision(rev,comp='>='):
     """Check that we have the requested revision number.
 
@@ -549,17 +594,56 @@ def checkRevision(rev,comp='>='):
     Default is to allow the specified revision and all later ones.
     """
     try:
-        cur = int(utils.splitStartDigits(GD.__revision__.split()[1])[0])
+        cur = int(utils.splitStartDigits(pyformex.__revision__.split()[1])[0])
         if not eval("%s %s %s" % (cur,comp,rev)):
             raise RuntimeError
     except:
-        raise RuntimeError,"Your current pyFormex revision (%s) does not pass the test %s %s" % (GD.__revision__,comp,rev)
+        raise RuntimeError,"Your current pyFormex revision (%s) does not pass the test %s %s" % (pyformex.__revision__,comp,rev)
    
+################### read and write files #################################
 
+def writeGeomFile(filename,objects,sep=' ',mode='w'):
+    """Save geometric objects to a pyFormex Geometry File.
 
-@deprecation("Use chdir(__file__) instead")
-def workHere():
-    """Change the current working directory to the script's location."""
-    chdir(GD.PF.get('curfile',''))
+    A pyFormex Geometry File can store multiple geometrical objects in a
+    native format that can be efficiently read back into pyFormex.
+    The format is portable over different pyFormex versions and 
+    even to other software.
+
+    - `filename`: the name of the file to be written
+    - `objects`: a list or a dictionary. If it is a dictionary,
+      the objects will be saved with the key values as there names.
+      Objects that can not be exported to a Geometry File will be
+      silently ignored.
+    - `mode`: can be set to 'a' to append to an existing file.
+    - `sep`: the string used to separate data. If set to an empty
+      string, the data will be written in binary format and the resulting file
+      will be smaller but less portable.
+
+    Returns the number of objects written to the file.
+    """
+    f = geomfile.GeometryFile(filename,mode='w',sep=sep)
+    f.write(objects)
+    f.close()
+    return len(objects)
+    
+
+def readGeomFile(filename):
+    """Read a pyFormex Geometry File.
+
+    A pyFormex Geometry File can store multiple geometrical objects in a
+    native format that can be efficiently read back into pyFormex.
+    The format is portable over different pyFormex versions and 
+    even to other software.
+
+    - `filename`: the name of an exisiting pyFormex Geometry File.
+    
+    Returns a dictionary with the geometric objects read from the file.
+    If object names were stored in the file, they will be used as the keys.
+    Else, default names will be provided.
+    """
+    f = geomfile.GeometryFile(filename,mode='r')
+    return f.read()
+    
 
 #### End
