@@ -1,7 +1,7 @@
 #!/usr/bin/env python
-# $Id: simple.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: simple.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -102,12 +102,38 @@ def line(p1=[0.,0.,0.],p2=[1.,0.,0.],n=1):
     The line is split up in n segments.
     """
     return Formex([[p1,p2]]).divide(n)
+ 
+
+def rect(p1=[0.,0.,0.],p2=[1.,0.,0.],nx=1,ny=1):
+    """Return a Formex which is a the circumference of a rectangle.
+
+    p1 and p2 are two opposite corner points of the rectangle.
+    The edges of the rectangle are in planes parallel to the z-axis.
+    There will always be two opposite edges that are parallel with the x-axis.
+    The other two will only be parallel with the y-axis if both points
+    have the same z-value, but in any case they will be parallel with the
+    y-z plane.
+
+    The edges parallel with the x-axis are divide in nx parts, the other
+    ones in ny parts.
+    """
+    p1 = Coords(p1)
+    p2 = Coords(p2)
+    p12 = Coords([p2[0],p1[1],p1[2]])
+    p21 = Coords([p1[0],p2[1],p2[2]])
+    return Formex.concatenate([
+        line(p1,p12,nx),
+        line(p12,p2,ny),
+        line(p2,p21,nx),
+        line(p21,p1,ny)
+        ])
 
 
 def rectangle(nx,ny,b=None,h=None,bias=0.,diag=None):
-    """Return a Formex representing a rectangle of size(b,h) with (nx,ny) cells.
+    """Return a Formex representing a rectangluar surface.
+
+    The rectangle has a size(b,h) divivded into (nx,ny) cells.
 
-    This is a convenience function to create a rectangle with given size.
     The default b/h values are equal to nx/ny, resulting in a modular grid.
     The rectangle lies in the (x,y) plane, with one corner at [0,0,0].
     By default, the elements are quads. By setting diag='u','d' of 'x',
@@ -129,36 +155,50 @@ def rectangle(nx,ny,b=None,h=None,bias=0.,diag=None):
     return base.replic2(nx,ny,bias=bias).scale([sx,sy,0.])
    
 
-def circle(a1=2.,a2=0.,a3=360.):
+def circle(a1=2.,a2=0.,a3=360.,r=None,n=None,c=None):
     """A polygonal approximation to a circle or arc.
 
     All points generated by this function lie on a circle with unit radius at
     the origin in the x-y-plane.
 
-    a1 (the dash angle) is the angle enclosed between the start and
-    end points of each line segment.
-    a2 (the module angle) is the angle enclosed between the start
-    points of two subsequent line segments. 
-    If a2==0.0 is given, it will be taken equal to a1.
-    a3 (the arc angle) is the total angle enclosed between the first
-    point of the first segment and the end point of the last segment.
+    - `a1`: the angle enclosed between the start and end points of each line
+      segment (dash angle).
+    - `a2`: the angle enclosed between the start points of two subsequent line
+      segments (module angle). If ``a2==0.0``, `a2` will be taken equal to `a1`.
+    - `a3`: the total angle enclosed between the first point of the first
+      segment and the end point of the last segment (arc angle).
 
     All angles are given in degrees and are measured in the direction from
     x- to y-axis. The first point of the first segment is always on the x-axis.
 
     The default values produce a full circle (approximately).
     If $a3 < 360$, the result is an arc.
-    Large values of a1 and a2 result in polygones. Thus
-    circle(120.) is an equilateral triangle and circle(60.) is regular hexagone.
+    Large values of `a1` and `a2` result in polygones. Thus
+    `circle(120.)` is an equilateral triangle and `circle(60.)`
+    is regular hexagon.
 
     Remark that the default a2 == a1 produces a continuous line,
     while a2 > a1 results in a dashed line.
+
+    Three optional arguments can be added to scale and position the circle
+    in 3D space:
+
+    - `r`: the radius of the circle
+    - `n`: the normal on the plane of the circle
+    - `c`: the center of the circle
     """
     if a2 == 0.0:
         a2 = a1
-    n = int(round(a3/a2))
+    ns = int(round(a3/a2))
     a1 *= pi/180.
-    return Formex([[[1.,0.,0.],[cos(a1),sin(a1),0.]]]).rosette(n,a2,axis=2,point=[0.,0.,0.])
+    F = Formex([[[1.,0.,0.],[cos(a1),sin(a1),0.]]]).rosette(ns,a2,axis=2,point=[0.,0.,0.])
+    if r is not None:
+        F = F.scale(r)
+    if n is not None:
+        F = F.swapAxes(0,2).rotate(rotMatrix(n))
+    if c is not None:
+        F = F.trl(c)
+    return F
 
 
 def polygon(n):
@@ -309,5 +349,26 @@ def cylinder(D,L,nt,nl,D1=None,angle=360.,bias=0.,diag=None):
     return C.cylindrical(dir=[2,1,0])
 
 
+def cuboid(xmin=[0.,0.,0.],xmax=[1.,1.,1.]):
+    """Create a rectangular prism
+
+    Creates a rectangular prism with faces parallel to the global
+    axes through the points xmin and xmax.
+
+    Returns a single element Formex with eltype 'hex8'.
+    """
+    x0,y0,z0 = xmin
+    x1,y1,z1 = xmax
+    x = Coords([[
+        [x0,y0,z0],
+        [x1,y0,z0],
+        [x1,y1,z0],
+        [x0,y1,z0],
+        [x0,y0,z1],
+        [x1,y0,z1],
+        [x1,y1,z1],
+        [x0,y1,z1],
+        ]])
+    return Formex(x,eltype='hex8')
 
 # End
