@@ -1,6 +1,6 @@
-# $Id: utils.py 1252 2009-12-09 11:05:19Z bverheg $
+# $Id: utils.py 1497 2010-06-05 08:50:48Z bverheg $
 ##
-##  This file is part of pyFormex 0.8.1 Release Wed Dec  9 11:27:53 2009
+##  This file is part of pyFormex 0.8.2 Release Sat Jun  5 10:49:53 2010
 ##  pyFormex is a tool for generating, manipulating and transforming 3D
 ##  geometrical models by sequences of mathematical operations.
 ##  Homepage: http://pyformex.org   (http://pyformex.berlios.de)
@@ -23,36 +23,34 @@
 ##
 """A collection of miscellaneous utility functions."""
 
-import pyformex
+import pyformex as pf
 
 import os,commands,re,sys
 from config import formatDict
 from distutils.version import LooseVersion as SaneVersion
 
 
-def procInfo(title):
-    print(title)
-    print('module name: %s' %  __name__)
-    print('parent process: %s' % os.getppid())
-    print('process id: %s' % os.getpid())
-
-
 # versions of detected modules/external commands
-the_version = {'pyformex':pyformex.__version__}
+the_version = {
+    'pyformex':pf.__version__,
+    'python':sys.version.split()[0],
+    }
 the_external = {}
 
+known_modules = [ 'python','numpy','pyopengl','pyqt4','pyqt4gl','calpy',
+                  'gnuplot','gl2ps' ]
 
-def _congratulations(name,version,typ='module',fatal=False,quiet=False):
-    """Report a detected module/program."""
-    if version and not quiet:
-        pyformex.message("Congratulations! You have %s (%s)" % (name,version))
-    if not version:
-        if not quiet or fatal:
-            pyformex.message("ALAS! I could not find %s '%s' on your system" % (typ,name))
-        if fatal:
-            pyformex.message("Sorry, I'm out of here....")
-            sys.exit()
-
+known_externals = {
+    'Python': ('python --version','Python (\\S+)'),
+    'ImageMagick': ('import -version','Version: ImageMagick (\S+)'),
+    'admesh': ('admesh --version', 'ADMesh - version (\S+)'),
+    'calpy': ('calpy --version','Calpy (\S+)'), 
+    'tetgen': ('tetgen -h |fgrep Version','Version (\S+)'),
+    'units': ('units --version','GNU Units version (\S+)'),
+    'ffmpeg': ('ffmpeg -version','FFmpeg version (\\S+)'),
+    'gts': ('gtsset -h','Usage(:) set'),
+    'calix': ('calix --version','CALIX-(\S+)'),
+    }
 
 def checkVersion(name,version,external=False):
     """Checks a version of a program/module.
@@ -64,11 +62,14 @@ def checkVersion(name,version,external=False):
     Return value is -1, 0 or 1, depending on a version found that is
     <, == or > than the requested values.
     This should normally understand version numbers in the format 2.10.1
+    Returns -2 if no version found.
     """
     if external:
-        ver = the_external.get(name,'0')
+        ver = hasExternal(name)
     else:
-        ver = the_version.get(name,'0')
+        ver = hasModule(name)
+    if not ver:
+        return -2
     if SaneVersion(ver) > SaneVersion(version):
         return 1
     elif SaneVersion(ver) == SaneVersion(version):
@@ -76,8 +77,38 @@ def checkVersion(name,version,external=False):
     else:
         return -1
 
-known_modules = [ 'numpy','pyopengl','pyqt4','pyqt4gl','calpy',
-                  'gnuplot','gl2ps' ]
+    
+def hasModule(name,check=False):
+    """Test if we have the named module available.
+
+    Returns a nonzero (version) string if the module is available,
+    or an empty string if it is not.
+
+    By default, the module is only checked on the first call. 
+    The result is remembered in the the_version dict.
+    The optional argument check==True forces a new detection.
+    """
+    if the_version.has_key(name) and not check:
+        return the_version[name]
+    else:
+        return checkModule(name)
+
+
+def hasExternal(name):
+    """Test if we have the external command 'name' available.
+
+    Returns a nonzero string if the command is available,
+    or an empty string if it is not.
+
+    The external command is only checked on the first call.
+    The result is remembered in the the_external dict.
+    """
+    if the_external.has_key(name):
+        return the_external[name]
+    else:
+        return checkExternal(name)
+
+    
 
 def checkModule(name=None):
     """Check if the named Python module is available, and record its version.
@@ -97,15 +128,15 @@ def checkModule(name=None):
             import numpy
             version =  numpy.__version__
         elif name == 'pyopengl':
-            fatal = pyformex.options.gui
+            fatal = pf.options.gui
             import OpenGL
             version =  OpenGL.__version__
         elif name == 'pyqt4':
-            fatal = pyformex.options.gui
+            fatal = pf.options.gui
             import PyQt4.QtCore
             version = PyQt4.QtCore.QT_VERSION_STR
         elif name == 'pyqt4gl':
-            fatal = pyformex.options.gui
+            fatal = pf.options.gui
             import PyQt4.QtOpenGL
             import PyQt4.QtCore
             version = PyQt4.QtCore.QT_VERSION_STR
@@ -126,34 +157,6 @@ def checkModule(name=None):
     the_version[name] = version
     return version
 
-    
-def hasModule(name,check=False):
-    """Test if we have the named module available.
-
-    Returns a nonzero (version) string if the module is available,
-    or an empty string if it is not.
-
-    By default, the module is only checked on the first call. 
-    The result is remembered in the the_version dict.
-    The optional argument check==True forces a new detection.
-    """
-    if the_version.has_key(name) and not check:
-        return the_version[name]
-    else:
-        return checkModule(name)
-
-            
-# tetgen -v no longer works in 1.4.2: use -h !!
-known_externals = {
-    'Python': ('python --version','Python (\\S+)'),
-    'ImageMagick': ('import -version','Version: ImageMagick (\S+)'),
-    'admesh': ('admesh --version', 'ADMesh - version (\S+)'),
-    'calpy': ('calpy --version','Calpy (\S+)'), 
-    'tetgen': ('tetgen -h |fgrep Version','Version (\S+)'),
-    'units': ('units --version','GNU Units version (\S+)'),
-    'ffmpeg': ('ffmpeg -version','FFmpeg version (\\S+)'),
-    'gts': ('gtsset -h','Usage(:) set'),
-    }
 
 
 def checkExternal(name=None,command=None,answer=None):
@@ -192,28 +195,13 @@ def checkExternal(name=None,command=None,answer=None):
         version = m.group(1)
     else:
         version = ''
-    _congratulations(name,version,'program')
+    _congratulations(name,version,'program',quiet=True)
     the_external[name] = version
     return version
 
 
-def hasExternal(name):
-    """Test if we have the external command 'name' available.
-
-    Returns a nonzero string if the command is available,
-    or an empty string if it is not.
-
-    The external command is only checked on the first call.
-    The result is remembered in the the_external dict.
-    """
-    if the_external.has_key(name):
-        return the_external[name]
-    else:
-        return checkExternal(name)
-
-
 def reportDetected():
-    s = "%s (%s)\n\n" % (pyformex.Version,pyformex.__revision__)
+    s = "%s (%s)\n\n" % (pf.Version,pf.__revision__)
     s += "Python version: %s\n" % sys.version
     s += "Operating system: %s\n\n" % sys.platform
     s += "Detected Python Modules:\n"
@@ -229,8 +217,35 @@ def reportDetected():
     return s
 
 
-def prefix(prefix,files):
-    """Prepend a prefix to a list of filename."""
+    
+def procInfo(title):
+    print(title)
+    print('module name: %s' %  __name__)
+    print('parent process: %s' % os.getppid())
+    print('process id: %s' % os.getpid())
+
+def strNorm(s):
+    """Normalize a string.
+
+    Text normalization removes all '&' characters and converts it to lower case.
+    """
+    return str(s).replace('&','').lower()
+
+
+def _congratulations(name,version,typ='module',fatal=False,quiet=True):
+    """Report a detected module/program."""
+    if version and not quiet:
+        pf.message("Congratulations! You have %s (%s)" % (name,version))
+    if not version:
+        if not quiet or fatal:
+            pf.message("ALAS! I could not find %s '%s' on your system" % (typ,name))
+        if fatal:
+            pf.message("Sorry, I'm out of here....")
+            sys.exit()
+
+
+def prefixFiles(prefix,files):
+    """Prepend a prefix to a list of filenames."""
     return [ os.path.join(prefix,f) for f in files ]
 
 
@@ -295,7 +310,7 @@ def listTree(path,listdirs=True,topdown=True,sorted=False,excludedirs=[],exclude
             files = [ f for f in files if matchNone(excludefiles,f) ]
         if includefiles:
             files = [ f for f in files if matchAny(includefiles,f) ]
-        filelist.extend(prefix(root,files))
+        filelist.extend(prefixFiles(root,files))
         if listdirs and not topdown:
             filelist.append(root)
     return filelist
@@ -355,7 +370,6 @@ def dos2unix(infile,outfile=None):
         cmd = "sed -i 's|\\r||' %s" % infile
     else:
         cmd = "sed -i 's|\\r||' %s > %s" % (infile,outfile)
-    print(cmd)
     return runCommand(cmd)
 
 def unix2dos(infile,outfile=None):
@@ -390,6 +404,7 @@ file_description = {
     'postproc': 'Postproc scripts (*_post.py *.post)',
     'pyformex': 'pyFormex scripts (*.py *.pye)',
     'pyf': 'pyFormex projects (*.pyf)',
+    'pgf': 'pyFormex geometry files (*.pgf *.formex)',
 }
 
 def fileDescription(ftype):
@@ -410,10 +425,10 @@ def findIcon(name):
 
     If no icon file is found, returns the question mark icon.
     """
-    fname = os.path.join(pyformex.cfg['icondir'],name) + pyformex.cfg['gui/icontype']
+    fname = os.path.join(pf.cfg['icondir'],name) + pf.cfg['gui/icontype']
     if os.path.exists(fname):
         return fname
-    return os.path.join(pyformex.cfg['icondir'],'question') + pyformex.cfg['gui/icontype']
+    return os.path.join(pf.cfg['icondir'],'question') + pf.cfg['gui/icontype']
                                                                
 
 def projectName(fn):
@@ -437,6 +452,27 @@ def mtime(fn):
     return os.stat(fn).st_mtime
 
 
+def timeEval(s,glob=None):
+    """Return the time needed for evaluating a string.
+
+    s is a string with a valid Python instructions.
+    The string is evaluated using Python's eval() and the difference
+    in seconds between the current time before and after the evaluation
+    is printed. The result of the evaluation is returned.
+
+    This is a simple method to measure the time spent in some operation.
+    It should not be used for microlevel instructions though, because
+    the overhead of the time calls. Use Python's timeit module to measure
+    microlevel execution time.
+    """
+    import time
+    start = time.time()
+    res = eval(s,glob)
+    stop = time.time()
+    pf.message("Timed evaluation: %s seconds" % (stop-start))
+    return res
+
+
 def countLines(fn):
     """Return the number of lines in a text file."""
     sta,out = runCommand("wc %s" % fn)
@@ -449,14 +485,14 @@ def countLines(fn):
 def runCommand(cmd,RaiseError=True,quiet=False):
     """Run a command and raise error if exited with error."""
     if not quiet:
-        pyformex.message("Running command: %s" % cmd)
+        pf.message("Running command: %s" % cmd)
 #    if GD.GUI:
 #        GD.GUI.setBusy(True)
     sta,out = commands.getstatusoutput(cmd)
 #    if GD.GUI:
 #        GD.GUI.setBusy(False)
     if sta != 0:
-        pyformex.message(out)
+        pf.message(out)
         if RaiseError:
             raise RuntimeError, "Error while executing command:\n  %s" % cmd
     return sta,out
@@ -466,10 +502,20 @@ def spawn(cmd):
     """Spawn a child process."""
     cmd = cmd.split()
     pid = os.spawnvp(os.P_NOWAIT,cmd[0],cmd)
-    pyformex.debug("Spawned child process %s for command '%s'" % (pid,cmd))
+    pf.debug("Spawned child process %s for command '%s'" % (pid,cmd))
     return pid
 
 
+def killProcesses(pids,signal):
+    """Send the specified signal to the processes in list"""
+    for pid in pids:
+        try:
+            os.kill(pid,signal)
+        except:
+            pf.debug("Error in killing of process '%s'" % pid)
+            
+
+
 def changeExt(fn,ext):
     """Change the extension of a file name.
 
@@ -505,6 +551,9 @@ def isPyFormex(filename):
 
        #!/usr/bin/env pyformex
     """
+    if filename.endswith(".pye"):
+        return True
+
     ok = filename.endswith(".py")
     if ok:
         try:
@@ -517,28 +566,6 @@ def isPyFormex(filename):
     
 
 
-# THIS MAY BE FASTER THAN olist.collectOnLength, BUT IT IS DEPENDENT ON NUMPY
-
-## def collectOnLength(items):
-##     """Collect items with same length.
-
-##     a is a list of items of any type for which the function len()
-##     returns an integer value.
-##     The items are sorted in a number of bins, each containing the
-##     items with the same length.
-##     The return value is a tuple of:
-##     - a list of bins with the sorted items,
-##     - a list of indices of these items in the input list,
-##     - a list of lengths of the bins,
-##     - a list of the item length in each bin.
-##     """
-##     np = array([ len(e) for e in items ])
-##     itemlen = unique1d(np)
-##     itemnrs = [ where(np==p)[0] for p in itemlen ]
-##     itemgrps = [ olist.select(items,i) for i in itemnrs ]
-##     itemcnt = [ len(i) for i in itemnrs ]
-##     return itemgrps,itemnrs,itemcnt,itemlen
-
 
 class NameSequence(object):
     """A class for autogenerating sequences of names.
@@ -615,20 +642,37 @@ def splitEndDigits(s):
 
 
 def splitStartDigits(s):
-    """Split a string in any prefix and a numerical end sequence.
+    """Split a string in a numerical sequence and any suffix.
 
-    A string like 'abc-0123' will be split in 'abc-' and '0123'.
+    A string like '0123-abc' will be split in '0123' and '-abc'.
     Any of both can be empty.
     """
     return digits_string.match(s).groups()
 
 
-def subDict(dic,keystart):
-    """Return a dict with the items from dic whose key starts with keystart.
+def prefixDict(d,prefix=''):
+    """Prefix all the keys of a dict with the given prefix.
 
-    This also strips the keystart form the keys.
+    - `d`: a dict where all the keys are strings.
+    - `prefix`: a string
+
+    The return value is a dict with all the items of d, but where the
+    keys have been prefixed with the given string.
     """
-    return dict([ (k.replace(keystart,'',1),v) for k,v in dic.items() if k.startswith(keystart)])
+    return dict([ (prefix+k,v) for k,v in d.items() ])
+
+
+def subDict(d,prefix=''):
+    """Return a dict with the items whose key starts with prefix.
+
+    - `d`: a dict where all the keys are strings.
+    - `prefix`: a string
+    
+    The return value is a dict with all the items from d whose key starts
+    with prefix. The keys in the returned dict will have the prefix
+    stripped off.
+    """
+    return dict([ (k.replace(prefix,'',1),v) for k,v in d.items() if k.startswith(prefix)])
 
     
 def stuur(x,xval,yval,exp=2.5):
